/*                                                                
**  Copyright (C) 2002-2004,2006-2008,2015,2018-2020,2023  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   dmnutils.h                                                  */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#ifndef DMNUTILS_H
#define DMNUTILS_H

/* ----------------------------------------------------- */
/* Supported DM Data Types                               */
/*   always picked up by dmerror.h                       */
/* ----------------------------------------------------- */
typedef enum { DT_VOID, DT_INT2, DT_PTR, DT_INT4, DT_REAL4, DT_REAL8,
	       DT_CHAR, DT_UINT1, DT_LOGICAL, DT_UNK, 
               DT_UINT2, DT_UINT4, DT_BIT, DT_INT8, DT_UINT8 } DataType;


#ifdef DMLIB_INTERNAL
/* ----------------------------------------------------------- */
/*  The following defines are only needed for internal code    */
/*  set DMLIB_INTERNAL to include this content as needed.      */
/*  ---------------------------------------------------------- */

#define JCMLIB "JCMLIB UTILS"
#define JCMLIBVER "CV2.0"

/* V0.1 1985 Nov    Initial version */
/* V1.1 1997 Feb 19 Made consistent use of long int */
/* V1.2 1997 Mar  9 Moved all prototypes into jcmproto.h */
/* V1.3 1997 Mar 10 Fixed SIO and TFR bugs, unitlib bug */
/* V1.4 1997 May  2 Added new allocate routines, vsub_d */
/* V1.5 1997 Nov 27 Cleaned up, added const declarations, moved stuff back to jcmlib.h */
/* V2.0 2001 Jun  2 Rewrite */
/* V2.1 2002 Jul  1 Revision for DM use */
/* V2.2 2018 Sep 20 Isolate methods NOT used in DM library for possible obsolescense. */

#define UT_ERR_START 1000

#include "dmconfig.h"
#include "dmlimits.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

/* boolean values */
#undef TRUE
#undef FALSE
#define TRUE  1
#define FALSE 0

#define UT_TRUE  1
#define UT_FALSE 0
#define UT_NULL  2

/* 20141217 - Support tri-state boolean values: true, false, null */
#define UT_TEXT_TRUE  "TRUE"
#define UT_TEXT_FALSE "FALSE"
#define UT_TEXT_NULL  "NULL"
#define UT_TEXT_YES   "YES"
#define UT_TEXT_NO    "NO"
#define UT_BOOL_TRUE_CHARS  "TtYy1"
#define UT_BOOL_FALSE_CHARS "FfNn0"
#define UT_BOOL_NULL_CHARS  "Uu2"
#define UT_NULL_CHAR '\0'

/* Standard sizes for common entities */
#define CARD_SIZE 80    // eg: FITS header card
#define TEXT_SIZE 1024  // generic string record length
#define WORD_SIZE   31  // single word length
#define FN_SIZE   512   // unused in DM proper

/* ---------------------------------------------------------------------- */
/* Special types */
/*  In general I have avoided the use of special types, which tend to
 *  make a general purpose library less reusable.
 *  For example, 3-vector arguments are just double* and you must read the
 *  documentation to know that you need a double[3].

 *  The 'logical' type is a typedef for int and is used for boolean values.
 *
 *  There are also some special string types:
 *     TextWord - for short strings,
 *     TextCard - for 80 byte ASCII cards,
 *     TextBuf  - for long strings,
 *     Filename - for pathnames.
 *  These are never used in the external prototypes, but you 
 *  may want to use FSIZE to check if your pathnames will exceed
 *  JCMLIB's capacity.
 *
 *  Note that all string tests in the library ignore trailing blanks;
 *  and a string of blanks is considered the same as a null string.
 *  This improves compatibility with Fortran programs.
 */

typedef double GenericType;   /* 8-bytes.. enough space for any type value    */
typedef int    logical;       /* C standard specifies result of logical expr is int */

/* Type shortcuts */
#define uchar   unsigned char /* Shortcut name for unsigned char */
#define dmcos   utm_cos       /* cosin method for DM             */
#define dmsin   utm_sin       /* sin method for DM               */
#define floor_d utm_floor_d   /* floor from double               */

typedef char TextCard[CARD_SIZE+1];
typedef char TextWord[WORD_SIZE+1];  // unused in DM proper
typedef char TextBuf[TEXT_SIZE+1];
typedef char Filename[FN_SIZE+1];    // unused in DM proper

/* Objects */
typedef struct ErrorClasses_s*  ErrorClasses;
typedef struct ErrorData_s*     Error;


/* ---------------------------------------------------------------------- */
/* FIO - File IO package */
#define FIO_NOPERM   -1
#define FIO_NOFILE    0
#define FIO_READONLY  1
#define FIO_READWRITE 2

/* FIO objects */
typedef struct FioFileObject_s *FioFile;

/* FIO - package prototypes */
long    fio_access( const char* name ); 
void    fio_dashline( long n ); 
logical fio_file_close( FioFile file ); 
FioFile fio_open_ar( const char* filename ); 
logical fio_read_line( FioFile file, char* line, long lmax );  
char*   fio_tmpname( void );
void    fio_tmsg( char* buf ); 

/* ---------------------------------------------------------------------- */
/* UT - No idea what that stands for.. probably 'utilities' */
#define UT_SZ_B 1
#define UT_SZ_R UT_SZ_FLOAT
#define UT_SZ_I UT_SZ_LONG
#define UT_SZ_K UT_SZ_LONGLONG
#define UT_SZ_C UT_SZ_CHAR
#define UT_SZ_S UT_SZ_SHORT
#define UT_SZ_D UT_SZ_DOUBLE
#define UT_SZ_Q UT_SZ_INT

#define UT_CIRCLE 360.0
#define UT_DEG_RAD  ( 2 * M_PI / UT_CIRCLE )

/* UT package prototypes */ 
void ut_init_jcmlib( void ); 
void ut_dbg_set( const long level ); 
void ut_error_set( Error* old_error, char* text, long code, long auxcode ); 
long ut_error_code( Error error ); 
void ut_error_message( Error* errorp, char* buf, long maxlen );
void ut_error_disp( Error* error, ErrorClasses error_context );
void ut_error_disp_select_msgs( long lErrorCode );  
void ut_error_clear( Error* error );  
long ut_error_get_num_occur( long code, const char* text);
long ut_error_class_register( ErrorClasses list, char* name ); 
Error* ut_error_state(void); 
float ut_nan_r( void );  
double ut_nan_d( void );  
logical ut_qnull_i( const long i ); 
logical ut_qnull_s( const short i ); 
logical ut_qnan_r( const float x ); 
logical ut_qnan_d( const double x ); 
logical ut_qnull_r( const float x ); 
logical ut_qnull_d( const double x ); 
logical ut_qinf_i( const long i );  
logical ut_qinf_s( const short i );  
logical ut_qinf_r( const float x );  
logical ut_qinf_d( const double x ); 
void*   ut_realloc( void* data, long const old, long const new, const long size );
long ut_sys_align( long total, long size, long old_size );

long   utm_floor_d( const double x );  
double utm_cos( const double x );
double utm_sin( const double x );


/* --------------------------------------------------------------------------- */
/* CS package:                                                                 */
/*   character string manipulation                                             */
/* --------------------------------------------------------------------------- */
/* cs package prototypes */
char*   cs_advptr( char** buf ); 
logical cs_alpha_char( const char c ); 
void    cs_append( char** pptr, char* sep, char* buf );
void    cs_blank( char* line, long maxlen ); 
void    cs_compose_paren_list( char** items, const long n, char* buf, const long maxlen );
void    cs_copy_siz( const char* source, char* dest, long siz );	 
void    cs_copy( const char* source, char* dest ); 
logical cs_digit_char( const char c ); 
char*   cs_dup( const char* buf ); 
char**  cs_dup_array( char** data, long n );
char*   cs_dup_cat( char* a, char* b, char* sep );
char*   cs_dup_siz( const char* buf, const long siz ); 
long    cs_ends( const char* buf ); 
logical cs_eq( const char* buf1, const char* buf2 );			 
logical cs_eq_siz( const char* buf1, const char* buf2, const long siz ); 
logical cs_eq_ss( const char* buf, const long pos1, const long pos2, const char* comp ); 
char*   cs_get_ss( const char* source, char* dest, const long pos1, const long pos2 );	 
logical cs_gt( const char* x, const char* y ); 
long    cs_index( const char* buf, const char* sub ); 
long    cs_index_char( const char* buf, const char c ); 
logical cs_is_blank( const char* buf );					 
void    cs_lower( char* buf );	 
logical cs_lt( const char* x, const char* y ); 
logical cs_numeric_char( const char c ); 
char**  cs_parse_comma_list( const char* buffer, long* np );
char**  cs_parse_paren_list( const char* buf, long* n ); 
char*   cs_parse_paren_token( char** pptr, char lsep, char rsep, char* result, long maxlen );
char*   cs_parse_paren_token_validate( char** pptr, char lsep, char rsep, char* result, long maxlen, logical *status );
char*   cs_parse_term_token( char** pptr, char rsep );
void    cs_remove_quotes_and_trail( const char* val, char* dest, const long maxlen );
logical cs_split_string( const char* buf, const char sep, char* buf1, char* buf2 );  
logical cs_ss( const char* buf, const char* sub ); 
logical cs_ss_char( const char* buf, const char sub ); 
void    cs_strip_digits( const char* name, char* pattern, const long maxlen );
void    cs_trim( char* buf );
logical cs_uceq( const char* buf1, const char* buf2 ); 
logical cs_uceq_siz( const char* buf1, const char* buf2, const long siz ); 
void    cs_upper( char* buf );	 
char*   cs_writef_clean_d( const double val, const char* fmt, const logical clean, const long sig, char* buf, const long siz );

/* ---------------------------------------------------------------------- */
/* AR - Array stuff */

/* AR package prototypes */
char**  ar_alloc_c( const long size, const long n ); 
void    ar_copy_d( const double* a, double* b, const long siz );  
void    ar_csort_d( const double* x, const long n, long* jsort ); 
void    ar_free_c( char** word ); 
void    ar_free_cp( char** word, const long n ); 
void*   ar_gen_alloc( const DataType type, const long n );
logical ar_gen_get_cast_elt( const void* array, const long offset, const long n, const DataType array_type, const DataType type, void* value );
logical ar_gen_get_elt( const void* array, const long offset, const DataType type, void* value );
logical ar_gen_set_elt( void* array, const long offset, const DataType type, void* value );
void*   ar_gen_ptr( const void* data, const DataType type, const long size, const long offset );
long    ar_match_i( const long value,  const long* list,  const long n );  
long    ar_match_c( const char* buf, char** array, const long n ); 
long    ar_match_sorted_c( const char* buf, char** array, const long n );
long    ar_search_c( const char* buf, char** array, const long n );
void    ar_sort_d( const double* x, const long n, long* jsort ); 
void    ar_sort_c( char** x, const long n, long* jsort );  

/* --------------------------------------------------------------------------- */
/* AliasList:                                                                  */
/*   No idea what this does                                                    */
/* --------------------------------------------------------------------------- */
typedef struct AliasList_s 
{
 char** name;
 char*** cpts;
 long* dim;
 long n;
 long nmax;
} *AliasList;

/* AliasList package prototypes */
AliasList ar_alias_alloc( const long n );
void ar_alias_free( AliasList array );
void ar_alias_get_cpt( AliasList array, long offset, long cpt,  char* name, long maxlen );
char** ar_alias_get_cpt_list( AliasList array, long offset );
long ar_alias_get_dim( AliasList array, long offset );
void ar_alias_get_name( AliasList array, long offset, char* name, long maxlen );
void ar_alias_set_cpts( AliasList array, long entry, char** cpts, long dim );
long ar_alias_set_name( AliasList array, char* name );
long ar_alias_search( const AliasList array, const char* cptname );

/* --------------------------------------------------------------------------- */
/* MixArray:                                                                   */
/*   Appears to facilitate an 'array' where each element can vary in type      */
/* --------------------------------------------------------------------------- */
typedef struct MixArray_s 
{
 void* data;
 DataType type;
 DataType* types;
 long size;
 long n;
 long nmax;
} *MixArray;

/* MixArray package prototypes */
MixArray ar_mix_alloc_type( const DataType type, const long size, const long n );
void     ar_mix_cast_array( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals );
void     ar_mix_copy_value( const void* ptr, const DataType type, void* value, logical copy, long size );
int      ar_mix_eq( const DataType type, void* value1, void* value2 );
void     ar_mix_free( MixArray array );
int      ar_mix_get_elt( const MixArray array, const long offset, DataType* type, void* value );
long     ar_mix_intersect( void* mins1, void* maxes1, long n1, void* mins2, void* maxes2, long n2, DataType type, void** minsp, void** maxesp, long* np );
logical  ar_mix_is_max( const DataType type, void* value );
logical  ar_mix_is_min( const DataType type, void* value );
logical  ar_mix_is_numeric( const DataType type );
void     ar_mix_numeric_cast( const void* in, const DataType intype, const DataType outtype, void* out );
void     ar_mix_numeric_cast_array( const void* in, const DataType intype, const DataType outtype, void* out, const long nvals );
void     ar_mix_numeric_cast_elt( const void* in, const DataType intype, const DataType outtype, void* out, long inoffset, long outoffset );
void     ar_mix_print_generic( void* ptr, DataType type, char* buf, long size );
void*    ar_mix_ptr( const MixArray array, const long offset );
void     ar_mix_type_min( const DataType type, void* value );
void     ar_mix_type_max( const DataType type, void *value );
void     ar_mix_type_name( const DataType type, char* buf, long maxlen );
long     ar_mix_type_size( const DataType type );

/* --------------------------------------------------------------------------- */
/* Misc:                                                                       */
/* Pixel coordinate functions in pixel.c */
long pixelno_from_pixel( long* Pixel, long* axes, long naxes );
void pixel_from_pixelno( long PixelNo, long* axes, long naxes, long* Pixel );


#endif  /* close DMLIB_INTERNAL  */

#endif  /* close DMNUTILS_H */
