/*                                                                
**  Copyright (C) 1998-2003,2006-2008,2010-2011,2015,2021  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* H******************************************************************
 * 
 * FILE NAME: %M%
 *
 * DEVELOPMENT: libdev
 * 
 * DESCRIPTION: header file for ds library tools
 * 
 * LAST MODIFIED: %G%
H********************************************************************/
#ifndef DSLIB_H
#define DSLIB_H

/* - - - - - - - - - - -  INCLUDE FILES - - - - - - - - - - - - - - */

#include <bool.h>      
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <math.h>
#include "parameter.h"
#ifndef LIBERR_H
#include "liberr.h"
#define LIBERR_H
#endif

#ifndef ASCDM_H
#include "ascdm.h"       /* data model */
#endif

#ifndef HDRLIB2_H
#include "hdrlib2.h"
#endif

#include <caldb4.h>


/* - - - - - - - - - - - - - - DEFINES  - - - - - - - - - - - - - - */

/* CONSTANTS */
/* for ds_event_open() */
/* for ds_event_create() also! */
#define EVENTBLOCKNAME "EVENTS"
#define SIZE_COMMAND   1024

/* verbose levels */
#define VERBOSE_0	0
#define VERBOSE_1	1
#define VERBOSE_2	2
#define VERBOSE_3	3
#define VERBOSE_4	4
#define VERBOSE_5	5

/* MISCELLANEOUS */
#define clgetr      clgetf /* I don't know why I have to do this */   

#ifndef MAX
#define MAX(a,b) ((a)<(b) ? (b) : (a))
#endif
#ifndef MIN
#define MIN(a,b) ((a)>(b) ? (b) : (a))
#endif

#ifndef SQUARE
#define SQUARE(x) ((x)*(x))
#endif

/* - - - - - - - - - - - GLOBAL VARIABLES - - - - - - - - - - - - - */


/* - - - - - - - - - -  FUNCTION PROTOTYPES - - - - - - - - - - - - */
/* MEDIAN routine related functions */
extern float      ds_median_floats(float *, long);
extern int        ds_median_sortLo2Hi_floats(const void *, const void *);
extern double     ds_median_doubles(double *, long);
extern int        ds_median_sortLo2Hi_doubles(const void *, const void *);

/* INTERPOLATION/EXTRAPOLAION related functions */
extern double     ds_interpolate_doubles(double, double, 
                                         double, double, double); 
extern float      ds_interpolate_floats(float, float, 
                                         float, float, float); 

/* ds_event file functions */

/* open read only */
extern short ds_open_event_file(char *event_file_name_a,
				dmDataset **event_file_ptr_p,
				dmBlock **event_block_ptr_p);

/* open with write permission */
extern short ds_open_event_file_update(char *event_file_name_a,
				       dmDataset **event_file_ptr_p,
				       dmBlock **event_block_ptr_p);

extern dmBlock* ds_create_event_file(char *outfile);


/* ds_strcmp_cis file functions */

extern int ds_strcmp_cis( const char *string1, const char *string2 );
extern int ds_strncmp_cis(const char *string1, const char *string2, short len );
extern int ds_strcmp_cis( const char *string1, const char *string2 );
extern int ds_strncmp_cis( const char *string1, const char *string2, short len );

/* ds_autoname functions */
extern int        ds_autoname(char *, char *, char * ext, int);

/* function to parse IRAF eventdef's for data model */
extern short ds_parse_eventdef(
             char       *eventdef,    /* I - string containing eventdef     */
             char       ***evt_names, /* O - ptr to array of column names   */
             dmDataType **evt_types,  /* O - ptr to array of column types   */
             short      *count);      /* O - number of elements in eventdef */

/* ds_string_conversion functions */ 
extern void  string_to_lowercase(char* phrase, short maxlen) ;
extern void  string_to_uppercase(char* phrase, short maxlen) ;

/* ds_get_date.c */
extern void  ds_get_date(char *out_date, int verbose) ;


/* custom error printing */
extern char ds_dm_error_msg[dmMAXSTR];
extern void ds_custom_error(const char *);
extern void ds_custom_dm_error(void);
extern void ds_get_dm_error(void);
extern void ds_custom_error_cleanup(void);

/* off-axis angle computations */
extern void ds_off_axis_pixels(long, long, long, long, long *, long *);

/* --------------------------------------------------------------------- */
/* constants -see ds_constants.c for details*/
/* --------------------------------------------------------------------- */
extern const double ASCDS_C;
extern const double ASCDS_C_CMS;
extern const double ASCDS_H_J;
extern const double ASCDS_H_EV;
extern const double ASCDS_H_ERG;
extern const double ASCDS_K_J;
extern const double ASCDS_K_EV;
extern const double ASCDS_K_ERG;
extern const double PHOTON_ENERGY_IN_eV_A;
extern const double ASCDS_PI;
extern const double ASCDS_BASE_E;
extern const double DEG2ASEC;
extern const double DEG2RAD;
extern const double RAD2DEG;
extern const double FWHM2SIGMA;
extern const double SIGMA2FWHM;
extern const double ANG2EV;
extern const double ANG2KEV;
extern const double EV2ANG;
extern const double KEV2ANG;
extern const double ANG2HZ;
extern const double HZ2ANG;
/* --------------------------------------------------------------------- */
/* macros to convert out of SI units */
/* --------------------------------------------------------------------- */
#define  M_TO_KM(xx)  (xx /1e3)	/* convert meters to km */
#define  KM_TO_M(xx)  (xx * 1e3) /* convert km to meters */
#define  M_TO_CM(xx)  (xx * 1e2) /* convert meters to cm */
#define  CM_TO_M(xx)  (xx / 1e2) /* convert cm to meters */
#define  EV_TO_KEV(xx) (xx / 1e3) /* convert eV to keV */
#define  KEV_TO_EV(xx) (xx * 1e3) /* convert keV to eV */
#define  ERG_TO_KEV(xx) (xx *6.24e8) /* convert erg to keV */
#define  KEV_TO_ERG(xx) (xx *1.6E-9) /* convert Kev to erg */
#define  ANG_TO_NM(xx)  (xx * 0.1) /* convert angstroms to nm */
#define  NM_TO_ANG(xx)  (xx * 10.) /* convert nm to angstroms */
/* --------------------------------------------------------------------- */
/* sizes -roughly follow iraf/FITS conventions  */
/* --------------------------------------------------------------------- */
#define DS_SZ_FNAME      1024             /* max chars in a file name*/
#define DS_SZ_GENSTR     1024             /* max chars size for most strings*/
#define DS_SZ_PATHNAME   4096             /* OS dependent file names*/
#define DS_SZ_LINE       1023            /* max chars in a line*/
#define DS_SZ_COMMAND    2047            /* max size command block*/
#define DS_SZ_COLNAME    67              /* max char in column name*/
#define DS_SZ_KEYWORD    511	         /* max char in keyword */
#define DS_SZ_EVENTDEF   511	         /* max event definition size*/
#define DS_SZ_COLUNIT    19	         /* max column unitname */
#define YES              1		 /* integer equivlalent to yes, from iraf.h */
#define NO               0		 /* integer equivlalent to no, from iraf.h */
#define OK		 0		 /* integer equivlalent to OK, from iraf.h */

/************************************************************************
 * Macros for header keyword.  The numbers are temporary, once we
 * know for sure what the iraf/FITS standard is, we will make the
 * necessary change. 
 ************************************************************************/
#define DS_SZ_KEY_VALUE   511              /* length of string that is
					    * used to store the string
					    * value of a header keyword */
#define DS_SZ_KEY_UNIT     80              /* length of string that is 
					    * used to store the unit of
					    * a header keyword */
#define DS_SZ_KEY_COMMENT  80              /* length of string that is
					    * used to store the comment
					    * of a keyword */

/* ds_roman_num_converter file functions */

extern long ds_roman2long(const char *str);

extern char* ds_long2roman(long val, char *buf, size_t buflen);

char *ds_set_kernel( dmDataset *inDataset, char *kernel );


/* ds_date2relsec  functions */

#define AXAF_FLIGHT_EPOCH "1998-01-01T00:00:00"
#define AXAF_CALIB_EPOCH "1993-12-31T00:00:00"

enum AxafEpoch
{
  FlightEpoch,
  CalibEpoch
};

double ds_date2relsec( char *date, char *time, enum AxafEpoch epoch );
double ds_date2sec( char *date, char *time );

/* ds_check_parameters function */
extern int ds_check_parameters(paramfile param_file,char* name); 

/* this function checks for existence of a series of parameters.  It is
   located in the same .c file as ds_check_parameters: ds_check_parameters.c */
extern dsErrBool ds_check_all_params(paramfile parfile_to_check,
				     char *par_name_p,
				     char **params_to_check_a,
				     long num_params,
				     dsErrList *param_check_errors_p);

/* ds_check_access */
extern int ds_check_access(char* filename,char* file_access);

/* ds_find_index_array */
extern void ds_find_index_array(float** input, 
				int* indexArray, 
				int size, 
				int dimension);

extern void ds_find_axes_lengths(float** input, 
				 int* index,
				 int* sizes,
				 int size, 
				 int dimension);

extern void ds_find_axes(float** input, 
			 const int* index,
			 const int* sizes,
			 float** outAxes,
			 int size, 
			 int dimension);

extern int ds_check_bit(unsigned char *quality, int array_index,
			unsigned short bit_position);




/* in file ds_createcol_minmax.c */

dmDescriptor* ds_ColumnCreate_minmax(
		       dmBlock *tableBlock,   /* i: data block            */
		       char    *columnName,   /* i: column name to create */
		       dmDataType   dataType, /* i: data type to create */
		       long     stringLen,    /* i: string length (0 if n/a) */
		       char    *unitName,     /* i: units for column */
		       char    *description,  /* i: description for column */
		       ...
		       );


dmDescriptor* ds_ColumnCreateArray_minmax(
		       dmBlock *tableBlock,   /* i: data block            */
		       char    *columnName,   /* i: column name to create */
		       dmDataType   dataType, /* i: data type to create */
		       long     stringLen,    /* i: string length (0 if n/a) */
		       char    *unitName,     /* i: units for column */
		       char    *description,  /* i: description for column */
		       long     size,         /* i: size of array */
		       ...
		       );


dmDescriptor* ds_ColumnCreateVector_minmax(
		       dmBlock *tableBlock,   /* i: data block            */
		       char    *columnName,   /* i: column name to create */
		       dmDataType   dataType, /* i: data type to create */
		       long     stringLen,    /* i: string length (0 if n/a) */
		       char    *unitName,     /* i: units for column */
		       char    *description,  /* i: description for column */
		       char   **cptNames,     /* i: component names */
		       long     dim,          /* i: size of array */
		       ...
		       );


/* ds_write_hdu_keys is a utility function that writes the HDUCLAS# keywords to
   the specified file.  If they cannot be written, an error will be stored.
   In addition, if any of the input hdu_clas strings are NULL, that keyword
   won't be written.

   Parameters:

   dest_block_p - I, block to write the keywords to
   hdu_class_p  - I, value of HDUCLASS keyword
   hdu_clas1_p  - I, value of HDUCLAS1 keyword
   hdu_clas2_p  - I, value of HDUCLAS2 keyword
   hdu_clas3_p  - I, value of HDUCLAS3 keyword
   file_name_p  - I, name of file being written to, for errors
   errors_p     - U, error list that will be updated if keywords 
   		     can't be written */

#define DS_HDUNAMES  "HDUCLASS","HDUCLAS1","HDUCLAS2","HDUCLAS3"

extern dsErrBool ds_write_hdu_keys(dmBlock *dest_block_p, 
				   char *hdu_class_p,
				   char *hdu_clas1_p,
				   char *hdu_clas2_p,
				   char *hdu_clas3_p,
				   char *file_name_p,
				   dsErrList *errors_p);

/* ds_clobber checks if a file exists and attempts to clobber it if 
   the user specifies to do so.  It does various access and error 
   checks to ensure that the file can be removed, and is removed.

   Parameters:

   file_name_p - I, Name of the file to clobber.
   clobber_e   - I, boolean specifying whether to clobber or not.
   errors_p    - U, Optional error list that will be populated
   		    with any errors that occur (if the arg is not NULL)
*/
extern long ds_clobber(char *file_name_p,
		       dsErrBool clobber_e,
		       dsErrList *errors_p);

extern int ds_check_quality_bit(unsigned char *qualiy,
				long column_number);


/* dateobs update functions */
/* are stored in ds_update_dateobs_keys.h to prevent the 
   header lib structures to be defined here. */

extern int ds_acis_s_gap_size(int verbose, double* gap);



/* CALDB quick access routines */


#define dsSETUP_CALDB_FROM_BLOCK 1
#define dsSETUP_CALDB_FROM_PARAM 2
extern int ds_caldb_lookup( char *caldbName,          /* file name for look up */
			    int   source,             /* whether to init from hdr or par*/
			    void *inBlock,            /* pointer to block/header */ 
			    char *dataProduct,        /* which data product?     */
			    char *overrideMission,    /* Override the mission    */
			    char *overrideInstrument, /* Override the instrument?*/
			    char *overrideDetector,   /* Override the detector?  */
			    dsErrList *errList
			    );

/* convert pha to energy in linear interpolation manner */
extern float ds_pha2energy
       (
	float pha,          /* Input, pha at the given point    */
	float pha_tb[],     /* Input, tabulized 2 pha points    */
	float eng_tb[]      /* Input, tabulized 2 energy points */
	);

/* convert energy to pha in linear interpolation manner */
extern float ds_energy2pha
       (
	float eng,          /* Input, energy at the given point */
	float pha_tb[],     /* Input, tabulized 2 pha points    */
	float eng_tb[]      /* Input, tabulized 2 energy points */
	);



/* bubble sort array or sort array at indices */
extern void ds_sort_array(      bool bubbleSort, long sz, double *x, long **index);
extern void ds_sort_array_float(bool bubbleSort, long sz,  float *x, long **index);
extern void ds_sort_array_long( bool bubbleSort, long sz,   long *x, long **index);


/* write pixlib history to a specified file pointer */
extern short ds_write_pixhist_in_dm(void *fpt);
extern short ds_write_pixhist_in_ascii(void *fpt);

extern int ds_copy_full_header(  dmBlock *inBlock, dmBlock *outBlock, 
				   char *tool, short primary );
extern int ds_fix_evt_hdr(dmBlock * evtBlock, Header_Type * obs_key);


extern int ds_map_hdr_to_caldb( char *calstr, Header_Type *hdr, calSEARCH *mySearch );
extern int ds_map_hdr_to_caldb_setDefault( Header_Type *hdr, calSEARCH *mySearch );
extern int ds_map_hdr_to_caldb_override( char *calstr, calSEARCH *mySearch );
extern int ds_map_hdr_to_caldb4( char *calstr, Header_Type *hdr, calSEARCH *mySearch, int verb);

Header_Type *ds_get_telins_fm_dm( dmBlock *inBlock, char *telescop, char *instrume, int verbose);
Header_Type *ds_get_telins(char *infile, char *telescop, char *instrume, int verbose);


/* calculate the cyclic data meaning of RA/DEC/ROLL */
/* the header below contains defines and prototyes  */
#include "ds_mean_pointing.h"




#endif   

/* end DSLIB_H */







