// 
//  Copyright (C) 2006-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



// fitsEmbeddedFunction_stub.h

// Header file for the "stub" embedded function library

#ifndef FITSEMBEDDEDFUNCTION_H
#define FITSEMBEDDEDFUNCTION_H

#include <ascdm.h>

  
// A guassian has three parameters: fwhm, position, and amplitude:
struct GaussRow
{
  float fwhm, pos, ampl;
};

// first type of data row: 10 gaussians
struct RowType1 
{
  short		ccd_id;
  long    	chipx[2];
  long 		chipy[2];
  long 		regnum;
  float 	energy;
  float 	channel;
  struct GaussRow	gaussians[10];
};

// second type of data row: 11 gaussians + power law + 2 errors
struct RowType2
{
  short		ccd_id;
  long    	chipx[2];
  long 		chipy[2];
  long 		regnum;
  float 	energy;
  float 	channel;
  struct GaussRow	gaussians[11];
  float		pow_gamma;
  float		pow_ampl;
  float		pow_ref;
  float		err1_ampl;
  float		err1_offset;
  float		err1_sigma;
  float		err2_ampl;
  float		err2_offset;
  float		err2_sigma;
};

// the embedded function class
class FitsEmbeddedFunction {

public:
  enum Error { Success, Fail };

  FitsEmbeddedFunction();
  ~FitsEmbeddedFunction();

  // io
  int   readG10Subr(struct RowType1 *rowTable);
  int   readG11Subr(struct RowType2 *rowTable);
  Error read(const char* fileName);

  // access methods

  Error setAxisName(int i, const char* name);
  Error getAxisName(int i, char* name, int len);

  int getGridSize(int i);
  Error getGridPoints(int axis, int length, float* outGrid);

  /* Sherpa-ism */
  int evaluateArrayAtConstAxis(int dim, int length, int axisNumberOfConstAxis,
			       float valueOfConstAxis, 
			       float** valuesOfNonConstAxes,
			       float* out);

  /* The following does nothing */
  int evaluateArrayAtConstAxisSetOneDataSpace(int i) {return(0);};

  /* the following copies the PHA array into the fef object */
  Error evaluateArrayAtConstAxisInitOneDataSpace(int length,
					       int axisNumberOfConstAxis,
					       float** valuesOfNonConstAxes);

  // legacy
  Error getAxisScale(int i, float& scale) {scale = 1.0; return Success;};
  Error getAxisOffset(int i, float& offset) {offset = 0.0; return Success;};

  // legacy
  void setDimension(int dim) {;};
  long getDimension() {return(2);};
  
 private:

  dmBlock *table;	// the DM table we read from

  struct RowType1 *rowTable1;	// the table in which we store the above data
  struct RowType2 *rowTable2;	// the table in which we store the above data
  long rowCount;	// the number of tables rows read
  float *phaArray;	// the PHA array
  long phaLength;	// number of entries in the PHA array
  long vfields;		// number of "fields" in the embedded function
  char *axisName[2];	// name of the evaluation axes
};
      
#endif //ifndef FITSEMBEDDEDFUNCTION_H
