// 
//  Copyright (C) 1998-2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)make_rmf.hh
 
* DEVELOPEMENT: Chandra Science Center rmf library
                hhe@cfa.harvard.edu (Initial version)

 
* DESCRIPTION:  Header file for make_rmf

 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             02/10/98
        0.3             08/26/98
 
*H****************************************************************************/

#ifndef _MAKE_RMF_HH_
#define _MAKE_RMF_HH_

// local  headers
#include "integral_rmf.hh"
#include <time.h>

//
// Class WeightARF provides methods to retrive multi-region ARF parameters 
// and the corresponding region-weight array in function of energy array, which
// are stored in the 2nd extension
//
class WeightARF : public RMFGeneral
{
  enum {CCD_ID, CHIPX, CHIPY, REGNUM, WEIGHT, TOTALCOLS};
  typedef struct rsp_region_s
  {
    char *infile;
    vector<string> colName;
    long numReg;
    vector<string> regSpec;
    vector<float> *rsp;
  } RSPRegion;
  void setColName();

public:
  WeightARF();
  WeightARF(dsErrList *, FILE*, short);
  ~WeightARF();

  dsErrCode createWRMFAt(long _regNo, long, vector<float>*);
  dsErrCode composeRSP(const char *infile);
  vector<float> *getWRMF(long *rows){*rows = d_rows; return d_wrmf;};
  vector<float> getEBOUNDS();
  long          getNumOfReg(){ return d_rsp->numReg;}
  char*         getRegSpecAt(long _regNo){ 
    if( _regNo <= d_rsp->numReg && d_rsp->numReg != 0 && _regNo != 0) 
      return (char*)d_rsp->regSpec[_regNo-1].c_str();
    else 
      return NULL;
  };
  vector<float> getRSPAt(long _regNo){
    if( _regNo <= d_rsp->numReg && d_rsp->numReg != 0 )
      return d_rsp->rsp[_regNo-1];
    else
      return vector<float>(0);
  };
  bool isValidREGNUM(char *_file, long *regNum);


private:
  dsErrCode createWARF();

  RSPRegion *d_rsp;
  long d_rows;
  vector<float> *d_wrmf;
  vector<float> d_warf;

};
//------------------------------------------------
//
//------------------------------------------------
class MakeRMF : public RMFGeneral
{

  typedef struct pfile_param_s
  {
    short dimension;       // dimension
    string *axis_name;    // array of axis names
    long  *length;         // length of grids on each axis
    vector<float> *range;  // number of elements of the grids, input from param file
    string *bin_method;   // bin method: linear or logrithm
    float   *bin_step;     // bin step

    double  lowThres;       // low threshold of energy cutoff
    string  infile,        // input file, such as ECD table
            outfile;        // output file, e.g. matrix fits file
    static string pf_name; // the name of the  parameter file
    char outfmt[DS_SZ_LINE];


  } pfile_param;

public:

  MakeRMF();
  MakeRMF(dsErrList *, FILE*, short);
  ~MakeRMF();

  void doTiming(const char *);
  dsErrCode processRMF();
  dsErrCode writeFitsMatrix(const char *output_fmt, vector<long> outchans);
  WeightARF *getRSPObj(){ return d_warf; };

  // Followed Set and Get functions are provided for user'r interface
  // to  override parameter file default setups
  void setFEF(string fef_file){   d_pfile->infile  = fef_file;};
  void setOutputFile(const char *out_file){   d_pfile->outfile  = out_file;  };
  void setLowThres(  const double low_thres){ d_pfile->lowThres = low_thres; };
  void setOutfmt( char *ofmt) { strcpy(d_pfile->outfmt, ofmt); };
  char *getOutfmt(){ return d_pfile->outfmt; };
  void setBinMethodAtAxis(short axis, string bin_method)
              { d_pfile->bin_method[axis] = bin_method;};
  void setPIOnFly(bool pi_on_fly){ d_fgrids->setPIOnFly(pi_on_fly); };
  bool isPIOnFlyReady(){ return d_fgrids->isPIOnFlyReady(); };
  bool getPIOnFly(){ return d_fgrids->isPIOnFlyReady(); };
  bool isPIFEF();

  dsErrCode setFEFBasic(string fef_file);
  dsErrCode setDimension(const short dimn);
  dsErrCode setAxisName(  short axis, const char *name);
  dsErrCode setAxisName(  short axis, string name){ return setAxisName(axis, name.c_str());};
  dsErrCode setBinStep(const short axis, float bin_step);
  dsErrCode setDataAtAxis(const short axis, vector<float> data);
  dsErrCode readFEF(long _theReg);

  vector<float> getEBOUNDS(){return d_ebounds;};
  const char *GetInputFile( void){ return  d_pfile->infile.c_str(); };
  const char *GetOutputFile(void){ return  d_pfile->outfile.c_str();};
  double GetLowThres(   void){ return  d_pfile->lowThres; };
  vector<short> GetGriddedAxis(){ return d_fgrids->GetGriddedAxis();};
  vector<string> GetFEFAxisName();

private:
  const char *const className() { return ("MakeRMF"); };
  dsErrCode setGrids();
  dsErrCode formEffectiveGrids();

  dsErrCode resetWRMF();

  dsErrCode calcEBounds();
  dsErrCode PI2EBounds();
  dsErrCode PHA2EBounds();
  dsErrCode chan2EBounds();
  dsErrCode EBounds2PHA(vector<float> &);
  dsErrCode convToPIRMF();
  dsErrCode initPIOnFly();

  UserGrids     *d_ugrids;  //pointer to UserGrids
  FEFGrids      *d_fgrids;  //pointer to FEFGrids
  EffGrids      *d_egrids;  //pointer to EffGrids
  IntegRMF      *d_intrmf;
  WeightARF     *d_warf;
  pfile_param   *d_pfile;   // make_rmf.par parameter container
  vector<float>  d_ebounds; // energy bounds

  vector<float> d_changrid;  // channel grid of user's input, start at 1
};


#endif //_MAKE_RMF_HH_




