// 
//  Copyright (C) 1998-2003,2006-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//




/*H****************************************************************************
 
* FILE NAME:    @(#)rmf_grids.hh
 
* DEVELOPEMENT: Chandra Science Center rmf library
                hhe@cfa.harvard.edu (Initial version)

* DESCRIPTION: Header file

 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             8/10/98

 
*H****************************************************************************/
#ifndef _RMF_GRIDS_HH_
#define _RMF_GRIDS_HH_

#include "rmf_general.hh"
#include "fitsEmbeddedFunction_stub.h"

//**************************************
//
// The base class of RMFGrids
//
//**************************************
class RMFGrids : public RMFGeneral
{

public:

  RMFGrids();
  RMFGrids(dsErrList *, FILE *, short);
  virtual ~RMFGrids();

  RMFGeneral::ERROR CheckGridSize();     // check memory size

  RMFGeneral::ERROR SetDimension ( const short n_dimn);
  RMFGeneral::ERROR SetGridsAtAxis(short axis,  long length, 
		       float *value     // input, grid values
			);
  RMFGeneral::ERROR SetGridsAtAxis(short axis,  vector<float> value);
  void  SetGridLengthAtAxis(short axis, long lngth){d_length[axis] = lngth;};
  RMFGeneral::ERROR SetAxisName(short axis, string name);

  string GetAxisName(short axis){return d_axis_name[axis];};
  short GetDimension(){ return d_dimension;};
  long *GetLength()   { return d_length;   };
  vector<float> *GetGrids(){   return d_grids;};
  vector<float> GetGridsAtAxis(short axis){   return d_grids[axis];};
  void PrintGrids();

protected:
  short  d_dimension;        // number of grid dimension
  long  *d_length;           // lengths of d_dimnson axises
                             // Currently, d_dimnson=2 for energy and pha
  vector<float> *d_grids;    // values of the d_dimension-axis variables
  string *d_axis_name;
 
  virtual const char *const className() = 0;

};


//**************************************
//
// class container of UserGrids
//
//**************************************
class UserGrids : public RMFGrids
{

public:

// default
  UserGrids() : RMFGrids() {};         // default
  UserGrids(dsErrList* _errLst, FILE* _log, short _verb) : RMFGrids(_errLst, _log,_verb) {};
  UserGrids(short dimension) : RMFGrids() {SetDimension(dimension);};           virtual ~UserGrids() {;};

private:
  virtual const char *const className() { return ("UserGrids"); };
};

//**************************************
//
// class container of FEFGrids
//
//**************************************
class FEFGrids : public RMFGrids 
{


public:
  
  FEFGrids() : RMFGrids(){
    d_fgrid = new FitsEmbeddedFunction;
    d_dmBk = NULL;
  };         // default
  FEFGrids(dsErrList* _errLst, FILE* _log, short _verb) : 
    RMFGrids(_errLst,_log,_verb) {
      d_fgrid = new FitsEmbeddedFunction;
      d_dmBk = NULL;
  };
  virtual ~FEFGrids() { 
    delete d_fgrid; 
    if(d_dmBk != NULL)
      dmTableClose(d_dmBk);
  };
  
  void SetInputFile( const char *in_file ){  d_infile   = in_file; };
  FitsEmbeddedFunction *GetFEFObj() { return d_fgrid; };
  dsErrCode fillUpGrids(const char *in_file);
  vector<short> GetGriddedAxis();

  dsErrCode setFEFBasic(const char *in_file);
  bool isMetaFEF();
  bool isPIOnFlyReady(){return d_piOnFly;};
  bool getPIOnFly(){return d_piOnFly;};
  void setPIOnFly(bool piOnFly){d_piOnFly = piOnFly;};
  dsErrCode getKeywordValue_f(char *key, float *val);

  dsErrCode createPHAGrids();
  void setPHAGrids(vector<float> _pha){d_pha.clear();d_pha=_pha;};
  vector<float> &getPHAGrids(){return d_pha;};

  dsErrCode getColsEnergyChan( vector<float> &_eng,  vector<float> &_chan);
  dsErrCode getPIBinParam(double *);

  RMFGeneral::ERROR GetAxisScale(short axis, double &scale);
  RMFGeneral::ERROR GetAxisOffset(short axis, double &offset);
  RMFGeneral::ERROR GetModelExpr(vector<string> model);

private:
  virtual const char *const className() { return ("FEFGrids"); };
  bool d_piOnFly;        // true if user's PI grids against FEF PHA grids and
                         // PIBINWID keyword exists in FEF header
  string d_infile;               // name of FITS embedded function file
  FitsEmbeddedFunction *d_fgrid;  // pointer to  FEF object
  vector<float>  d_pha;
  dmBlock       *d_dmBk;
  float          d_cbounds[2]; // channel bounds, low-high
};


#endif //_RMF_GRIDS_HH_
