#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# acis_build_badpix.t
# test script for acis_build_badpix


# !!2
# syntax:
# acis_build_badpix.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# Initialization

# !!3
toolname="acis_build_badpix"

# set up list of tests
# !!4
alltests="test1 fwb test2 test3 test4 test5 test6 test7 fep usrfile usrfile2a usrfile2b \
          usrfile3 usrfile4 bitflag bitflag2 bitflag3 bitflag4 bitflag5 bitflag6 bitflag7a bitflag7b \
          falsefep0 maxErr maxErr2 maxErrCCMODE"


# "short" test to run
# !!5
shortlist="test1 fwb test2 test3 test4 test5"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}.log

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directoryacis_build_badpix.t
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff

# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=0 #`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk.stk \
            berrfile=@$INDIR/berr.stk \
            calibfile=$INDIR/calib.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes  \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=6"
            echo $test1_string | tee -a  $LOGFILE 
            eval $test1_string | tee -a  $LOGFILE  2>&1
            badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
            ;;

    # Same as test1 except that a new calibfile 
    # New calibfile does not contain data with bit_5=1 or bit_6=1
    test7 ) test7_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk.stk \
            berrfile=@$INDIR/berr.stk \
            calibfile=$INDIR/calib_rm_row_col.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes  \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=6"
            echo $test7_string | tee -a  $LOGFILE 
            eval $test7_string | tee -a  $LOGFILE  2>&1
            badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
            ;;

    fwb ) fwb_string="acis_build_badpix \
          pbkfile=$INDIR/acisf053104213N002_pbk0.fits \
          berrfile=@$INDIR/null.lis \
          maxerr=10 \
          calibfile='CALDB(CCD_ID=0)' \
          usrfile="NONE" \
          procbias=yes \
          biasfile=@$INDIR/null.lis \
          evt0file=$INDIR/acisf053104213N002_0_evt0.fits \
          writebias=yes \
          outbias=$OUTDIR/fwb \
          obsfile=$INDIR/obs_1192.par \
          outfile=$outfile \
          clobber=yes \
          verbose=0 \
          mode=ql \
          biasthresh=6"
          echo $fwb_string | tee -a  $LOGFILE 
          eval $fwb_string | tee -a  $LOGFILE  2>&1
          badId="badpix3"
          ;;

    # fep0: uses calib file acisD1999-09-16badpixN0002.fits
    #       and axaff00333_000N002_obs0a.par
    fep ) fep_string="acis_build_badpix \
          pbkfile=$INDIR/acisf055915346N002_pbk0.fits \
          berrfile=@$INDIR/berrfep.stk \
          calibfile=$INDIR/acisD1999-09-16badpixN0002.fits \
          usrfile="NONE" \
          procbias=yes \
          biasfile=@$INDIR/biasfep.stk \
          evt0file=none writebias=yes \
          outbias=$OUTDIR/fep \
          obsfile=$INDIR/axaff00333_000N002_obs0a.par \
          outfile=$outfile \
          clobber=yes \
          biasthresh=6"
          echo $fep_string | tee -a  $LOGFILE 
          eval $fep_string | tee -a  $LOGFILE  2>&1
          badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9"
          ;;

    # vfaint:  weird bias #4 + test biasthresh
    test2 ) test2_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk2.stk \
            berrfile=@$INDIR/berr2.stk \
            calibfile=$INDIR/calib.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias2.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=5"
            echo $test2_string | tee -a  $LOGFILE 
            eval $test2_string | tee -a  $LOGFILE  2>&1
            badId="badpix0 badpix1 badpix2 badpix3 badpix6 badpix7"
            ;;

    # vfaint: feed 'test2' output file back into tool as calibration file
    test6 ) test6_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk2.stk \
            berrfile=@$INDIR/berr2.stk \
            calibfile=$OUTDIR/test2.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias2.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=5"
            echo $test6_string | tee -a  $LOGFILE 
            eval $test6_string | tee -a  $LOGFILE  2>&1
            badId="badpix0 badpix1 badpix2 badpix3 badpix6 badpix7"
            ;;

    # CC mode
    test3 ) test3_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk3.stk \
            berrfile=@$INDIR/berr3.stk \
            calibfile=$INDIR/calib.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias3.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes  \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=6"
            echo $test3_string | tee -a  $LOGFILE 
            eval $test3_string | tee -a  $LOGFILE  2>&1
            badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9"
            ;;

    test4 ) test4_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk4.stk \
            berrfile=@$INDIR/berr4.stk \
            calibfile='CALDB(CCD_ID=0)' \
            usrfile="NONE" \
            biasfile=@$INDIR/bias4.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=6"
            echo $test4_string | tee -a  $LOGFILE 
            eval $test4_string | tee -a  $LOGFILE  2>&1
            badId="badpix0 badpix1 badpix2 badpix3 badpix6 badpix7"
            ;;

    # feed test4 output (sav-dir) file back into tool as calibration file
    test5 ) test5_string="acis_build_badpix \
            pbkfile=@$INDIR/pbk4.stk \
            berrfile=@$INDIR/berr4.stk \
            calibfile=$SAVDIR/test4.fits \
            usrfile="NONE" \
            biasfile=@$INDIR/bias4.stk \
            procbias=yes \
            evt0file=none \
            writebias=no \
            clobber=yes \
            outfile=$outfile \
            obsfile=$INDIR/badpix_obs.par \
            outbias=$OUTDIR/foo \
            biasthresh=6"
            echo $test5_string | tee -a  $LOGFILE 
            eval $test5_string | tee -a  $LOGFILE  2>&1
            badId="badpix0 badpix1 badpix2 badpix3 badpix6 badpix7"
            ;;

    # usrfile test 
    usrfile ) usrfile_string="acis_build_badpix \
              pbkfile=@$INDIR/pbk.stk \
              berrfile=@$INDIR/berr.stk \
              calibfile=$INDIR/calib_rm_row_col.fits \
              usrfile=$INDIR/input.dat \
              biasfile=@$INDIR/bias.stk \
              procbias=yes \
              evt0file=none \
              writebias=no \
              clobber=yes  \
              outfile=$outfile \
              obsfile=$INDIR/badpix_obs.par \
              outbias=$OUTDIR/foo \
              biasthresh=6"
              echo $usrfile_string | tee -a  $LOGFILE 
              eval $usrfile_string | tee -a  $LOGFILE  2>&1
              badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
              ;;

    # usrfile2a test 
    usrfile2a ) usrfile2a_string="acis_build_badpix \
               pbkfile=$INDIR/acisf055915346N002_pbk0.fits \
               berrfile=none \
               calibfile=$INDIR/00333_bpix1b.fits \
               usrfile=$INDIR/usrfile2.dat \
               bitflag="00000000000000122221100020022222" \
               procbias=no \
               biasfile=none \
               evt0file=none \
               obsfile=$INDIR/axaff00333_000N002_obs0a.par \
               outfile=$outfile \
               clobber=yes"
               echo $usrfile2a_string | tee -a  $LOGFILE 
               eval $usrfile2a_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # usrfile2b test 
    usrfile2b ) usrfile2b_string="acis_build_badpix \
               pbkfile=$INDIR/acisf055915346N002_pbk0.fits \
               berrfile=none \
               calibfile=$INDIR/00333_bpix1b.fits \
               usrfile=$INDIR/usrfile2.dat \
               bitflag="00000000000000022221100020022222" \
               procbias=no \
               biasfile=none \
               evt0file=none \
               obsfile=$INDIR/axaff00333_000N002_obs0a.par \
               outfile=$outfile \
               clobber=yes"
               echo $usrfile2b_string | tee -a  $LOGFILE 
               eval $usrfile2b_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # usrfile3 test 
    usrfile3 ) usrfile3_string="acis_build_badpix \
               pbkfile=$INDIR/acisf055915346N002_pbk0.fits \
               berrfile=none \
               calibfile=$INDIR/00333_bpix1b.fits \
               usrfile=none \
               bitflag="00000000000000122221100020022222" \
               procbias=no \
               biasfile=none \
               evt0file=none \
               obsfile=$INDIR/axaff00333_000N002_obs0a.par \
               outfile=${outfile} \
               clobber=yes"
               echo $usrfile3_string | tee -a  $LOGFILE 
               eval $usrfile3_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # usrfile4 test 
    usrfile4 ) usrfile4_string="acis_build_badpix \
               pbkfile=$INDIR/acisf055915346N002_pbk0.fits \
               berrfile=none \
               calibfile=$INDIR/usrfile3_bpix1b.fits \
               usrfile=$INDIR/usrfile4.dat \
               bitflag="00000000000000222221100020022222" \
               procbias=no \
               biasfile=none \
               evt0file=none \
               obsfile=$INDIR/axaff00333_000N002_obs0a.par \
               outfile=$outfile \
               clobber=yes"
               echo $usrfile4_string | tee -a  $LOGFILE 
               eval $usrfile4_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # bitflag test 
    bitflag ) bitflag_string="acis_build_badpix \
              pbkfile=@$INDIR/pbk.stk \
              berrfile=@$INDIR/berr.stk \
              calibfile=$INDIR/calib_rm_row_col.fits \
              usrfile="NONE" \
              bitflag="00000000000000000000000010000001" \
              biasfile=@$INDIR/bias.stk \
              procbias=yes \
              evt0file=none \
              writebias=no \
              clobber=yes  \
              outfile=$outfile \
              obsfile=$INDIR/badpix_obs.par \
              outbias=$OUTDIR/foo \
              biasthresh=6"
              echo $bitflag_string | tee -a  $LOGFILE 
              eval $bitflag_string | tee -a  $LOGFILE  2>&1
              badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
              ;;

    # bitflag2 test 
    bitflag2 ) bitflag2_string="acis_build_badpix \
               pbkfile=@$INDIR/pbk.stk \
               berrfile=@$INDIR/berr.stk \
               calibfile=$INDIR/calib_rm_row_col.fits \
               usrfile="NONE" \
               bitflag="00000000000000122221100020022222" \
               biasfile=@$INDIR/bias.stk \
               procbias=yes evt0file=none \
               writebias=no \
               clobber=yes  \
               outfile=$outfile \
               obsfile=$INDIR/badpix_obs.par \
               outbias=$OUTDIR/foo \
               biasthresh=6"
               echo $bitflag2_string | tee -a  $LOGFILE 
               eval $bitflag2_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # bitflag3 test 
    bitflag3 ) bitflag3_string="acis_build_badpix \
               pbkfile=@$INDIR/pbk.stk \
               berrfile=@$INDIR/berr.stk \
               calibfile=$INDIR/calib_rm_row_col.fits \
               usrfile="NONE" \
               bitflag="00000000000000000000000010000011" \
               biasfile=@$INDIR/bias.stk \
               procbias=yes \
               evt0file=none \
               writebias=no \
               clobber=yes  \
               outfile=$outfile \
               obsfile=$INDIR/badpix_obs.par \
               outbias=$OUTDIR/foo \
               biasthresh=6"
               echo $bitflag3_string | tee -a  $LOGFILE 
               eval $bitflag3_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

    # bitflag4 test 
    bitflag4 ) bitflag4_string="acis_build_badpix \
               pbkfile=@$INDIR/pbk.stk \
               berrfile=@$INDIR/berr.stk \
               calibfile=$INDIR/calib_rm_row_col.fits \
               usrfile="NONE" \
               bitflag="00000000000000000000000010000021" \
               biasfile=@$INDIR/bias.stk \
               procbias=yes \
               evt0file=none \
               writebias=no \
               clobber=yes  \
               outfile=$outfile \
               obsfile=$INDIR/badpix_obs.par \
               outbias=$OUTDIR/foo \
               biasthresh=6"
               echo $bitflag4_string | tee -a  $LOGFILE 
               eval $bitflag4_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
               ;;

     # bitflag5 test
     bitflag5 ) bitflag5_string="acis_build_badpix \
                pbkfile=$INDIR/bitflag5_pkb0.fits \
                berrfile=none \
                calibfile=$INDIR/bitflag5_bpix1b.fits \
                usrfile="NONE" \
                bitflag="00000000000000022220100020022222" \
                biasfile=none \
                procbias=no \
                evt0file=none \
                writebias=no \
                clobber=yes  \
                outfile=$outfile \
                obsfile=$INDIR/bitflag5_obs0a.par \
                outbias=$OUTDIR/foo \
                biasthresh=6"
                echo $bitflag5_string | tee -a  $LOGFILE 
                eval $bitflag5_string | tee -a  $LOGFILE  2>&1
                badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
                ;;

    # bitflag6 test 
    bitflag6 ) bitflag6_string="acis_build_badpix \
               pbkfile=@$INDIR/pbk.stk \
               berrfile=@$INDIR/berr.stk \
               calibfile=$INDIR/bitflag6_bpix1b.fits \
               usrfile="NONE" \
               bitflag="00000000000000100000000010000001" \
               biasfile=@$INDIR/bias.stk \
               procbias=yes \
               evt0file=none \
               writebias=no \
               clobber=yes  \
               outfile=$outfile \
               obsfile=$INDIR/badpix_obs.par \
               outbias=$OUTDIR/foo \
               biasthresh=6"
               echo $bitflag6_string | tee -a  $LOGFILE 
               eval $bitflag6_string | tee -a  $LOGFILE  2>&1
               badId="badpix4 badpix5 badpix6 badpix7 badpix8 badpix9" 
              ;;

    # bitflag7a test 
    bitflag7a ) bitflag7a_string="acis_build_badpix \
               pbkfile=${INDIR}/bit_flag/acisf072038471N004_pbk0.fits \
               berrfile=none \
               calibfile=${INDIR}/bit_flag/aglow_bpix1.fits \
               usrfile=NONE \
               bitflag="00000000000000122221100020022222" \
               biasfile=none \
               procbias=no \
               evt0file=none \
               clobber=yes  \
               outfile=$outfile \
               obsfile=${INDIR}/bit_flag/obs.par"
               echo $bitflag7a_string | tee -a  $LOGFILE 
               eval $bitflag7a_string | tee -a  $LOGFILE  2>&1
               badId="badpix0 badpix1 badpix2 badpix3 badpix6" 
              ;;

    # bitflag7b test 
    bitflag7b ) bitflag7b_string="acis_build_badpix \
               pbkfile=${INDIR}/bit_flag/acisf072038471N004_pbk0.fits \
               berrfile=none \
               calibfile=${INDIR}/bit_flag/aglow_bpix1.fits \
               usrfile=NONE \
               bitflag="00000000000000022221100020022222" \
               biasfile=none \
               procbias=no \
               evt0file=none \
               clobber=yes  \
               outfile=$outfile \
               obsfile=${INDIR}/bit_flag/obs.par"
               echo $bitflag7b_string | tee -a  $LOGFILE 
               eval $bitflag7b_string | tee -a  $LOGFILE  2>&1
               badId="badpix0 badpix1 badpix2 badpix3 badpix6" 
              ;;

     falsefep0 ) falsefep0_string="acis_build_badpix \
                 pbkfile=$INDIR/acisf392639608N001_pbk0.fits \
                 berrfile=@$INDIR/falseberr.lis \
                 calibfile='NONE' \
                 procbias='no' \
                 biasfile='' \
                 obsfile=$INDIR/axaff12218_000N001_obs0a.par \
                 outfile=$outfile"
                 echo $falsefep0_string | tee -a  $LOGFILE 
                 eval $falsefep0_string | tee -a  $LOGFILE  2>&1
                 badId="badpix0 badpix1 badpix2 badpix3 badpix6" 
                 ;;
      
     maxErr ) maxErr_string="acis_build_badpix \
              pbkfile=$INDIR/maxErr_pbk0.fits \
              berrfile=$INDIR/maxErr_berr0.fits \
              maxerr=10 \
              calibfile='NONE' \
              procbias='no' \
              biasfile='' \
              obsfile=$INDIR/maxErr_obs0a.par \
              outfile=$outfile"
              echo $maxErr_string | tee -a  $LOGFILE 
              eval $maxErr_string | tee -a  $LOGFILE  2>&1
              badId="badpix4 badpix7 badpix5 badpix6 badpix8 badpix9"
              ;;

     maxErr2 ) maxErr2_string="acis_build_badpix \
              pbkfile=$INDIR/maxErr2_pbk0.fits \
              berrfile=$INDIR/maxErr2_berr0.fits \
              maxerr=1 \
              calibfile='NONE' \
              procbias='no' \
              biasfile='' \
              obsfile=$INDIR/maxErr2_obs0a.par \
              outfile=$outfile"
              echo $maxErr2_string | tee -a  $LOGFILE 
              eval $maxErr2_string | tee -a  $LOGFILE  2>&1
              badId="badpix4 badpix7 badpix5 badpix6 badpix8 badpix9"
              ;;
      
     maxErrCCMODE ) maxErrCCMODE_string="acis_build_badpix \
              pbkfile=$INDIR/maxErrCCMODE_pbk0.fits \
              berrfile=$INDIR/maxErrCCMODE_berr0.fits \
              maxerr=1 \
              calibfile='NONE' \
              procbias='no' \
              biasfile='' \
              obsfile=$INDIR/maxErrCCMODE_obs0a.par \
              outfile=$outfile"
              echo $maxErrCCMODE_string | tee -a  $LOGFILE 
              eval $maxErrCCMODE_string  | tee -a  $LOGFILE  2>&1
              badId="badpix4 badpix7 badpix5 badpix6 badpix8 badpix9"
              ;;
  esac


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # FITS table    (duplicate for as many tables per test as needed)

  # compare
  # !!12
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #     /dev/null 2>>$LOGFILE
  for badid in $badId
  do
    dmdiff $outfile"[${badid}]" $savfile"[${badid}]" tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
      echo "dmdiff ${outfile}'[${badid}]' ${savfile}'[${badid}]' tol=${SAVDIR}/tolerance" >> $LOGFILE
      mismatch=0
    fi
  done


  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
