#!/usr/bin/env perl
# 
#  Copyright (C) 2019,2021  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

#
#############################################################
# Regression Test Script Template
#############################################################
# Turn off buffering
select(STDOUT); $| = 1;

my $tool = "asp_offaxis_corr"; # Name of tool to be tested
my $unittest = "unittest_asp_offaxis_corr";
#----------------------------------------------------------------------
# Find Test Utilities File
#----------------------------------------------------------------------
use File::Basename;
$cwd = dirname($0); # relative to where this script is run from (cwd).
$tutils = "${cwd}/regtest_utils.pl";
if ( ! -e "$tutils" )
{
   print "ERROR: Test Utilities file not found. '${tutils}' \n";
   exit(1);
}
else
{
   require "$tutils";
}

#----------------------------------------------------------------------
# Define base directories
#----------------------------------------------------------------------
#Get test paths with tool specific extensions
($indir, $outdir, $savdir, $logdir) = &get_test_dirs( $tool );

#Create output directories as needed
&gen_test_dirs( $outdir, $logdir );
  
#----------------------------------------------------------------------
# Define Test Suite.
#----------------------------------------------------------------------
my @testlist = define_test_suite();

#----------------------------------------------------------------------
# Standard Test Run.
#----------------------------------------------------------------------
do_standard_test( $tool, $outdir, $logdir, $savdir, \@testlist );



#----------------------------------------------------------------------
# SUBROUTINES
#----------------------------------------------------------------------
# ===================================================================
# define_test_suite( void )
#   Create TestCase objects defining each test.
#
#   Arguments
#     none
#
#   Globals
#     tool    = Tool being tested
#     indir   = Test input directory
#     outdir  = Test output directory
#
#   Returns
#     tests: array of TestCase objects
# --------------------------------------------------------------------
sub define_test_suite
{
  my @testlist = [];
  my $ntests   = 5;   #!! how many test cases

  # pre-size the testslist array
  $endndx = $ntests - 1;
  $#testlist = $endndx;  # this sizes the array

  #!! Find test executable - adjust as needed to support command generation.
  my $toolexe = "";
  $toolexe = find_in_path( $tool, $ENV{'PATH'}, ':');

  if ( $toolexe eq "" )
  {
    print "ERROR: Tool executable not found. \n";
  }
  # Add unit test exe
  my $unitexe = dirname($toolexe);
  my $find = 'bin';
  my $repl = 'test/bin/'.${tool}.'/'.${unittest};
  $unitexe =~ s/$find/$repl/;
  if ( $unitexe eq "" )
  {
    print "ERROR: Unit test executable not found. \n";
  }
  # Find system utilities used
  #!! Add any system utilities used in test command strings
  my $CP = find_in_path("cp", $ENV{'PATH'}, ':');

  my $comstr = ""; # command string
  my $asol = "asol.orig"; # Name of input asol file
  for $ii (1 .. $ntests) 
  {
    my $testid = sprintf("test%02d", $ii);
    my $t = TestCase->new( id => $testid, mksubdir => 0 );
    my $outfile = "asol_${testid}"; # Output asol file name
    my $outverbose = "${testid}.txt"; # Output verbose file name

    #Setup each TestCase.
    if ( $testid eq "test01" )
    {
     $comstr  = "${CP} ${indir}/${asol} ${outdir}/${outfile}; ";
     $comstr .= "${toolexe} ";
     $comstr .= "${outdir}/${outfile} ";
     $comstr .= "ACIS ";
     $comstr .= "ver=2 1> ${outdir}/${outverbose}";

     $t->desc("Check header keys, new/updated cols for ACIS.");
     $t->type("regression");
     $t->command( $comstr );
     $t->verify("$outfile", dmdiff);
     $t->verify("$outverbose", diff);
    }
    elsif ( $testid eq "test02" )
    {
     $comstr  = "${CP} ${indir}/${asol} ${outdir}/${outfile}; ";
     $comstr .= "${toolexe} ";
     $comstr .= "${outdir}/${outfile} ";
     $comstr .= "HRC ";
     $comstr .= "ver=2 1> ${outdir}/${outverbose}";
    
     $t->desc("Check header keys, new/updated cols for HRC.");
     $t->type("regression");
     $t->command( $comstr );
     $t->verify("$outfile", dmdiff);
     $t->verify("$outverbose", diff);
    }
    elsif ( $testid eq "test03" )
    {
     $comstr  = "${CP} ${indir}/${outfile} ${outdir}/${outfile}; ";
     $comstr .= "${toolexe} ";
     $comstr .= "${outdir}/${outfile} ";
     $comstr .= "acis ";
     $comstr .= "ver=2 2> ${outdir}/${outverbose}";
    
     $t->desc("Check that tool exits if BOREVER==2.");
     $t->type("regression");
     $t->command( $comstr );
     $t->filters(0, "sub ${toolexe} -> ${tool}");
     $t->filters(1, "sub TEST -> ");
     $t->verify("$outverbose", diff);
    }
    elsif ($testid eq "test04")
    { $comstr  = "${CP} ${indir}/${asol} ${outdir}/${outfile}; ";
      $comstr .= "${unitexe} ";
      $comstr .= "${outdir}/${outfile} 1> ${outdir}/${outverbose}";
      
      $t->desc("Check quaternion calculation.");
      $t->type("unit");
      $t->command( $comstr );
      $t->verify("$outverbose", diff);
    }
    elsif ($testid eq "test05")
    { 
      # SL-244, Calc. mean pointing at 0|360 boundary

      $mpnt = "pcadm368717826N000_asol1.fits.orig"; 

      $comstr  = "${CP} ${indir}/${mpnt} ${outdir}/${outfile}; "; 
      $comstr .= "${toolexe} ";
      $comstr .= "${outdir}/${outfile} ";
      $comstr .= "ACIS ";
      $comstr .= "ver=2 1> ${outdir}/${outverbose}";
    
      $t->desc("Check ra/dec/roll mean-pointing calculation.");
      $t->type("regression");
      $t->command( $comstr );
      $t->verify("$outfile", dmdiff);
    }

    $ndx = $ii - 1;
    $testlist[$ndx] = $t;
  }

  return(@testlist);
}
