#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# asphist.t
# test script for asphist


# !!2
# syntax:
# asphist.t [<testid> ... ]
######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;s/COMMENT/Dataset/g;
  s/CHECKSUM/Dataset/g; s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}

######################################################################
# Initialization

# !!3
toolname="asphist"

# set up list of tests
# !!4
alltests="test1 test2 test3 test4 test5 test6 test_offchip empty1 empty2 \
          test_214 test_214_b test_6443 reprojected_events oscol_file oscol_file2"

# "short" test to run
# !!5
shortlist="test1"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*

# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi

# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc

# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

# ----- copy some files -----
cd ${OUTDIR}
rm -f ${OUTDIR}/dmgti_out.fits
rm -f ${OUTDIR}/acis_gti.fits
cp ${INDIR}/dmgti_out.fits  ${OUTDIR}/.
cp ${INDIR}/acis_gti.fits  ${OUTDIR}/.


script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # delete old outputs
  rm -f $OUTDIR/${testid}.fits

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test1_string="asphist \
               infile=@${INDIR}/pcadf01318_000N001_asol1_withGTI.lis \
               outfile=${outfile}  evtfile=@${INDIR}/acis1318_small_evt1.stk \
               dtffile=\"\" res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
            echo $test1_string | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval $test1_string
            ;;

    # !!7
    test2 ) test2_string="asphist \
               infile=${INDIR}/acis_newsim.fits[@${INDIR}/acis_gti.fits] \
               outfile=${outfile} evtfile=@${INDIR}/acis1318_small_evt1.stk \
               dtffile=\"\" res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
	    echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
	    eval $test2_string
           ;;

    # !!8  for HRC and (dtffile!=NONE )
    test3 ) test2_string="asphist \
               infile=@${INDIR}/hrc1292_asol1.lis \
               outfile=${outfile} evtfile=${INDIR}/hrc1292_evt1.fits \
               dtffile=${INDIR}/hrc1292_dtf1.fits res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
	    echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
	    eval $test2_string
           ;;
    # !!9  for HRC and (dtffile=="")
    test4 ) test2_string="asphist \
               infile=@${INDIR}/hrc1292_asol1.lis \
               outfile=${outfile} evtfile=${INDIR}/hrc1292_evt1.fits \
               dtffile=\"\" res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
            echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test2_string
           ;;
    # !! 10  for HRC and (dtffile==NONE)
    test5 ) test2_string="asphist \
               infile=@${INDIR}/hrc1292_asol1.lis \
               outfile=${outfile} evtfile=${INDIR}/hrc1292_evt1.fits \
               dtffile=NONE res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
            echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test2_string
           ;;
    # !! 10  : there's no row in the 1st pcad_asol1.fits
    test6 ) test2_string="asphist \
               infile=@${INDIR}/pcad_test6.lis \
               outfile=${outfile} evtfile=${INDIR}/acis_test6_evt1.fits \
               dtffile=NONE res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
            echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test2_string
           ;;


    test_offchip ) test2_string="asphist \
               infile=${INDIR}/pcadf058130999N002_asol1.fits \
               outfile=${outfile} evtfile=${INDIR}/acisf01112N002_evt2.fits \
               dtffile=NONE res_xy=0.5 res_roll=600.0 max_bin=10000.0 \
               clobber=yes > /dev/null"
            echo $test2_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test2_string
           ;;
    # !! from test1; only the 1st infile is empty and 2nd is not; output has data;
    empty1 ) run_string="asphist \
               infile=@${INDIR}/empty1.lis \
               outfile=${outfile}  evtfile=@${INDIR}/acis1318_small_evt1.stk \
               dtffile=\"\" res_xy=0.5 res_roll=600.0 clobber=yes > /dev/null"
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
            ;;

    # !! from test1; empty infile ; zero row in the output ; save empty0.fits;
      empty2 ) run_string="asphist \
               infile=${INDIR}/pcadf054653555N001_asol1.fits\"[time<54653550.0]\" \
               outfile=${outfile}  evtfile=@${INDIR}/acis1318_small_evt1.stk \
               dtffile=\"\" res_xy=0.5 res_roll=600.0 clobber=yes  > /dev/null" 
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
            ;;

    #
    # !! for HRC and (evtfile==hrc_noDtcor_evt1.fits)
    # ( crash test ;  HRC :  no DTCOR in evt and dtffile=NONE )
      test7 ) run_string="asphist \
               infile=@${INDIR}/hrc1292_asol1.lis \
               outfile=${outfile} evtfile=${INDIR}/hrc_noDtcor_evt1.fits \
               dtffile=NONE res_xy=0.5 res_roll=600.0 clobber=yes > /dev/null"
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
           ;;
    # 
    # no GTI filter in 'infile' (recommended) ;
    # total duration  = livetime7 of evtfile
    # screening aspect with Evt's or Aspect's GTI has different results ;
    #
    test_214 ) run_string="asphist \
               infile=${INDIR}/pcad_214_asol1.fits \
               outfile=${outfile} evtfile=${INDIR}/acis_214_evt2.fits\"[ccd_id=7]\" \
               dtffile=NONE res_xy=0.5 res_roll=600.0 clobber=yes > /dev/null"
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
           ;;
    # 
    # GTI filter in 'infile' is from 'evtfile';
    # test_214_b's duration Less than test_214's duration ;
    # screening aspect with Evt's or Aspect's GTI has same results ;
    #
    test_214_b ) rm -f ${OUTDIR}/acis_214_evt2.fits
                 cp ${INDIR}/acis_214_evt2.fits  ${OUTDIR}/.
               run_string="asphist \
               infile=${INDIR}/pcad_214_asol1.fits\"[@acis_214_evt2.fits[ccd_id=7]]\"  \
               outfile=${outfile}  evtfile=\"${INDIR}/acis_214_evt2.fits[ccd_id=7]\" \
               dtffile=NONE res_xy=0.5 res_roll=600.0 clobber=yes > /dev/null"
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
           ;;
    #
    # test warnings. asol_6443_part1+3 have all time rec's outside evt-gti. 
    #
    test_6443 ) run_string="asphist \
                infile='@${INDIR}/asol_6443.list' \
                outfile=${outfile} \
                evtfile='${INDIR}/small_evt2_6443.fits[CHIP_ID=3]' \
                dtffile=${INDIR}/dtf1_6443.fits \
                res_xy=0.5 res_roll=600.0 \
                max_bin='40000' clobber='yes' > /dev/null "
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
           ;;

    #
    # Reprojected datasets require taking into account the tangent plane
    # of the reprojecte data not the default pixlib one.
    #
    reprojected_events ) run_string="asphist \
               infile=$INDIR/pcadf344841918N001_asol1.fits \
	       outfile=$outfile \
               evtfile=${INDIR}/reprojected_events.fits \
               res_xy=0.5 res_roll=600.0 clob+"
            echo $run_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $run_string
           ;;

    # Test where the extname is OSCOL but has all the columns of ASPSOL
    oscol_file )  run_string="asphist \
                     infile=${INDIR}/asol.sso \
                     outfile=$outfile \
                     evtfile=${INDIR}/evt.sso \
                     dtf= clob+"
                  echo $run_string | tee -a  $LOGFILE
                  echo "" | tee -a  $LOGFILE
                  eval $run_string
                  ;;

    # Test where the exname is OSCOL but purposely missing a column
    oscol_file2 )  run_string="asphist \
                     infile=${INDIR}/asol.sso_missing_col \
                     outfile=$outfile \
                     evtfile=${INDIR}/evt.sso \
                     dtf= clob+"
                   echo $run_string | tee -a  $LOGFILE
                   echo "" | tee -a  $LOGFILE
                   eval $run_string
                   mystatus=$?
                   echo " -- status of call was $mystatus"  >>  $LOGFILE
                   echo " -- status of call was $mystatus"  > ${outfile}.status
                   ;;
  esac


  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)
  # compare
  # !!12
  if [ ${testid} = "oscol_file2" ]; then
    # this test checks for expected errors so diff the exit status of run
    diff $outfile.status $savfile.status  > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile.status" >> $LOGFILE
      mismatch=0
    fi
  else
    dmdiff $outfile $savfile tol=$SAVDIR/tolerance  error_on_wcs- >/dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
      mismatch=0
    fi

    # compare - GTI
    dmdiff $outfile"[GTI]" $savfile"[GTI]" keys=yes tol=$SAVDIR/tolerance >>$LOGFILE
    if  test $? -ne 0 ; then
       echo "ERROR: -- GTI mismatch in $outfile" >> $LOGFILE
       mismatch=0
    fi
  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
