#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]
 
# Developer's comments:
#
# default CALDB env for the regression test
# setenv CALDB /data/regression_test/development/CALDB4
#
# ---------------------------------------------------------





######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR,da/analysis/caltools/calquiz HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/WasteDataset/g;s/COMMENT/WasteDataset/g;
  s/DATE/WasteDataset/g;s/CREATOR/WasteDataset/g;s/HISTORY/WasteDataset/g; 
  s/DATASUM/WasteDataset/g;s/ASCDSVER/WasteDataset/g;s/HISTNUM/WasteDataset/g; 
  s/CVSD0001/WasteDataset/g; s/CVST0001/WasteDataset/g; s/FEFFILE/WasteDataset/g' | \
  grep -v WasteDataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}
######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}

exportenv()
{
 key=`echo $1`
 val=`echo $2`
 if test "x`uname`" = "xSunOS"; then
     if test $key = "CALDB"; then 
	 CALDB=$val; export CALDB 
     elif test $key = "CALDBCONFIG";then 
	 CALDBCONFIG=$val; export CALDBCONFIG
     elif test $key = "CALDBALIAS";then
	 CALDBCONFIG=$val; export CALDBALIAS
     fi
 else
     export $key=$val
 fi

 if test $key = "CALDB"; then 
     echo "export $key=" $CALDB; 
 elif test $key = "CALDBCONFIG";then 
     echo "export $key=" $CALDBCONFIG
 elif test $key = "CALDBALIAS";then
     echo "export $key=" $CALDBALIAS
 fi
}

######################################################################
# Initialization

# !!3
toolname="calquiz"

# set up list of tests
# !!4
#alltests="lookup_time"
alltests="mkgrmf mkarf_qe mkarf_qe2 mkarf_qeu mkarf_axe mkarf_vig mkarf_gre ccd_simz echo_qual echo_fidel cdev_http cdev_ftp cdev_foo cdev_foo2 cdev_dot cdev_dot2 cdev_ofl cdev_ofl2"

# "short" test to run
# !!5
#shortlist="lookup_time"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool calquiz



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
punlearn $toolname
pset $toolname verbose=0

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.txt
  savfile=$SAVDIR/${testid}.txt

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in

      mkgrmf) command_str="${toolname} \
	infile=${INDIR}/obs_1921_evt1.fits \
	product=LSFPARM  \
	calfile='CALDB(ORDER=1,RAND_TG=0.5,SHELL=1100)'"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      mkarf_qe) command_str="${toolname} \
        infile=${INDIR}/obs_3399_evt1.fits \
        product=QE calfile=CALDB"
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;
 
      mkarf_qeu) command_str="${toolname} \
        infile=${INDIR}/obs_3399_evt1.fits \
        product=QEU \
        calfile=CALDB "
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      mkarf_axe) command_str="${toolname} \
        infile=${INDIR}/obs_3399_evt1.fits \
        product=AXEFFA \
        calfile='CALDB(GRATING=LETG,SHELL=1000)' "
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;
      
      mkarf_vig) command_str="${toolname} \
        infile=${INDIR}/obs_3399_evt1.fits \
        product=VIGNET \
        calfile='CALDB(GRATING=LETG,SHELL=1000)' "
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      mkarf_gre) command_str="${toolname} \
        infile=${INDIR}/obs_3399_evt1.fits \
        product=GREFF \
        calfile='CALDB(GRATING=LETG,SHELL=0100)' "
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      ccd_simz) command_str="${toolname} \
        infile=${INDIR}/evt2.ccd7.fits \
        product=BKGRND \
        calfile='CALDB(CCD_ID=7)' "
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      lookup_time) command_str="${toolname} \
        infile=${INDIR}/acism62566_000N000_bpix1.fits \
        product=DET_GAIN \
        calfile='CALDB(FP_TEMP=183.4)' outfile=$outfile"
	 #setcaldb="source /data/CALDB/test_CP/caldbinit.unix" 
	 #echo $setcaldb | tee -a $LOGFILE
      	 #eval $setcaldb | tee -a $LOGFILE
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

       echo_qual) command_str="${toolname} \
        infile=none telescope=chandra instrume=acis \
        product=DET_GAIN  outfile=$outfile \
        calfile='CALDB(FP_TEMP=153.2, cal_qual=:5)' \
        echo_qual=y echo_fidel=y"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

       echo_fidel) command_str="${toolname} \
        infile=none telescope=chandra instrume=hrc \
        product=2d_psf  outfile=$outfile \
        calfile='CALDB(detnam=hrc-s,fidel=0:100, qual=:5)' \
        echo_qual=y echo_fidel=y"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      cdev_http) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_http"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      cdev_ftp) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_ftp"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      cdev_foo) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_foo"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
             ;;

      cdev_foo2) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_foo2"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1 
             ;;

      cdev_dot) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_dot"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1 
             ;;

      cdev_dot2) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_dot2"
	  echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1 
             ;;

      cdev_ofl) command_str="${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN calfile=CALDB outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_ofl"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
          ;;

      ## ST-93, validate the new defalt: 'calfile=CALDB'
      ## the output shuld match to cdev_ofl.txt
      cdev_ofl2) command_str="punlearn calquiz; ${toolname} \
         infile=$TESTIN/caldb4/CALDB_Cases/acisf00635_000N004_evt1.fits.gz \
         product=DET_GAIN outfile=$outfile"
	  exportenv 'CALDB'       "$TESTIN/caldb4/CALDB_Cases"
	  exportenv 'CALDBCONFIG' "$CALDB/software/tools/caldb.config.CDEV_ofl"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
          ;;
  
      ## ST-93, validate the new defalt: 'calfile=CALDB'
      ## the output shuld match to mkarf_qe.txt
      mkarf_qe2) command_str="punlearn calquiz; ${toolname} \
         infile=${INDIR}/obs_3399_evt1.fits product=QE"
          echo $command_str  | tee -a $LOGFILE
          eval $command_str  | tee -a $LOGFILE  2>&1
          ;;
      
  esac

  pget $toolname outfile | tr "," "\012" > $outfile

#--- get file without path from <tool>.par's 'outfile' field -----
#line1 read .par and single out 'outfile=' line
#line2 remove string "outfile = "
#line3 remove string "File(s) found"
#line4 remove the file path
 # plist ${toolname} | grep outfile |\
 # sed 's/'outfile\ \=\ '//g'       |\
 # sed 's/'File\(s\)\ found'//g'    |\
 # sed 's/\/*[a-Z,0-9, _]*\///g'     > $outfile

  outf=$OUTDIR/${testid}.txt_o
  savf=$OUTDIR/${testid}.txt_s

  mismatch=1
  if test -f $outfile; then
      if test "x${testid}" = "xcdev_http" \
	  -o  "x${testid}" = "xcdev_ftp"  \
	  -o  "x${testid}" = "xcdev_foo"  \
	  -o  "x${testid}" = "xcdev_foo2" \
	  -o  "x${testid}" = "xcdev_dot"  \
	  -o  "x${testid}" = "xcdev_dot2" \
	  -o  "x${testid}" = "xcdev_ofl"  \
	  -o  "x${testid}" = "xcdev_ofl2"; then        # ST-93 cdev_ofl2
	  cat $outfile > $outf
      else
	  cat $outfile | sed -e 's/\/.*\///g' > $outf  # ST-93 mkarf_qe2 included
      fi
  else  
      echo "not found " $outfile >> $LOGFILE
      mismatch=0
  fi

  ## ST-93, validate the new defalt: 'calfile=CALDB'
  ## the outputs should match to mkarf_qe.txt and cdev_ofl.txt, respectively
  if test "x${testid}" = "xmkarf_qe2"; then    # ST-93
      cat $SAVDIR/mkarf_qe.txt | sed -e 's/\/.*\///g' > $savf
  elif test "x${testid}" = "xcdev_ofl2"; then  # ST-93
      cat $SAVDIR/cdev_ofl.txt > $savf
  elif test -f $savfile; then
      if test "x${testid}" = "xcdev_http" \
	  -o  "x${testid}" = "xcdev_ftp"  \
	  -o  "x${testid}" = "xcdev_foo"  \
	  -o  "x${testid}" = "xcdev_foo2" \
	  -o  "x${testid}" = "xcdev_dot"  \
	  -o  "x${testid}" = "xcdev_dot2" \
	  -o  "x${testid}" = "xcdev_ofl"; then
	  cat $savfile > $savf
      else
	  cat $savfile | sed -e 's/\/.*\///g' > $savf
      fi
  else
      echo "not found " $savfile  >> $LOGFILE
      mismatch=0
  fi


  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  if test $mismatch -ne 0; then
      diff $outf $savf  > /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
	  echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
	  mismatch=0
      fi
  fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi
  echo ""
done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
