#! /bin/sh

# celldetect.t
# test script for celldetect

# syntax:
# celldetect.t [<testid> ... ]

######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter <infile> <outfile>
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}


######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length 
# Use this to protect yourself against empty files  (which will 
# 'diff' without error)  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="celldetect"

# set up list of tests
alltests="test1 test2 test3 test4 test5 test6 test7 test8 test9 test_double test_fixed_double test_extreme test_gradient test_gradient_convolve test_block test_arcsec test_physical test_sin test_rot test_nowcs test_lin test_lin2 test_sep test_convolve_fixedcell"

# "short" test to run
shortlist="test7"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

script_succeeded=0

######################################################################
# Begin per-test loop

  for testid in $testlist
  do

  punlearn ${toolname}
    
  # delete old outputs
  rm -f $OUTDIR/*${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_src.fits
  savfile=$SAVDIR/${testid}_src.fits

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    test1 )  test_string="celldetect log=no clobber=yes 
                infile=$TESTIN/$toolname/cell_SpB.fits 
                expstk=  bkgfile=   bkgvalue=0.0  
                bkgerrvalue=0.0 
                outfile=$outfile 
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
                regfile=none  findpeaks=yes  fixedcell=0  centroid=no 
		cellfile=$OUTDIR/cells_test1.fits
                thresh=4  convolve=no snrfile="
           ;;

    # Files cell_SpB_convolve.fits is a soft link to cell_spB.fits.  Different names 
    # are used so that the cellfile output of test1 and test2 are different.

    test2 )  test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB_convolve.fits
		expstk=  bkgfile=
		bkgvalue=0.0 bkgerrvalue=0.0 
		regfile=none 
		outfile=$outfile
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
		findpeaks=yes fixedcell=0  
		centroid=no thresh=4 convolve=yes 
		cellfile=$OUTDIR/cells_test2.fits
		snrfile=$OUTDIR/${testid}_snr.fits
		"
	    ;;

    test3 )  test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB.fits
		expstk=   bkgfile=  
		bkgvalue=0.0 bkgerrvalue=0.0 
		outfile=$outfile
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
		regfile=none
		findpeaks=yes fixedcell=0 
		centroid=yes cellfile=none
		thresh=4 ellsigma=3.0 
		convolve=no snrfile=
		"
	    ;;

    test4 )  test_string="celldetect log=no clobber=yes
		infile=$INDIR/cell_SpB.fits
		expstk=   bkgfile=
		bkgvalue=0.5 bkgerrvalue=0.0 
		outfile=$outfile
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
		regfile=none
		findpeaks=yes fixedcell=0 
		centroid=yes cellfile=none
		thresh=4 ellsigma=3.0 
		convolve=no snrfile=
		"
	    ;;

    test5 )  test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB.fits
		expstk=  bkgfile=$INDIR/cell_B.fits
		bkgvalue=0.0 bkgerrvalue=0.0 
		outfile=$outfile
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
		regfile=none
		findpeaks=yes fixedcell=0 
		centroid=yes cellfile=none
		thresh=4  ellsigma=3.0 
		convolve=no snrfile=
		"
	    ;;

    # Same data as test 5, maxlogicalwindow used to force use of blocking algorithm

    test_block ) test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB_bin1.fits
		psffile=@$INDIR/cell_SpB_bin1_psfmap.lis
                maxlogicalwindow=512 
		expstk=   bkgfile=  
		bkgvalue=0.0 bkgerrvalue=0.0 
		xoffset=INDEF yoffset=INDEF 
		outfile=$outfile
		regfile=none
		findpeaks=yes fixedcell=0 
		centroid=yes cellfile=$OUTDIR/cells_test_block.fits
		thresh=4 ellsigma=3.0 
		convolve=no snrfile=
		"
	    ;;

    test6 )  test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB.fits
                psffile=$TESTIN/$toolname/cell_SpB_psfmap.fits 
		expstk=  bkgfile=
		bkgvalue=0.0 bkgerrvalue=0.0 
		outfile=$outfile
		regfile=none
		findpeaks=yes fixedcell=9 
		centroid=yes cellfile=none
		thresh=3  ellsigma=3.0 
		convolve=no snrfile= snr_diminution=1.0
		"
	    ;;

    test7 )  test_string="celldetect log=no clobber=yes 
		infile=$INDIR/cell_SpB_evt.fits[EVENTS]
		expstk=  bkgfile=
                xoffset=INDEF yoffset=INDEF 
		bkgvalue=0.0 bkgerrvalue=0.0 
		outfile=$outfile
                psffile=@$INDIR/cell_SpB_evt_psfmap.lis 
		regfile=none maxlogicalwindow=2048 
		findpeaks=yes fixedcell=0 
		centroid=yes cellfile=$OUTDIR/cells.fits
		thresh=3  ellsigma=3.0 
		convolve=no snrfile=
		"
	    ;;

    test8 )  test_string="celldetect log=no clobber=yes 
             infile=$INDIR/acisf03226_000N020_evt3.fits
             expstk= bkgfile= 
             bkgvalue=0.0 bkgerrvalue=0.0 
             outfile=$outfile 
             psffile=@$INDIR/acisf03226_psfmap.lis
             regfile=none maxlogicalwindow=2048 
             findpeaks=yes fixedcell=0 
             centroid=yes cellfile=none thresh=4 
             convolve=no snrfile= snr_diminution=1.0"
           ;;

    # test8 and test9 are the same except for snr_diminution=0.8

    test9 )  test_string="celldetect log=no clobber=yes 
             infile=$INDIR/acisf03226_000N020_evt3.fits
             expstk= bkgfile= 
             bkgvalue=0.0 bkgerrvalue=0.0 
             outfile=$outfile 
             psffile=@$INDIR/acisf03226_psfmap.lis
             regfile=none maxlogicalwindow=2048 
             findpeaks=yes fixedcell=0 
             centroid=yes cellfile=none thresh=4 
             convolve=no snrfile= snr_diminution=0.8"
           ;;

    test_double ) test_string="celldetect log=no clobber=yes
		    infile=$INDIR/macis-10-0-0-10ksec_22_img.fits
                    psffile=$INDIR/macis-10-0-0-10ksec_22_psfmap.fits
		    outfile=$outfile thresh=1.8
		    findpeaks=yes centroid=yes
		    ellsigma=3    expratio=0
		    fixedcell=0   cellfile=
		    bkgfile=      bkgvalue=0
		    bkgerrvalue=0 convolve=no
		    snrfile= snr_diminution=1.0"
            ;;

    test_fixed_double ) test_string="celldetect log=no clobber=yes
                        infile='$INDIR/all_flt_acisf00613_000N001_evt1.fits.gz[EVENTS][bin x=3902:4302,y=3861:4261][opt type=i4]' 
                        outfile=$outfile fixedcell=15 
                        psffile=  expstk=none 
                        regfile=none thresh=10 snr_diminution=1 findpeaks=yes centroid=yes ellsigma=3 
                        expratio=0 xoffset=INDEF yoffset=INDEF cellfile=none maxlogicalwindow=8192 
                        bkgfile=none bkgvalue=0 bkgerrvalue=0 convolve=no snrfile=none verbose=0"
            ;;

    test_extreme ) test_string="celldetect
           infile='$INDIR/hrcEvt2.fits[sky=box(32737,32653,30000,15.5,0)]'
           outfile=$outfile maxlogicalwindow=2048 
           thresh=3 fixedcell=0
           findpeaks=yes centroid=yes ellsigma=3 expratio=0
           xoffset=INDEF yoffset=INDEF
           psffile=@$INDIR/hrcEvt2_psfmap.lis
           bkgvalue=0 bkgerrvalue=0 convolve=no
           expstk=none regfile=none cellfile=none bkgfile=none snrfile=none
           verb=0 log=no clobber=yes"
   ;;

   test_gradient ) test_string="celldetect log=no clobber=yes
		    infile=$INDIR/gradient_img.fits
                    psffile=$INDIR/gradient_psfmap.fits
		    outfile=$outfile
		    findpeaks=yes centroid=yes
		    ellsigma=3    expratio=0
		    fixedcell=0 cellfile=
		    bkgfile=      bkgvalue=0
		    bkgerrvalue=0 convolve=no
		    snrfile= thresh=4"
            ;;

    test_gradient_convolve ) test_string="celldetect log=no clobber=yes
		    infile=$INDIR/gradient_img.fits
                    psffile=$INDIR/gradient_psfmap.fits
		    outfile=$outfile
		    findpeaks=yes centroid=no
		    ellsigma=3    expratio=0
		    fixedcell=0   cellfile= 
		    bkgfile=      bkgvalue=0
		    bkgerrvalue=0 convolve=yes
		    snrfile=$OUTDIR/${testid}_snr.fits
		    thresh=4"
            ;;
    test_arcsec ) test_string="celldetect in=$INDIR/test_broad_thresh.img out=$outfile 
               psffile=$INDIR/test_broad_arcsec.psfmap fixed=0 expstk=@$INDIR/expmap.lis 
               mode=h clob+"
            ;;
    test_physical ) test_string="celldetect in=$INDIR/test_broad_thresh.img out=$outfile 
               psffile=$INDIR/test_broad_physical.psfmap fixed=0 expstk=@$INDIR/expmap.lis 
               mode=h clob+"
            ;;

    test_convolve_fixedcell) test_string="celldetect in=$INDIR/cell_SpB.fits out=$outfile 
             fixed=6 convolve=yes mode=h clob+ maxlogical=8192 snrfile=$OUTDIR/${testid}_snr.fits"
            ;;
    test_sin ) test_string="celldetect in=$INDIR/faked_sin.fits out=$outfile fixed=9 cl+"
            ;;
    test_rot ) test_string="celldetect in=$INDIR/fake_crota.img out=$outfile fixed=9 cl+"
            ;;
    test_nowcs ) test_string="celldetect in=$INDIR/nowcs.img out=$outfile fixed=9 cl+"
            ;;
    test_lin ) test_string="celldetect in=$INDIR/chip.img out=$outfile fixed=18 cl+"
            ;;
    test_lin2 ) test_string="celldetect in=$INDIR/chip.mod.img out=$outfile fixed=18 cl+"
            ;;
    test_sep) test_string="celldetect in=$INDIR/separate_linear_wcs.img out=$outfile fixed=18 cl+"
            ;;
  esac

  echo ""
  echo $test_string | tee -a $LOGFILE
  echo ""

  eval $test_string | tee -a $LOGFILE  2>&1

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  ####################################################################
  # check source list
  echo ""
  echo "dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
  dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  if test $testid = "test1" ; then

    ##################################################################
    # check SNR image
    echo "" 

    echo "dmdiff $OUTDIR/cell_SpB_cell_1.fits $SAVDIR/cell_SpB_cell_1.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_cell_1.fits $SAVDIR/cell_SpB_cell_1.fits \
      tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_cell_1.fits" >> $LOGFILE
      mismatch=0
    fi
  fi

  if test $testid = "test2" ; then

    ##################################################################
    # check SNR image
    echo "" 
    echo "dmdiff $OUTDIR/test2_snr.fits $SAVDIR/test2_snr.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/test2_snr.fits $SAVDIR/test2_snr.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/test2_snr.fits" >> $LOGFILE
      mismatch=0
    fi

    echo "dmdiff $OUTDIR/cell_SpB_convolve_cell_1.fits $SAVDIR/cell_SpB_convolve_cell_1.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_convolve_cell_1.fits $SAVDIR/cell_SpB_convolve_cell_1.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_convolve_cell_1.fits" >> $LOGFILE
      mismatch=0
    fi
  fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  if test $testid = "test7" ; then

    ##################################################################
    # check SNR image

    echo "dmdiff $OUTDIR/cell_SpB_evt_cell_1.fits $SAVDIR/cell_SpB_evt_cell_1.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_evt_cell_1.fits $SAVDIR/cell_SpB_evt_cell_1.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_evt_cell_1.fits" >> $LOGFILE
      mismatch=0
    fi

    echo "dmdiff $OUTDIR/cell_SpB_evt_cell_2.fits $SAVDIR/cell_SpB_evt_cell_2.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_evt_cell_2.fits $SAVDIR/cell_SpB_evt_cell_2.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_evt_cell_2.fits" >> $LOGFILE
      mismatch=0
    fi

    echo "dmdiff $OUTDIR/cell_SpB_evt_cell_4.fits $SAVDIR/cell_SpB_evt_cell_4.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_evt_cell_4.fits $SAVDIR/cell_SpB_evt_cell_4.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_evt_cell_4.fits" >> $LOGFILE
      mismatch=0
    fi

    echo "dmdiff $OUTDIR/cell_SpB_evt_cell_8.fits $SAVDIR/cell_SpB_evt_cell_8.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_evt_cell_8.fits $SAVDIR/cell_SpB_evt_cell_8.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_evt_cell_8.fits" >> $LOGFILE
      mismatch=0
    fi

    echo "dmdiff $OUTDIR/cell_SpB_evt_cell_16.fits $SAVDIR/cell_SpB_evt_cell_16.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE"
    dmdiff $OUTDIR/cell_SpB_evt_cell_16.fits $SAVDIR/cell_SpB_evt_cell_16.fits tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      
    if  test $? -ne 0 ; then
      echo "ERROR: HEADER MISMATCH in $OUTDIR/cell_SpB_evt_cell_16.fits" >> $LOGFILE
      mismatch=0
    fi


  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
