#!/usr/bin/env perl
# 
#  Copyright (C) 2004-2009,2012,2016-2017,2021-2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '023';
my $semantic = 'asciiCopyTo';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";
$ENV{'LC_ALL'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,    # Event table w/nulls           FITS      to RAW
2,    #   <same>                                to SIMPLE
3,    #   <same>                                to DTF
4,    #   <same>                                to DTF-FIXED
5,    #   <same>                                to FITS
6,    # Custom table                  SIMPLE    to RAW
7,    #   <same>                                to SIMPLE
8,    # Event table                   SIMPLE    to DTF
9,    #   <same>                                to DTF-FIXED
10,   #   <same>                                to FITS
11,   # Event table                   DTF       to RAW
12,   #   <same>                                to SIMPLE
13,   #   <same>                                to DTF
14,   #   <same>                                to DTF-FIXED
15,   #   <same>                                to FITS
16,   # Event table                   DTF-FIXED to RAW
17,   #   <same>                                to SIMPLE
18,   #   <same>                                to DTF
19,   #   <same>                                to DTF-FIXED
20,   #   <same>                                to FITS
21,   # Event table                   FITS      to RAW
22,   #   <same>                                to SIMPLE
23,   #   <same>                                to DTF
24,   #   <same>                                to DTF-FIXED
25,   #   <same>                                to FITS
26,   # Test table - NULLSTR option usage
27,   # Event table w/nulls           FITS      to CSCRDB
28,   # Custom table                  SIMPLE    to CSCRDB
29,   # Event table                   DTF       to CSCRDB
30,   #   <same>                      DTF-FIXED to CSCRDB
31,   #   <same>                      FITS      to CSCRDB
32,   # (indt022m018.cscrdb)          CSCRDB    to RAW
33,   #   <same>                                to SIMPLE
34,   # (indt022m019.cscrdb)          CSCRDB    to DTF
35,   #   <same>                                to DTF-FIXED
36,   #   <same>                                to FITS
37,   #   <same>                                to CSCRDB
38,   # Test image [2D_I2_SCALED]     DTF       to FITS
39,   #   <same>   [2D_I2_SCALED]     FITS      to DTF
40,   # Test table [SCALAR]           DTF       to FITS
41,   #   <same>   [SCALAR]           FITS      to DTF
42,   # Test image [2D_I2_BASIC]      DTF       to FITS
43,   #   <same>   [2D_I2_BASIC]      FITS      to DTF
44,   #   <same>   [2D_UI2_BASIC]     DTF       to FITS
45,   #   <same>   [2D_UI2_BASIC]     FITS      to DTF
46,   #   <same>   [2D_I4_BASIC]      DTF       to FITS
47,   #   <same>   [2D_I4_BASIC]      FITS      to DTF
48,   #   <same>   [2D_UI4_BASIC]     DTF       to FITS
49,   #   <same>   [2D_UI4_BASIC]     FITS      to DTF
50,   #   <same>   [2D_I8_BASIC]      DTF       to FITS
51,   #   <same>   [2D_I8_BASIC]      FITS      to DTF
52,   #   <same>   [2D_UI8_BASIC]     DTF       to FITS
53,   #   <same>   [2D_UI8_BASIC]     FITS      to DTF 
54,   # Custom img [2D_IMAGE_NOSPEC]  DTF       to DTF 
55,   # Test table [BigNames]         DTF       to FITS
56,   #   <same>   [BigNames]         FITS      to DTF
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 1 => { "text" => [] },
 2 => { "text" => [] },
 3 => { "text" => [] },
 4 => { "text" => [] },
 5 => { "table" => [ 1, 2 ] },
 6 => { "text" => [] },
 7 => { "text" => [] },
 8 => { "text" => [] },
 9 => { "text" => [] },
 10 => { "table" => [ 1, 2 ] },
 11 => { "text" => [] },
 12 => { "text" => [] },
 13 => { "text" => [] },
 14 => { "text" => [] },
 15 => { "table" => [ 1, 2 ] },
 16 => { "text" => [] },
 17 => { "text" => [] },
 18 => { "text" => [] },
 19 => { "text" => [] },
 20 => { "table" => [ 1 ] },
 21 => { "text" => [] },
 22 => { "text" => [] },
 23 => { "text" => [] },
 24 => { "text" => [] },
 25 => { "table" => [ 1 ] },
 26 => { "text" => [] },
 27 => { "text2" => [] },
 28 => { "text2" => [] },
 29 => { "text2" => [] },
 30 => { "text2" => [] },
 31 => { "text2" => [] },
 32 => { "text" => [] },
 33 => { "text" => [] },
 34 => { "text" => [] },
 35 => { "text" => [] },
 36 => { "table" => [ 2 ] },
 37 => { "text2" => [] },
 38 => { "list" => [] },
 39 => { "list" => [] },
 40 => { "table" => [ 2 ] },
 41 => { "text" => [] },
 42 => { "table" => [ 1 ] },
 43 => { "text" => [] },
 44 => { "table" => [ 1 ] },
 45 => { "text" => [] },
 46 => { "table" => [ 1 ] },
 47 => { "text" => [] },
 48 => { "table" => [ 1 ] },
 49 => { "text" => [] },
 50 => { "text" => [] },
 51 => { "text" => [] },
 52 => { "text" => [] },
 53 => { "text" => [] },
 54 => { "text" => [] },
 55 => { "table" => [ 2 ] },
 56 => { "text" => [] },
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
#  ** Standard **
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
 'HISTORY.*\n' => "",
 'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\t' => "NAN\t",
 'nan ' => "NAN "
);

my %test_rdb_words = (
# ** CSCRDB files have different HISTORY line record format.. **
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
 '# HISTORY.*\n' => "",
 'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\t' => "NAN\t",
 'nan ' => "NAN "
);

		     
# Find Tool Executable in PATH
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

#########################################################################
# Loop over all tests
foreach my $testid (@testlist)
{
   # Init per-test variables.
   my $mismatch = 0;
   my $time_user;
   my $time_sys;
   $err_msg = "";

   # delete old outputs
   my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
   system("$RM -f $testout/${basename}*");

   # PROGRAMMER - assign filenames here
   # Each test must assign the appropriate input/output extensions.
   my $infile = "$testin/in${basename}";
   my $outfile = "$testout/${basename}";
   my $savfile = "$testsav/${basename}";
   
   my $outfile_filt = "$testout/${basename}_filt";
   my $savfile_filt = "$testout/${basename}_savfil";
   my $test_diff = "$testout/${basename}.diff";
   
   # --------------------------------------------------------------------------------
   # set test command
   my $test_string = "";

   if ( $testid == 1 )
   {
      # copy FITS event file to RAW file
      $infile = "${infile}.fits";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 2 )
   {
      # copy FITS event file to SIMPLE file
      $infile = "${infile}.fits";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 3 )
   {
      # copy FITS event file to DTF file
      $infile = "${infile}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 4 )
   {
      # copy FITS event file to DTF-FIXED file
      $infile = "${infile}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   } 
   elsif ( $testid == 5 )
   {
      # copy from RAW to FITS
      $infile = "${infile}.raw";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 6 )
   {
      # copy from SIMPLE to RAW
      $infile = "${infile}.simple";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 7 )
   {
      # copy from SIMPLE to SIMPLE
      $infile = "${infile}.simple";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 8 )
   {
      # copy from SIMPLE to DTF
      $infile = "${infile}.simple";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 9 )
   {
      # copy from SIMPLE to DTF-FIXED
      $infile = "${infile}.simple";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   elsif ( $testid == 10 )
   {
      # copy from SIMPLE to FITS
      $infile = "${infile}.simple";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 11 )
   {
      # copy from DTF to RAW
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 12 )
   {
      # copy from DTF to SIMPLE
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 13 )
   {
      # copy from DTF to DTF
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 14 )
   {
      # copy from DTF to DTF-FIXED
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   elsif ( $testid == 15 )
   {
      # copy from DTF to FITS
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 16 )
   {
      # copy from DTF-FIXED to RAW
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 17 )
   {
      # copy from DTF-FIXED to SIMPLE
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 18 )
   {
      # copy from DTF-FIXED to DTF
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 19 )
   {
      # copy from DTF-FIXED to DTF-FIXED
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   elsif ( $testid == 20 )
   {
      # copy from DTF-FIXED to FITS
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 21 )
   {
      # copy from FITS to RAW
      $infile = "${infile}.fits";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 22 )
   {
      # copy from FITS to SIMPLE
      $infile = "${infile}.fits";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 23 )
   {
      # copy from FITS to DTF
      $infile = "${infile}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 24 )
   {
      # copy from FITS to DTF-FIXED
      $infile = "${infile}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   elsif ( $testid == 25 )
   {
      # copy from FITS to FITS
      $infile = "${infile}.fits";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 26 )
   {
      # copy from DTF with NULLSTR option to RAW with different NULLSTR
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile'[opt NULLSTR=-]' outfile=$outfile'[opt kernel=text/raw,NULLSTR=NULL]'";
   }
   elsif ( $testid == 27 )
   {
      # copy from FITS to CSCRDB file
      $infile = "${infile}.fits";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 28 )
   {
      # copy from SIMPLE to CSCRDB
      $infile = "${infile}.simple";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 29 )
   {
      # copy from DTF to CSCRDB
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 30 )
   {
      # copy from DTF-FIXED to CSCRDB
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 31 )
   {
      # copy from FITS to CSCRDB
      $infile = "${infile}.fits";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 32 )
   {
      # copy from CSCRDB to RAW
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   elsif ( $testid == 33 )
   {
      # copy from CSCRDB to SIMPLE
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   elsif ( $testid == 34 )
   {
      # copy from CSCRDB to DTF
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 35 )
   {
      # copy from CSCRDB to DTF-FIXED
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   elsif ( $testid == 36 )
   {
      # copy from CSCRDB to FITS
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 37 )
   {
      # copy from CSCRDB to CSCRDB
      $infile = "${infile}.cscrdb";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   elsif ( $testid == 38 )
   {
      # SL-35 Test Plan - 7.1.1 copy a scaled DTF image [2D_I2_SCALED] to FITS
      $infile = "${infile}.dtf";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=${infile}\'[2D_I2_SCALED]\' outfile=$outfile kernel='FITS'";
   }
   elsif ( $testid == 39 )
   {
      # SL-35 Test Plan - 7.2.1 copy from scaled FITS image [2D_I2_SCALED] to DTF
      $infile = "${infile}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile\'[2D_I2_SCALED]\' outfile=$outfile kernel='TEXT/DTF'";
   }
   elsif ( $testid == 40 )
   {
     # Table round-robin: part 1 = DTF -> FITS
     #  o simple TABLE, all datatypes in keys/cols, SCALAR values
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[SCALARS]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 41 )
   {
     # Table round-robin: part 2 = FITS -> DTF
     #  o simple TABLE, all datatypes in keys/cols, SCALAR values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[SCALARS]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 42 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - SHORT values
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[2D_I2_BASIC]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 43 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - SHORT values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_I2_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 44 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - USHORT values
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[2D_UI2_BASIC]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 45 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - USHORT values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_UI2_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 46 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - LONG values
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[2D_I4_BASIC]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 47 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - LONG values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_I4_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 48 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - ULONG values
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[2D_UI4_BASIC]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 49 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - ULONG values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_UI4_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 50 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - LONGLONG values
     #  o DMLIST/DMDIFF do not support [U]LONGLONG images at this time..
     $infile = "${infile}.dtf";
     my $tmpfile = "$testout/${basename}.fits"; # the actual outfile
     $outfile = "${outfile}.dtf";               # what we can compare
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=${infile}'\[2D_I8_BASIC]\' outfile=${tmpfile} kernel='fits' ; $DMCOPY ${tmpfile} ${outfile}'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 51 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - LONGLONG values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_I8_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 52 )
   {
     # Image round-robin: part 1 = DTF -> FITS
     #  o simple IMAGE - ULONGLONG values
     #  o DMLIST/DMDIFF do not support [U]LONGLONG images at this time..
     $infile = "${infile}.dtf";
     my $tmpfile = "$testout/${basename}.fits"; # the actual output file 
     $outfile = "${outfile}.dtf";               # what we can compare
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=${infile}'\[2D_UI8_BASIC]\' outfile=${tmpfile} kernel='fits' ; $DMCOPY ${tmpfile} ${outfile}'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 53 )
   {
     # Image round-robin: part 2 = FITS -> DTF
     #  o simple IMAGE - ULONGLONG values
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_UI8_BASIC]\' outfile=${outfile} kernel='text/dtf'";
   }
   elsif ( $testid == 54 )
   {
     # Handling of DTF image with no AXIS info
     #  o ascii kernel should derive image size from data
     $infile  = "${infile}.dtf";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[2D_IMAGE_NOSPEC\]' outfile=${outfile} kernel='text/dtf' ";
   }
   elsif ( $testid == 55 )
   {
     # Table round-robin: part 1 = DTF -> FITS
     #  o TABLE containing columns/vectors with long names (>80 chars)
     $infile = "${infile}.dtf";
     $outfile = "${outfile}.fits";
     $savfile = "${savfile}.fits";
     $test_string = "$DMCOPY infile=${infile}'\[BigNames]\' outfile=${outfile} kernel='fits'";
   }
   elsif ( $testid == 56 )
   {
     # Table round-robin: part 2 = FITS -> DTF
     #  o TABLE containing columns/vectors with long names (>80 chars)
     $infile = "${infile}.fits";
     $outfile = "${outfile}.dtf";
     $savfile = "${savfile}.dtf";
     $test_string = "$DMCOPY infile=$infile'\[BigNames]\' outfile=${outfile} kernel='text/dtf'";
   }
   $test_string .= " >> $logfname 2>&1";

   # --------------------------------------------------------------------------------
   # run the test
   dt_print_test_header( $testid, $test_string, $logfile, $printfile );
   close $logfile;
   
   ($rc, $time_user, $time_sys) = dt_run_test($test_string);
   
   open $logfile, ">>$logfname";
   select($logfile); $| = 1; select($printfile);
   unless ( $rc == 0 )
   {
      $err_msg = "ERROR: $toolname failed to run\n";
   }
   
   # --------------------------------------------------------------------------------
   # validate results
   my $block_typesref = $test_blocks{$testid};
   my %block_types = %$block_typesref;
   if ( $block_types{"table"} )
   {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"image"} )
   {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"list"} )
   {
      $err_msg .= dmlist_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \@tolkeys, $logfname, $test_diff, "" );
   }
   if ( $block_types{"text"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_words, $logfname, $test_diff );
   }
   if ( $block_types{"text2"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_rdb_words, $logfname, $test_diff );
   }

   # --------------------------------------------------------------------------------
   # report results
   unless ( $err_msg eq "" )
   {   
      push( @faillist, $testid );
      $mismatch = 1;
   }
   dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
   $time_user_total += $time_user;
   $time_sys_total  += $time_sys;

}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
