#!/usr/bin/env perl
# 
#  Copyright (C) 2018-2020,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '102';
my $semantic = 'Kernel';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################

my @testlist = 
( 
# PROGRAMMER - List the test ids here
1,    # FITS Kernel unit tests: ftkerneltest.c
2,    # ASCII Kernel unit tests: afkerneltest.c
#4,    # Kernel DatasetOpen tests: kernelDatasetOpenTest.c ### COMMENTED OUT UNTIL READY TO WORK ON DATASETOPEN FUNCTIONS
#41,   # ASCII Kernel Thread test- dtf: afkernelThreadTest.c
#42,   # ASCII Kernel Thread test- raw: afkernelThreadTest.c
#43,   # ASCII Kernel Thread test- sim: afkernelThreadTest.c
#44,   # ASCII Kernel Thread test- tsv: afkernelThreadTest.c
#45,   # ASCII Kernel Thread test- rdb: afkernelThreadTest.c
);

my %toolnames =
(
# PROGRAMMER - put tool name here
 1 => "ftkerneltest",
 2 => "afkerneltest",
# 4 => "kernelDatasetOpenTest",
#41 => "afkernelThreadTest",
#42 => "afkernelThreadTest",
#43 => "afkernelThreadTest",
#44 => "afkernelThreadTest",
#45 => "afkernelThreadTest",
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
1 => { "pass" => [] },
2 => { "pass" => [] },
#4  => { "text" => []},
#41 => { "text" => []},
#42 => { "text" => []},
#43 => { "text" => []},
#44 => { "text" => []},
#45 => { "text" => []},
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
# $version => "ASCDS_VERSION"
);
             
# Set parameters to defaults
#system( "$PUNLEARN $toolname" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;
my @flavors = ("none", "DTF", "RAW", "SIMPLE", "TSV", "RDB" );

#########################################################################
# Loop over all tests
foreach my $testid (@testlist)
{
  # Init per-test variables.
  my $mismatch = 0;
  my $time_user;
  my $time_sys;
  $err_msg = "";

  # delete old outputs
  my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
  system("$RM -f $testout/${basename}*");

  my $block_typesref = $test_blocks{$testid};
  my %block_types = %$block_typesref;

  my $infile = "$testin/in${basename}.txt";
  # PROGRAMMER - put output filename here
  my $outfile = "$testout/${basename}.txt";
  # PROGRAMMER - put saved output filename here
  my $savfile = "$testsav/${basename}.txt";
  # PROGRAMMER - put name of diff file here, if needed
  my $test_diff = "$testout/${basename}.diff";

  # verify test executable exists
  my $toolname = $toolnames{$testid};
  my $TESTEXEC = find_inpath( $toolname, $ENV{'PATH'}, ':' );

  # --------------------------------------------------------------------------------
  # set test command
  my $test_string = "";

  if ( $testid == 1 )
  {
     $test_string = "$TESTEXEC > $outfile 2>&1";
  }
  elsif ( $testid == 2 )
  {
     $test_string = "$TESTEXEC > $outfile 2>&1";
  }
  elsif ( $testid == 4 )
  {
     $test_string = "$TESTEXEC ${testin} ${testout} filterTestFile > $outfile 2>&1";
  }
  elsif ( (sort {$a <=> $b} 39, 50, $testid)[1] == $testid ) # testid range(40:49)
  {
     my $ndx = $testid - 40;
     $test_string = "$TESTEXEC $infile $outfile $flavors[$ndx] >> $logfname 2>&1";
  }

  # --------------------------------------------------------------------------------
  # run the test
  dt_print_test_header( $testid, $test_string, $logfile, $printfile );
  close $logfile;
  ($rc, $time_user, $time_sys) = dt_run_test($test_string);

  if ( $block_types{"pass"} )
  { # Pass/Fail test.
    if ( $rc != 0 )
    {
    $err_msg = "  ERROR: Test returned failure status.\n";
    } 
    $rc = 0; 
  }

  open $logfile, ">>$logfname";
  select($logfile); $| = 1; select($printfile);
  unless ( $rc == 0 )
  {
     $err_msg = "ERROR: $toolname failed to run\n";
  }
  
  # --------------------------------------------------------------------------------
  # validate results
  if ( $block_types{"table"} )
  {
     my @blocks = @{$block_types{"table"}};
     foreach my $block ( @blocks )
     {
     $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
     }
  }
  if ( $block_types{"image"} )
  {
     my @blocks = @{$block_types{"image"}};
     foreach my $block ( @blocks )
     {
     $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
     }
  }
  if ( $block_types{"list"} )
  {
     $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
  }
  if ( $block_types{"text"} )
  {
     $err_msg .= diff_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \%test_words, $logfname, $test_diff );
  }

  # --------------------------------------------------------------------------------
  # report results
  unless ( $err_msg eq "" )
  {
     push( @faillist, $testid );
     $mismatch = 1;
  }
  dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
  $time_user_total += $time_user;
  $time_sys_total  += $time_sys;
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
