#! /bin/sh

# 07 March 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmcoords.t
# test script for dmcoords


# !!2
# syntax:
# dmcoords.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, COMMENT, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, DATE, and Dataset
# substitute  $ASCDS_VERSION => VERSION
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e "s/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g;
  s/${ASCDS_VERSION}/VERSION/g" | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
  punlearn $1
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmcoords"

# set up list of tests
# !!4
alltests="test1 test2 test3 test4 test5 test6 test7 test8 test9 test10 test11 test12 test13 test14 test_noinfile test_hrc1 test_hrc2     test_noinfile_2  test_noinfile_3 test_noinfile_4 acishetg acishetg_2 nonChandra_1 nonChandra_2 nonChandra_3 nonChandra_4 test4_img test9_B test9_C nonChandra_5 nonChandra_6 noCALDB"

# "short" test to run
# !!5
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
# punlearn $toolname
# pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop
caldb=$CALDB
caldb_tmp="${caldb}_TMP"

for testid in $testlist
do
  punlearn ${toolname}
  pset $toolname verbose=$bose
    
  # delete old outputs
  rm -f $OUTDIR/${testid}.lis*

  # Set up file names
  outfile=$OUTDIR/${testid}.lis
  savfile=$SAVDIR/${testid}.lis

  echo "" | tee -a $LOGFILE
  echo "running ${testid}" | tee -a $LOGFILE
  echo "" >> $LOGFILE
  ####################################################################
  # run the tool


  #
  # Need to force tool to run with verbose=1 to get values
  #

  case ${testid} in
    # !!6
    test1 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=cel ra=95.792 dec=-64.671 celfmt=deg verbose=1 > $outfile"
            ;;

    # !!7
    test2 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=chip chipx=512.0 chipy=512.0 chip_id=7 verbose=1 > $outfile"
           ;;

    # !!8
    test3 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=cel ra=06:23:12 dec=-64:40:12 celfmt=hms verbose=1 > $outfile"
           ;;

    # !!9
    test4 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits'[bin x=4,y=4]' asol= opt=logical logicalx=210.0 logicaly=200.0 verbose=1 > $outfile"
           ;;

    # ref. test4 ; infile is image; 
    test4_img ) test_string="dmcoords infile=$INDIR/img_chandra.fits asol= opt=logical logicalx=210.0 logicaly=200.0 verbose=1 > $outfile"
           ;;

    test5 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=msc theta=0 phi=0 verbose=1 > $outfile"
           ;;

    test6 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=chip sim='0.0 0.0 -190.2' chipx=512.0 chipy=512.0 chip_id=7 verbose=1 > $outfile"
           ;;

    test7 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=msc theta=5.0 phi=220.0 verbose=1 > $outfile"
           ;;

    test8 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=chip chipx=0.5 chipy=0.5 chip_id=8 verbose=1 > $outfile"
           ;;

    test9 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=sky x=4090.0 y=4220.0 verbose=1 > $outfile"
           ;;

    #- ref. test9. infile TELESCOPE=chandra and doesn't have INSTRUMENT & GRATING. 
    #       it's chandra data.
    test9_B ) test_string="dmcoords infile=$INDIR/noInstGrt.fits asol= opt=sky x=4090.0 y=4220.0 verbose=1 > $outfile"
           ;;

    #- ref. test9_B. changed the keys RA/DEC/ROLL_NOM slightly differerent from WCS.
    test9_C ) test_string="dmcoords infile=$INDIR/nomKeyUpd.fits asol= opt=sky x=4090.0 y=4220.0 verbose=1 > $outfile"
           ;;

    test10 ) test_string="dmcoords infile=$INDIR/dmcoords1.fits opt=chip sim='0.0 0.0 -190.2' chipx=512.0 chipy=512.0 chip_id=7 verbose=1 asolfile=@$INDIR/asol.stk displace='1 2 3.3 2 3 4' > $outfile"
           ;;
    test11 ) test_string="dmcoords infile=$INDIR/dmcoords2.fits opt=chip sim='0.0 0.0 -190.2' chipx=512.0 chipy=512.0 chip_id=7 verbose=1 asolfile=none displace='1 2 3.3 2 3 4' > $outfile"
           ;;

    test12 ) test_string="dmcoords infile=$INDIR/acisf14661_001N002_evt1a.fits.gz op=chip chip_id=8 chipx=152 chipy=223 grating=heg order=1 ra_asp=257.2862649278 dec_asp=-36.4070576224 roll_asp=252.6437917696 ra_zo=257.2817923311092 dec_zo=-36.40711661793967 verb=1 > $outfile"
	   ;; 

    test13 ) test_string="dmcoords infile=$INDIR/acisf14661_001N002_evt1a.fits.gz op=sky x=3727.386718750 y=5055.050781250 grating=heg order=1 ra_asp=257.2862649278 dec_asp=-36.4070576224 roll_asp=252.6437917696 ra_zo=257.2817923311092 dec_zo=-36.40711661793967 verb=1 energy=3.141592 > $outfile" 
	   ;;

    test14 ) test_string="dmcoords infile=$INDIR/acisf14661_001N002_evt1a.fits.gz op=chip chip_id=8 chipx=152 chipy=223 grating=heg order=2 ra_asp=257.2862649278 dec_asp=-36.4070576224 roll_asp=252.6437917696 ra_zo=257.2817923311092 dec_zo=-36.40711661793967 verb=1 > $outfile"
 	   ;; 

    # infile=none.  parameter grating=none. output Grating=NONE
    test_noinfile ) test_string="dmcoords infile=none asol= option=cel ra=98.94 dec=-75.27 celfmt=deg detector=ACIS grating=NONE sim='-0.6394396079644659 0 -190.1199515274594' ra_nom=98.93359 dec_nom=-75.26487 roll_nom=131.674691 verb=1 > $outfile"
           ;;

    # ref. test_noinfile.  parameter grating='' or not specified (by default). 
    #      output Grating=HEG ( by default).
    test_noinfile_2 ) test_string="dmcoords infile=none asol= option=cel ra=98.94 dec=-75.27 celfmt=deg detector=ACIS grating='' sim='-0.6394396079644659 0 -190.1199515274594' ra_nom=98.93359 dec_nom=-75.26487 roll_nom=131.674691 verb=1 > $outfile"
           ;;

    # ref. the example#4 in the tool's help file.  parameter grating not specified.
    #      output Grating=HEG by default
    test_noinfile_3 ) test_string="dmcoords infile=none asolfile=none opt=det detx=4963.0 dety=4012.0 sim='0.6 0.0 -192.3' detector=acis celfmt=deg ra_nom=96.1043 dec_nom=-0.0243 roll_nom=212.3 ra_asp=')ra_nom' dec_asp=')dec_nom' verbose=1  > $outfile"
           ;;

    # ref. test_noinfile_3.  parameter grating=NONE
    #      output Grating=HEG by default
    test_noinfile_4 ) test_string="dmcoords infile=none asolfile=none opt=det detx=4963.0 dety=4012.0 sim='0.6 0.0 -192.3' detector=acis celfmt=deg ra_nom=96.1043 dec_nom=-0.0243 roll_nom=212.3 ra_asp=')ra_nom' dec_asp=')dec_nom' verbose=1  grating=none > $outfile"
           ;;

    # ref. test_noinfile ;  set the parameter grating=''; 
    test_hrc1 ) test_string="dmcoords infile=$INDIR/hrc_evt.fits asol= op=sky x=16593 y=16399 verb=1 > $outfile"
           ;;

    test_hrc2 ) test_string="dmcoords infile=$INDIR/hrc_evt.fits asol= op=chip chipx=7321 chipy=7723 chip_id=0 verb=1 > $outfile"
           ;;

   # user's grating=""; infile's grating="hetg"; 
   # output-grating="HEG"+other grating stuff
    acishetg ) test_string="dmcoords infile=$INDIR/acishetg_evt1.fits asol=  opt=cel ra=79.1796 dec=46.0026 celfmt=deg verbose=1 > $outfile"
           ;;

   # user's grating="NONE"; infile's grating="hetg";
   # output-grating="NONE" and no other grating stuff  except grating-order=0
   acishetg_2 ) test_string="dmcoords infile=$INDIR/acishetg_evt1.fits asol=  opt=cel ra=79.1796 dec=46.0026 celfmt=deg verbose=1 grating=NONE  > $outfile"
           ;;

   # ref. test9;  infile's telescop=not-chandra (modified from dmcoords.fits) ; 
   #
    nonChandra_1 ) test_string="dmcoords infile=$INDIR/tbl_nonChandra_A.fits asol= opt=sky x=4090.0 y=4220.0 verbose=1 > $outfile"
           ;;

   # ref. acishetg;  infile's telescop=not-chandra (modified from acishetg_evt1.fits) ;
   #
    nonChandra_2 ) test_string="dmcoords infile=$INDIR/tbl_nonChandra_B.fits asol=  opt=cel ra=79.1796 dec=46.0026 celfmt=deg verbose=1 > $outfile"
           ;;

   # ref. test4;  infile's telescop=not-chandra (modified from dmcoords.fits) ;
    nonChandra_3 ) test_string="dmcoords infile=$INDIR/tbl_nonChandra_A.fits'[bin x=4,y=4]' asol= opt=logical logicalx=210.0 logicaly=200.0 verbose=1 > $outfile"
           ;;

   # ref. test4_img & nonChandra_3;  infile is image ( modified from img_chandra.fits to set  telescop=not-chandra; )
    nonChandra_4 ) test_string="dmcoords infile=$INDIR/img_nonChandra.fits asol= opt=logical logicalx=210.0 logicaly=200.0 verbose=1 > $outfile"
           ;;

    #- ref. test9_B. infile doesn't have TELESCOPE, INSTRUMENT & GRATING. 
    nonChandra_5 ) test_string="dmcoords infile=$INDIR/noTel.fits asol= opt=sky x=4090.0 y=4220.0 verbose=1 > $outfile"
           ;;

    #- infile doesn't have TELESCOPE, INSTRUMENT, GRATING and RA/DEC/ROLL_NOM
    nonChandra_6 ) test_string="dmcoords infile=$INDIR/ds9.fits asol= op=sky x=4096 y=4096 verb=1 > $outfile"
           ;;

    #SL-193: dmcoords shouldn't segv if caldb can't be found, but this test should fail. set caldb to bogus path temporarily
    noCALDB )
       test_string="dmcoords infile=$INDIR/dmcoords1.fits asol= opt=cel ra=95.792 dec=-64.671 celfmt=deg verbose=1 > $outfile  2>&1"
       ;;

  esac

  #echo $test_string | tee -a  $LOGFILE 
  echo $test_string >> $LOGFILE 
  if [ ${testid} = "noCALDB" ] ; then
       export CALDB="$caldb_tmp"
       #eval $test_string | tee -a  $LOGFILE  2>&1
       eval $test_string >> $LOGFILE  2>&1
       export CALDB="$caldb"
  else
       #eval $test_string | tee -a  $LOGFILE  2>&1
      eval $test_string >> $LOGFILE  2>&1
  fi      


  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  # dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  # dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #           2>>$LOGFILE

  # compare
  # !!12
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #      /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
   keyfilter $outfile  ${outfile}.dmp2
   keyfilter $savfile  ${outfile}.dmp2_std
   diff  ${outfile}.dmp2 ${outfile}.dmp2_std > /dev/null 2>>$LOGFILE
   if  test $? -ne 0 ; then
     echo "ERROR: TEXT MISMATCH in $outfile" >> $LOGFILE
     mismatch=0
   fi

  ####################################################################
  # Did we get an error?
  #echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "  ${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "  ${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
