#! /bin/sh

# 06 March 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmextract.t
# test script for dmextract


# !!2
# syntax:
# dmextract.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}


######################################################################
# Initialization

# !!3
toolname="dmextract"

# set up list of tests
# !!4
alltests="1 2 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 23b 24 25 26 27 28 29a 29b 30a 30b 31a 31b 32a 32b 32c 32d 32e 33a 33b 33c 33d 33e 34b 34a 35a 36a 37a _hrc_a _hrc_b _dbl_img _leak _weighta _weightb _invalid_filt 38 _invalid_mask 39a 39b 40a 40b 41"

# 4/7 removed '3' because of stacks

# "short" test to run
# !!5
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  punlearn $toolname
  pset $toolname verbose=$bose
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/dmextract${testid}.out
  savfile=$SAVDIR/dmextract${testid}.out

  echo "" | tee -a $LOGFILE
  echo "running test $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
# TEST #1: dmextract a single file
   1 ) test_string="dmextract infile='$INDIR/dmextract1.fits[bin pha=500:3000:50]' outfile=$outfile error=gaussian verbose=0  clobber=yes"
            ;;

    # !!7
# TEST #2: dmextract another single file
   2 ) test_string="dmextract infile='$INDIR/dmextract2.fits[bin pha=0:20000:400][sky=circle(4700,2600,200)]' outfile=$outfile error=gaussian clobber=yes"
           ;;

    # !!8
# TEST #3: a list of 2 input files into a list of 2 (type 1 pha) output files
   3 ) test_string="dmextract infile='@$INDIR/inlist[bin pha=20:150:10]' outfile=@$OUTDIR/outlist error=gaussian clobber=yes"
           cp $INDIR/outlist $OUTDIR/outlist
           outfile=@$OUTDIR/outlist
           savfile=@$OUTDIR/outlist
           ;;

    # !!9
# TEST #4: dmextract a list of 2 input files into 1 (type 2 pha) output file
   4 ) test_string="dmextract infile='@$INDIR/inlist[bin X=200:700:50]' outfile=$outfile error=gaussian clobber=yes"
           ;;

    # !!9
# TEST #5: dmextract a list of reals into 1 type 1 pha output file using
#          nonintegral values for minbin, maxbin, and stepsize
   5 ) test_string="dmextract infile='$INDIR/dmextract3b.fits[bin TIME=1606800.3:1606804.9:0.7]' outfile=$outfile opt=ltc1  error=gaussian clobber=yes"
           ;;


    # !!9
# TEST #6: dmextract the source counts from a single region using poisson (gehrels) 
#errors and no background subtraction. Input is image.
   6 ) test_string="dmextract infile='$INDIR/dmextract6.fits[bin sky=circle(4233.5,3753.5,18)]' outfile=$outfile bkg=none error=gehrels clobber=yes"
           ;;

    # !!9
# TEST #7: dmextract the source counts from a single region with background
# subtraction ON THE SAME REGION(should get zero net counts). Input is an
# image.
   7 ) test_string="dmextract infile='$INDIR/dmextract7.fits[bin sky=circle(4233.5,3753.5,18)]' outfile=$outfile bkg='$INDIR/dmextract7.fits[bin sky=circle(4233.5,3753.5,18)]' error=gehrels bkgerror=gehrels clobber=yes"
           ;;

    # !!9
# TEST #8: dmextract a list of source regions with a singluar background.
#          USE AN EVENT FILE.
   8 ) test_string="dmextract infile='$INDIR/dmextract8.fits[bin sky=@$INDIR/dmextract8.lis]' outfile=$outfile bkg='$INDIR/dmextract8.fits[bin sky=circle(4096,4096,20)]' error=gehrels bkgerror=gehrels clobber=yes"
           ;;

    # !!9
# TEST #9: Produce a wmap while doing a pha extractions
   9 ) test_string="dmextract infile='$INDIR/dmextract9.fits[sky=circle(4000.7799,3957.4344,30.14439)][bin pi=1:1024:1]' outfile=$outfile error=gaussian wmap='det=32' clobber=yes"
           ;;

    # !!9
# TEST #10: check on rectangular regions
   10 ) test_string="dmextract infile='$INDIR/dmextract10.fits[bin sky=@$INDIR/dmextract10.reg]' outfile=$outfile bkg=none error=gehrels clobber=yes"

           ;;

    # !!9
# TEST #11: from fap (test 12): verify bugfix 7104 - proper calculation of COUNT_RATE, NET_COUNTS, SUR_BRI 
#           w/un-normalized exposure map: Input binned image, Exposure Maps are identical (cm**2-sec)
   11 ) test_string="dmextract infile='$INDIR/dmextract11.fits[bin sky=@$INDIR/dmextract11_src.reg]' outfile=$outfile bkg='$INDIR/dmextract11.fits[bin sky=@$INDIR/dmextract11_bkg.reg]' exp=$INDIR/dmextract11_src_emap.fits bkgexp=$INDIR/dmextract11_bkg_emap.fits error=gaussian clobber=yes bkgerror=gaussian"

           ;;

    # !!9
# TEST #12: from fap (test 13): verify bugfix 7104 - proper calculation of COUNT_RATE, NET_COUNTS, SUR_BRI 
#           with unequal units in src/bkg exp maps: Input binned image, src exp = cm**2-sec, bkg = cm**2
   12 ) test_string="dmextract infile='$INDIR/dmextract12.fits[bin sky=@$INDIR/dmextract12_src.reg]' outfile=$outfile bkg='$INDIR/dmextract12.fits[bin sky=@$INDIR/dmextract12_bkg.reg]' exp=$INDIR/dmextract12_src_emap.fits bkgexp=$INDIR/dmextract12_bkg_emap.fits error=gaussian clobber=yes bkgerror=gaussian"

           ;;

    # !!9
# TEST #13: from fap (test 17): verify bugfix 7104 - proper calculation of COUNT_RATE, NET_COUNTS, SUR_BRI 
#           with only src exp map present: Input binned image, src exp = cm**2-sec, no bgd exp
   13 ) test_string="dmextract infile='$INDIR/dmextract13.fits[bin sky=@$INDIR/dmextract13_src.reg]' outfile=$outfile bkg='$INDIR/dmextract13.fits[bin sky=@$INDIR/dmextract13_bkg.reg]' exp=$INDIR/dmextract13_src_emap.fits error=gaussian clobber=yes bkgerror=gaussian"

           ;;

    # !!9
# TEST #14: similar to test 11, but with both exposure maps normalized (units of cm**2)
   14 ) test_string="dmextract infile='$INDIR/dmextract14.fits[bin sky=@$INDIR/dmextract14_src.reg]' outfile=$outfile bkg='$INDIR/dmextract14.fits[bin sky=@$INDIR/dmextract14_bkg.reg]' exp=$INDIR/dmextract14_src_emap.fits bkgexp=$INDIR/dmextract14_bkg_emap.fits error=gaussian clobber=yes bkgerror=gaussian"

           ;;
#
# test 'grid(<file>)' syntax
#

# 
# TEST 15=#5: same as test 5 but TIME grids are read from the file
# so the there should be no difference between the two outputs
   15 ) test_string="dmextract infile='$INDIR/dmextract3b.fits[bin TIME=grid($TESTSAV/dmextract/dmextract5.out[cols time_min, time_max])]' outfile=$outfile opt=ltc1  error=gaussian verbose=0 clobber=yes"
           ;;

# TEST 16=#9: same as test 9 but [sky ..][bin ..] is in 1st-in-2nd rder
# so the there should be no difference between the two outputs
   16 ) test_string="dmextract infile='$INDIR/dmextract9.fits[bin pi=1:1024:1][sky=circle(4000.7799,3957.4344,30.14439)]' outfile=$outfile error=gaussian wmap='det=32' clobber=yes"
           ;;

# TEST 17: test simulated binning arraies: overlap pha array
   17 ) test_string="dmextract infile='$INDIR/dmextract1.fits[bin pha=grid($INDIR/overlap_pha.fits[cols lo,hi])]' outfile=$outfile opt=generic clobber=yes"
           ;;

# TEST 18: test simulated binning arraies: gapped pha array
   18 ) test_string="dmextract infile='$INDIR/dmextract1.fits[bin pha=grid($INDIR/partial_pha.fits[cols lo,hi])]' outfile=$outfile opt=generic clobber=yes"
           ;;
# TEST 19: test simulated binning arraies: no gapped pha array
   19 ) test_string="dmextract infile='$INDIR/dmextract1.fits[bin pha=grid($INDIR/full_pha.fits[cols lo,hi])]' outfile=$outfile opt=generic clobber=yes"
           ;;

# TEST 20: test logarithm-number binning syntax: min:max:#<>L
   20 ) test_string="dmextract infile='$INDIR/numberarr.fits[bin dat=1:1000:#3l]' outfile=$outfile opt=generic clobber=yes"
           ;;

# TEST 21: test logarithm-step binning syntax: min:max:<log-stp>L
   21 ) test_string="dmextract infile='$INDIR/numberarr.fits[bin dat=1:1000:1L]' outfile=$outfile opt=generic clobber=yes"
	;;
# TEST 22: test lightycurve 
   22 ) test_string="dmextract infile='$INDIR/acisf00637_001N002_evt3.fits[sky=circle(4988.5,4044.5,87.864959)][bin time=74822924:74920610:30]'  outfile=$outfile opt=ltc1 clobber=yes"
	;;

# TEST #23: verify bugfix 8678 - proper calculation of NET_COUNTS 
#           w/bkg={numeric value}.  Also populate BG_SUR_BRI output 
#           column based on input bkg value (adjusted for ontime/livetime).
   23 ) test_string="dmextract infile='$INDIR/dmextract23.fits[bin pos=@$INDIR/dmextract23_src.reg]' outfile=$outfile bkg='5062.3974609' error='gaussian' bkgerror='gaussian' opt=generic clobber=yes"
         ;;
# same as 23 except opt=pha1. output same as 23 except 23b will see a warning 
   23b ) test_string="dmextract infile='$INDIR/dmextract23.fits[bin pos=@$INDIR/dmextract23_src.reg]' outfile=$outfile bkg='5062.3974609' error='gaussian' bkgerror='gaussian' opt=pha1 clobber=yes"
         ;;
# TEST #24: verify bugfix 9324 - increase internal file/path/filter array sizes to 
#           DS_SZ_PATHNAME, add error checking for larger file/path/filter names.
   24 ) test_string="dmextract infile='$INDIR/dmextract24.fits[bin sky=ellipse(4138.5,3949.25,2.5,2.25,0)]' outfile=$outfile bkg='$INDIR/dmextract24_bkg.fits[bin sky=ellipse(4138.5,3949.25,5,4.5,0)-ellipse(4138.5,3949.25,2.5,2.25,0)-ellipse(4134.5872,3944.4593,3.4536592,2.3207029,171.78377)-ellipse(4141.5872,3943.9593,3.2738025,3.0163679,171.78377)]' clobber=yes"
         ;;
# TEST #25: verify bugfix 9625 - default to "gaussian" error with variance file (when opt=pha) 
   25 ) test_string="dmextract infile='$INDIR/dmextract25.fits[bin pha=1:1000:10]' outfile=$outfile opt='pha1' error='varfile' bkgerror='varfile' clobber=yes"
         ;;

   26) test_string="dmextract infile='$INDIR/acisf00637_001N002_evt3.fits[sky=circle(3279.5,3605.5,36.233389),ccd_id=1][bin time=::100]' outfile=$outfile bkg='$INDIR/acisf00637_001N002_evt3.fits[sky=circle(3685.5,3587.5,78.670175),ccd_id=1]' error='gaussian' bkgerror='gaussian' bkgnorm='1' exp='' bkgexp='' sys_err='0' opt='ltc1' clobber='yes'"
         ;;

   27) test_string="dmextract infile='$INDIR/dmextract27.fits[EVENTS][pos=circle(25920,25920,300)][bin time=grid($INDIR/dmextract27.lc[cols time_min,time_max])]' outfile=$outfile bkg='$INDIR/dmextract27.fits[EVENTS][pos=circle(21184,30496,1200)]' error='gehrels' bkgerror='gehrels' opt='ltc1' clobber='yes'"
         ;;

   # [ #9340 ]
   28) test_string="dmextract infile='$INDIR/dmextract28.fits[bin pos=circle(962,1058,15)]' outfile=$outfile opt='generic' clobber='yes'"
         ;;

   # [ #9605 ]
   29a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=::100]' bkg='$INDIR/dmextract29.fits[EVENTS][pos=circle(32896,22976,1200)]' exp='$INDIR/dmextract29.fits[EXPOSU05][cols time,dtf=fracexp]' outfile=${outfile}   clob+  verb=0  opt=pha1"
         ;;

   29b) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=::100]' bkg='$INDIR/dmextract29.fits[EVENTS][pos=circle(32896,22976,1200)]' exp='$INDIR/dmextract29.fits[EXPOSU05][cols time,dtf=fracexp]' outfile=${outfile}   clob+  verb=0  opt=ltc1"
         ;;

   # [ #11726 ]
   30a) test_string="dmextract infile='$INDIR/dmextract30.fits[EVENTS][bin pi=1:1024:1]' outfile=${outfile}   clob+  verb=0  opt=pha1"
         ;;

   30b) test_string="dmextract infile='$INDIR/dmextract30.fits[EVENTS][bin expno=1:10000:1]' outfile=${outfile}   clob+  verb=0  opt=ltc1"
         ;;

   # [ #10909 ]
   31a) test_string="dmextract infile='$INDIR/dmextract31a.fits[sky=circle(4096,4096,200)][bin expno=1:1e5:100]' outfile=${outfile} opt=ltc1 clob+"
         ;;

   # [ #11726 ]
   31b) test_string="dmextract infile='$INDIR/dmextract31b.fits[bin expno=1:20:10]' outfile=${outfile} op=generic clob+"
         ;;

   # [ #12575 ]: exe_cmd same as 29b, Except bkg=NONE, [bin time=T1:T2:3.0], exp=File[time=T1:T2]
   #  delt_bin_time = 3.1 s; delt_dtf_time = 2.6 s ;
   32a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/dmextract29.fits[EXPOSU05][cols time,dtf=fracexp][time=105562658.0:105562680.0]' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32a, except exp=exp32b.ascii
   32b) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp32b.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32a, except exp=exp32c.ascii
   32c) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp32c.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32a, except exp=exp32d.ascii
   32d) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp32d.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32a, except exp=exp32e.ascii
   32e) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp32e.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;
   # ---
   # [ bug 12575 ]: exe_cmd same as 32a, Except exp file is ascii w/ constant dtf=1.
   33a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp33a.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32b, except exp=exp33b.ascii w/ constant dtf
   33b) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp33b.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32c, except exp=exp33c.ascii w/ constant dtf
   33c) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp33c.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32d, except exp=exp33d.ascii w/ constant dtf
   33d) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp33d.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32e, except exp=exp33e.ascii w/ contant dtf
   33e) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp33e.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32a except expfile is ascii and w/ diff. dtf value 
   34a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp34a.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 32b, except exp=exp34b.ascii w/ more records in expfile
   34b) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp34b.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 34a except exp35a has more records
   35a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp35a.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # same as 35a except exp36a has constant dtf 
   36a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='$INDIR/exp36a.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # test warning when expfile can't be found 
   37a) test_string="dmextract infile='$INDIR/dmextract29.fits[EVENTS][pos=circle(25920,25920,300)][bin time=105562658.0:105562678.0:3.0]' exp='yyyexpfile.ascii' outfile=${outfile} bkg=NONE bkgexp=NONE clob+ verb=0 opt=ltc1"
         ;;

   # HRC obsid 11933 ; both exp and bkgexp are found
  _hrc_a ) test_string="dmextract \
           infile='$INDIR/hrcf11933N002_evt2.fits[sky=region($INDIR/regsrc.reg)][bin time=::50]' \
           outfile=${outfile} \
           bkg='$INDIR/hrcf11933N002_evt2.fits[sky=region($INDIR/regbkg.reg)]' \
           exp='$INDIR/hrcf11933_000N001_dtf1.fits' \
           bkgexp='$INDIR/hrcf11933_000N001_dtf1.fits' \
           opt='ltc1' mode=h clob+ "
         ;;

   # HRC obsid 11933 ; both exp and bkgexp are not found
  _hrc_b ) test_string="dmextract \
           infile='$INDIR/hrcf11933N002_evt2.fits[sky=region($INDIR/regsrc.reg)][bin time=::50]' \
           outfile=${outfile} \
           bkg='$INDIR/hrcf11933N002_evt2.fits[sky=region($INDIR/regbkg.reg)]' \
           exp='zzz_src.fits' \
           bkgexp='zzz_bkg.fits' \
           opt='ltc1' mode=h clob+ "
         ;;

   # input image in double w/ NaN
  _dbl_img ) test_string="dmextract \
           infile='$INDIR/dbl_img.fits[bin sky=@$INDIR/ann_dbl_img.reg]' \
           outfile=${outfile} bkg=NONE opt='generic' clob+ "
         ;;

   # input exposure image : memory leak : bug 12876 ;   warning w/o opt=generic;
  _leak ) test_string="dmextract \
           infile='$INDIR/in_leak.fits[bin sky=@$INDIR/src_leak.reg]' \
           outfile=${outfile}   \
           exp=$INDIR/exp_leak.fits  bkgexp=   \
           bkg=  error=gehrels bkgerror=gaussian  bkgnorm=1 \
           sys_err=0  clobber=yes verbose=0 "
         ;;

   # 14100 error out when weight column missing/dne
   _weighta )  test_string="dmextract \
           infile='$INDIR/evt.flx[bin energy=300:9000:100;colname_DNE]' \
           outfile=${outfile}   \
           exp=  bkgexp=   \
           bkg=  opt=generic error=gaussian bkgerror=gaussian  bkgnorm=1 \
           sys_err=0  clobber=yes verbose=0 "
         ;;

   _weightb )  test_string="dmextract \
           infile='$INDIR/evt.flx[bin energy=300:9000:100;flux]' \
           outfile=${outfile}   \
           exp=  bkgexp=   \
           bkg=  opt=generic error=gaussian bkgerror=gaussian  bkgnorm=1 \
           sys_err=0  clobber=yes verbose=0 "
         ;;

  _invalid_filt ) test_string="dmextract \
           infile='$INDIR/hrcf11933N002_evt2.fits[bin time=::#100]' \
           outfile=${outfile} \
           bkg='' \
           exp='$INDIR/hrcf11933_000N001_dtf1.fits' \
           bkgexp='' \
           opt='ltc1' mode=h clob+ "
         ;;

   # HRC obsid 11933 ; both exp and bkgexp are not found

  # 13701 long column name: kernel overflow causes column name to disappear
  # using data from #13701
   38 )  test_string="dmextract \
           infile='$INDIR/orion.tsv[opt kernel=text/tsv][bin flux_significance_b=1:250:1]' \
           outfile=${outfile} \
           opt=generic \
           clobber=yes \
           sys_err=0 \
           verbose=0 "
	 ;;

   _invalid_mask )  test_string="dmextract 'evt[bin sky=mask()]' 'tmp' verbose=0 "
        ;;

   # TEST #39a/b: Bug SAT-5. Apply weight to sky extractions
   # Test original bug, with data therein. Circle extraction with 1 count in region. 
    39a ) test_string="dmextract \
          infile='$INDIR/dmextract39a.fits[bin sky=circle(4165,4052,2.5);energy]'
          outfile=$outfile \
          clobber=yes \
          opt=generic"
    ;;

    # Test weighting for sky extraction with multiple overlapping and non overlapping regions with 1 count in region
    39b ) test_string="dmextract \
          infile='$INDIR/dmextract39b.fits[bin sky=@$INDIR/dmextract39b.reg;energy]'
          outfile=$outfile \
          clobber=yes \
          opt=generic"
    ;;

    # TEST #40a/b: Bug SAT-27. Convert coordinates properly when binning on virtual column 
    # Show that binning on eqpos gives correct counts/radius
     40a ) test_string="dmextract \
           infile='$INDIR/dmextract40a.fits[bin eqpos=@$INDIR/dmextract40a.reg]'
           outfile=$outfile \
           clobber=yes \
           opt=generic"
     ;;

     # Show that binning on sky still gives correct counts/radius
    40b ) test_string="dmextract \
           infile='$INDIR/dmextract40b.fits[bin sky=@$INDIR/dmextract40b.reg]'
           outfile=$outfile \
           clobber=yes \
           opt=generic"
      ;;

    # TEST #41: Bug SL-65. Allocate memory for region with no shape
    41 ) test_string="dmextract \
          infile='$INDIR/dmextract41.fits[bin (col1,col2)=region($INDIR/dmextract41_noRows.fits)]'
          outfile=$outfile \
          clobber=yes \
          opt=generic"
    ;;

    # TEST #42: Bug SAT-146. Using the wrong GTI (last not first).
    42 ) test_string="dmextract \
           infile='$INDIR/dmextract42.fits[sky=region($INDIR/dmextract42.reg)][bin time=::1000]' \
           outfile=${outfile} \
           opt='ltc1' mode=h clob+ "
         ;;

  esac

  if [ ${testid} = "_weighta" ] || [ ${testid} = "_invalid_filt" ] || [ ${testid} = "_invalid_mask" ]; then 
      echo $test_string | tee -a  $LOGFILE
      eval $test_string
      mystatus=$?
      echo " -- status of call was $mystatus"  >>  $LOGFILE
      echo " -- status of call was $mystatus"  > ${outfile}.status
  else
      echo $test_string | tee -a  $LOGFILE
      eval $test_string | tee -a  $LOGFILE  2>&1
  fi

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # compare
  # !!12
  #  diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE

   if [ ${testid} = "_weighta" ] || [ ${testid} = "_invalid_filt" ] || [ ${testid} = "_invalid_mask" ]; then 
      # this test checks for expected errors so diff the exit status of run
      diff $outfile.status $savfile.status  > /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
        echo "ERROR: MISMATCH in $outfile.status" >> $LOGFILE
        mismatch=0
      fi
   else 
      if [ ${testid} = "29a" ] || [ ${testid} = "29b" ] ; then
          dmdiff $outfile $savfile sub- tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
      else
          if [ ${testid} = "_dbl_img" ] ; then
              dmdiff $outfile $savfile tol=$SAVDIR/tolerance2 > /dev/null 2>>$LOGFILE
          else
              dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
	  fi
      fi
      if  test $? -ne 0 ; then
          echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
          mismatch=0
      fi
   fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  echo ""
  if test $mismatch -eq 0 ; then
    # Yes
    echo "test ${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
