#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmimgpick.t
# test script for dmimgpick


# !!2
# syntax:
# dmimgpickt. [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmimgpick"

# set up list of tests
# !!4
alltests="test_avg test_weight test_min test_max test_far test_close \
test_random_weight test_random_close test_random_far test_random_min \
test_random_max test_random_avg test_random_min_nans test_multi_cols \
test_filtr_cols test_incor_cols test_null test_wcs_sin test_wcs_rot \
test_wcs_nowcs test_wcs_lin test_wcs_lin2 test_wcs_sep"

# "short" test to run
# !!5
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo ""
  echo "running $testid" | tee -a $LOGFILE
  echo ""

  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test_avg ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=average clob+"
            ;;

    test_weight ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=weight clob+"
            ;;

    test_far ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=furthest clob+"
            ;;

    test_close ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=close clob+"
            ;;

    test_max ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=max clob+"
            ;;

    test_min ) test_string="dmimgpick infile=$INDIR/in_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=min clob+"
            ;;

    test_random_weight ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_delta_image.fits outfile=$outfile method=weight clob+"
            ;;

    test_random_avg ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_delta_image.fits outfile=$outfile method=av clob+"
            ;;

    test_random_close ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=close clob+"
            ;;

    test_random_far ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=fur clob+"
            ;;

    test_random_min ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=min clob+"
            ;;

    test_random_max ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=max clob+"
            ;;

    test_random_min_nans ) test_string="dmimgpick infile=$INDIR/in_random_table.fits imgfile=$INDIR/in_delta_with_nans_image.fits outfile=$outfile method=min clob+"
            ;;

    test_avg_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=average clob+"
             ;;

    test_weight_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=weight clob+"
             ;;

    test_far_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=furthest clob+"
             ;;

    test_close_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=close clob+"
             ;;

    test_max_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=max clob+"
             ;;

    test_min_ext ) test_string="dmimgpick infile=$INDIR/in_table_ext.fits imgfile=$INDIR/in_image.fits outfile=$outfile method=min clob+"
             ;;

     test_multi_cols  ) test_string="dmimgpick infile=$INDIR/in_table_st87.dtf imgfile=$INDIR/in_image_st87.dtf outfile=$outfile method=closest clob+"
             ;;

      test_filtr_cols   ) test_string="dmimgpick infile=$INDIR/in_table_st87.dtf\"[cols sky]\" imgfile=$INDIR/in_image_st87.dtf outfile=$outfile method=closest clob+"
             ;;

      test_incor_cols  ) test_string="dmimgpick infile=$INDIR/in_table_st87.dtf\"[cols det]\" imgfile=$INDIR/in_image_st87.dtf outfile=$outfile method=closest clob+"
            ;;

      test_null ) test_string="dmimgpick infile=$INDIR/pos.dat imgfile=$INDIR/hrc1411_mod.fits outfile=$outfile method=closest clob+"
            ;;

      test_wcs_sin ) test_string="dmimgpick infile=$INDIR/coords_sin.dat\"[cols x=col1,y=col2]\" imgfile=$INDIR/faked_sin.fits outfile=$outfile method=closest clob+"
            ;;

      test_wcs_rot ) test_string="dmimgpick infile=$INDIR/coords.dat\"[cols x=col1,y=col2]\" imgfile=$INDIR/fake_crota.img outfile=$outfile method=closest clob+"
            ;;

      test_wcs_nowcs ) test_string="dmimgpick infile=$INDIR/coords.dat\"[cols x=col1,y=col2]\" imgfile=$INDIR/nowcs.img outfile=$outfile method=closest clob+"
            ;;

      test_wcs_lin ) test_string="dmimgpick infile=$INDIR/coords.dat\"[cols chipx=col1,chipy=col2]\" imgfile=$INDIR/chip.img outfile=$outfile method=closest clob+"
            ;;

      test_wcs_lin2 ) test_string="dmimgpick infile=$INDIR/coords_lin2.dat\"[cols x=col1,y=col2]\" imgfile=$INDIR/chip.mod.img outfile=$outfile method=closest clob+"
            ;;

      test_wcs_sep ) test_string="dmimgpick infile=$INDIR/coords.dat\"[cols aa=col1,bb=col2]\" imgfile=$INDIR/separate_linear_wcs.img outfile=$outfile method=closest clob+"

  esac

  echo $test_string  | tee -a $LOGFILE 
  if test "x${testid}" != "xtest_incor_cols" ;  then
     eval $test_string  | tee -a $LOGFILE  2>&1
  fi
 
  ####################################################################
  # check the outputs

  # Init per-test error flag
  if test "x${testid}" = "xtest_incor_cols" ;  then
      eval $test_string
      mismatch=$?
  else
      mismatch=1
  fi

  ####################################################################

if test "x${testid}" = "xtest_incor_cols" ; then
    if test $mismatch -ne 0 ; then  # expected error-status (255)
	# printf "\n OK, Expect error message from test_incor_cols. \n" |tee -a $LOGFILE 
	mismatch=1
    else
	mismatch=0
    fi
else
    dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
    if  test $? -ne 0 ; then
	printf "\n dmdiff $outfile $savfile tol=$SAVDIR/tolerance \n" |tee -a $LOGFILE 
	echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
	mismatch=0
    fi
fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"|tee -a $LOGFILE
    script_succeeded=1
  else
    # No
    echo "${testid} OK"|tee -a $LOGFILE
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
