#!/bin/sh

##########################################################
# test script to run the dmkeypar code and generate
# the Test documentation
#
# IMPORTANT:  IF MAKING ANY CHANGES TO THE TEST CASES
#     THERE ARE 2 LINES THAT MUST BE UNCOMMENTED
#     - FIRST LINE WILL ALLOW .PAR FILES TO BE WRITTEN TO
#       THE SAVE DIRECTORY
#     - SECOND LINE WILL ALLOW THE TEST DOCUMENT IN THE
#       UDF TO BE MODIFIED
#     THESE 2 LINES ARE PRECEDED WITH THE LINE:
#       "THE FOLLOWING LINE SHOULD BE COMMENTED"
#     ALSO BE SURE TO CHANGE THE NUMBER OF REFERENCE
#      FILES AND INPUT FILES IN:
#        verify_reference_files()
#        verify_input_files()
#
# Joseph Masters,  24 Aug 2000  - Last Modified 9/8/2000
############################################################

# UNCOMMENT THE FOLLOWING LINE FOR DEBUGGING
# set -x

#######################################################
# set the date, environment variables, etc.
#######################################################
setup()
{
  echo
  echo "DMKEYPAR REGRESSION TEST"
  echo

  DATE=`date +'%y%m%d_%T'`
  exit_flag=0

  # TESTIN  is where the test data resides
  # TESTOUT is where output goes
  # TESTLOG is where the log output goes
  # TESTSAV is where reference output exists

  # check to see if the necessary environment variables were set
  if [ "$TESTIN" = "" ]; then
    echo "Environment variable \$TESTIN is not set."
    exit_flag=1
  fi
  if [ "$TESTOUT" = "" ]; then
    echo "Environment variable \$TESTOUT is not set."
    exit_flag=1
  fi
  if [ "$TESTLOG" = "" ]; then
    echo "Environment variable \$TESTLOG is not set."
    exit_flag=1
  fi
  if [ "$TESTSAV" = "" ]; then
    echo "Environment variable \$TESTSAV is not set."
    exit_flag=1
  fi

  # if the exit flag was set, then exit
  if [ $exit_flag = 1 ]; then
    exit 1
  fi
  
  # set the environment variable to local variables
  indir=${TESTIN}/dmkeypar
  outdir=${TESTOUT}/dmkeypar
  savdir=${TESTSAV}/dmkeypar
  logdir=${TESTLOG}/dmkeypar

  # Make sure we have a log directory
  if test -d $logdir ; then
   :
  else
    mkdir -p $logdir 
    if test $? -ne 0 ; then
      error_exit ""
    fi
  fi

  LOGFILE=$logdir/dmkeypar_log.${DATE}

  # Make sure we have an output directory
  if test -d $outdir ; then
   :
  else
    mkdir -p $outdir >> $LOGFILE 2>&1
    if test $? -ne 0 ; then
      error_exit "can't create output directory $outdir"
    fi
  fi


  echo "Input directory     = "$indir  | tee -a $LOGFILE
  echo "Output directory    = "$outdir | tee -a $LOGFILE
  echo "Reference directory = "$savdir | tee -a $LOGFILE
  echo "Log directory       = "$logdir | tee -a $LOGFILE
  echo

  # temporary path for dmkeypar Test document
  temp_udf_test=$outdir/dmkeypar_Test

  # path for dmkeypar Test document
  udf_test=/stage/ascds_extra/staff/UDF/Tools/flight/dmtools/dmkeypar/Test
}

######################################################################
#  verify that all input files exist                                 #
######################################################################
verify_input_files()
{
  for FILE_NUM in 1 2 3
  do
    if test -f $indir/dmkeypar$FILE_NUM.fits; then
      :
    else
      echo "Input file does not exist:"| tee -a $LOGFILE
      echo " Missing $indir/dmkeypar$FILE_NUM.fits"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All input files exist" | tee -a $LOGFILE
}

######################################################################
#  verify that all saved reference files exist                       #
######################################################################
verify_reference_files()
{
  for FILE_NUM in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18
  do
    if test -f $savdir/test${FILE_NUM}_sav.par; then
      :
    else
      echo "Reference file $FILE_NUM does not exist:"| tee -a $LOGFILE
      echo " Missing $savdir/test${FILE_NUM}_sav.par"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All reference files exist" | tee -a $LOGFILE
  echo
}

####################################
# delete old output files          #
####################################
delete_old_outputs()
{
  rm -f $outdir/dmkeypar*
}

###############################################################################
# generate the header for the Test document in the UDF                        #
###############################################################################
write_test_doc_header()
{
   echo "Testing Status" > $temp_udf_test
   echo "Date: 24 August 2000" >> $temp_udf_test
   echo "Last Modified: 24 August 2000" >> $temp_udf_test
   echo "Author: Joseph Masters" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Regression test:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Testing is run by the regression script dmkeypar.t" >> $temp_udf_test
   echo "The input data is in $indir" >> $temp_udf_test
   echo "The script runs the following test cases:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo >> $temp_udf_test
}

##########################################################################
# copy the paramater file to the "save directory" for a later comparison #
##########################################################################
copy_param()
{
  # THE FOLLOWING LINE SHOULD BE COMMENTED
  # cp /home/jmasters/ascds_iraf/uparm/dmkeypar.par $savdir/dmkeypar${ii}.par

  echo > /dev/null  # keep this line, function must execute a statement
}

###############################################################################
# do plists on saved and current parameter files and compare                  #
###############################################################################
compare_outputs()
{
  # plist the saved parameter file for comparison
  plist $savdir/test${ii}_sav.par > $outdir/dmkeypar${ii}.buf
  grep -v "Parameters for" $outdir/dmkeypar${ii}.buf | \
  grep -v "infile = " > $outdir/dmkeypar${ii}.saved

  # plist the latest parameter file for comparison
  plist dmkeypar.par > $outdir/dmkeypar${ii}.buf
  grep -v "Parameters for" $outdir/dmkeypar${ii}.buf | \
  grep -v "infile = " >  $outdir/dmkeypar${ii}.latest

  # compare the current parameter output with the saved parameter output.
  # if there is a difference between the two,
  # tell the user that the test fails.

  diff $outdir/dmkeypar${ii}.latest $outdir/dmkeypar${ii}.saved >> \
       $LOGFILE

  if [ $? -eq 0 ]; then
      echo "Test # ${ii} OK" | tee -a $LOGFILE
      echo
  else
      tool_status=FAIL
      echo "Test # ${ii} NOT-OK" | tee -a $LOGFILE
      echo "     saved and latest param outputs do not match for test# ${ii}" \
           | tee -a $LOGFILE
      echo
  fi
}

###############################################
# write_test_doc                              #
###############################################
write_test_doc()
{
  echo "TEST #${ii}:" >> $temp_udf_test
  printf " $summary\n" >> $temp_udf_test
  printf " $command\n" >> $temp_udf_test
  echo >> $temp_udf_test
}

  
##########################
# increase count         #
##########################
increase_count()
{
  ii=`expr $ii + 1`
}


###############################################################################
# generate the trailer for the test document                                  #
###############################################################################
write_test_doc_trailer()
{
   printf "~make test~ will run the test script AND CAN BE USED TO REGENERATE THIS FILE.\n" >>\
           $temp_udf_test
}


##########################
# cleanup                #
##########################
cleanup()
{
  # copy the temporary Test document to the more permanent Test document
  # THE FOLLOWING LINE SHOULD BE COMMENTED
  #  cp $temp_udf_test $udf_test

  # remove the temporary Test document
    rm $temp_udf_test

  # remove the plist buffer files
    rm $outdir/dmkeypar*.buf

    printf "\ndmkeypar: $tool_status\n" | tee -a $LOGFILE
}



#################
#---------------#
#   MAIN CODE   #
#---------------#
#################

ii=1        # used for test count
tool_status=PASS

# set the date, environment variables, etc.
setup

verify_input_files
verify_reference_files
delete_old_outputs
write_test_doc_header

# run each test

# 1 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=MJDREF
summary="Using a FITS infile w/o an extension and keyword of type pos. real."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=MJDREF"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 2 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=sim_z
summary="Using a FITS infile w/o an extension and keyword of type neg. real."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=sim_z"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 3 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=HISTNUM
summary="Using a FITS infile w/o an extension and keyword of type pos. int."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=HISTNUM"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 4 #
dmkeypar infile="$indir/dmkeypar3.fits" keyword=raDEcsys
summary="Using a FITS infile w/o an extension and keyword of type string."
command="dmkeypar infile=\"$indir/dmkeypar3.fits\" keyword=raDEcsys"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 5 #
dmkeypar infile="$indir/dmkeypar3.fits" keyword=cloCkaPP
summary="Using a FITS infile w/o an extension and keyword of type boolean."
command="dmkeypar infile=\"$indir/dmkeypar3.fits\" keyword=cloCkaPP"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 6 #
dmkeypar infile="$indir/dmkeypar1.fits[events]" keyword=tstop
summary="Using a FITS infile w/ an extension and keyword of type pos. real."
command="dmkeypar infile=\"$indir/dmkeypar1.fits[events]\" keyword=tstop"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 7 #
dmkeypar infile="$indir/dmkeypar1.fits[2]" keyword=histnum
summary="Using a FITS infile w/ an extension and keyword of type pos. int."
command="dmkeypar infile=\"$indir/dmkeypar1.fits[2]\" keyword=histnum"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 8 #
dmkeypar infile="$indir/dmkeypar1.fits[EVENTS]" keyword=instrume
summary="Using a FITS infile w/ an extension and keyword of type string."
command="dmkeypar infile=\"$indir/dmkeypar1.fits[EVENTS]\" keyword=instrume"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 9 #
dmkeypar infile="$indir/dmkeypar2.fits[3]" keyword=CLOCKAPP
summary="Using a FITS infile w/ an extension and keyword of type boolean."
command="dmkeypar infile=\"$indir/dmkeypar2.fits[3]\" keyword=CLOCKAPP"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 10 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=revision
summary="Using a FITS infile w/o an extension and keyword of type int=0."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=revision"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 11 #
dmkeypar infile="$indir/dmkeypar1.fits[2]" keyword=tg_r 2> /dev/null
summary="Using a FITS infile w/ an extension and keyword of type real.  ERROR -- tg_r is a column name."
command="dmkeypar infile=\"$indir/dmkeypar1.fits[2]\" keyword=tg_r"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 12 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=comment 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type string.  ERROR -- comment is a bad keyword."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=comment"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 13 #
dmkeypar infile="$indir/jba_dfile.fits" keyword=history 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type string.  ERROR -- bad file name and keyword."
command="dmkeypar infile=\"$indir/jba_dfile.fits\" keyword=history"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 14 #
dmkeypar infile="$indir/jbadfile.fits" keyword=MJDREF 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type real.  ERROR -- bad file name."
command="dmkeypar infile=\"$indir/jbadfile.fits\" keyword=MJDREF"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 15 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=jbadkey 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type string.  ERROR -- non-existant keyword."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=jbadkey"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 16 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=COMMENT 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type string.  ERROR -- using COMMENT as keyword."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=COMMENT"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 17 #
dmkeypar infile="$indir/dmkeypar1.fits" keyword=HISTORY 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type string.  ERROR -- using HISTORY as keyword."
command="dmkeypar infile=\"$indir/dmkeypar1.fits\" keyword=HISTORY"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

# 18 #
dmkeypar infile="$indir/dmkeypar4.fits" keyword=BIASTART 2> /dev/null
summary="Using a FITS infile w/o an extension and keyword of type ULONG."
command="dmkeypar infile=\"$indir/dmkeypar4.fits\" keyword=BIASTART"
echo ${command} | tee -a $LOGFILE
copy_param
compare_outputs
write_test_doc
increase_count

write_test_doc_trailer
cleanup

echo
exit 0
