#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# dmmerge.t
# test script for dmmerge


# !!2
# syntax:
# dmmerge.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="dmmerge"

# set up list of tests
# !!4
alltests="test1 test1_a test1_b test2 test3 hrcs1246 acisout acis_2469 calcgti_1 calcgti_2 calcgti_3 calcforce_3 calcgti_4 calcforce_4  calcgti_5  test1_calcgti hrcs1246_calcgti acisout_calcgti sky_mosaic ss_default ss_filter ss_filter_x pos_mosaic pos_mosaic_no b13651"

# "short" test to run
# !!5
shortlist="test1 test2 test3"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
punlearn $toolname

script_succeeded=0


#
# Because we need an absolute path, we change the @FOO@ in the hrc.lis file
# using sed and put the .lis file in the output directory
#

sed "s,@FOO@,${INDIR},g" $INDIR/hrcs_evt1.lis > $OUTDIR/hrcs_evt1.lis

######################################################################
# Begin per-test loop


for testid in $testlist
do

  # Init per-test error flag
  # 
  mismatch=1
    
  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  # delete old outputs
  rm -f ${outfile}

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test1 ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=${INDIR}/dmmerge_header_lookup.txt clobber=yes verbose=5> /dev/null"
        gtId="GTI"
            ;;

    # !! set lookupTab="",  infile=dmmergeIn.lis
    test1_a ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=  clobber=yes > /dev/null"
        gtId=""
            ;;

    # !! set lookupTab="",  infile=dmmergeIn.lis2
    test1_b ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis2[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=  clobber=yes > /dev/null"
        gtId="GTI"
            ;;

    # !! use calcgti ; output same as test1 ;
    test1_calcgti ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes > /dev/null"
        gtId="GTI"
            ;;

    # !!6.2 (.vs. test1)
    test2 ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=${INDIR}/lev2_dmmerge_lookup.txt clobber=yes > /dev/null"
        gtId="GTI"
            ;;
    # !!6.3 ( .vs. test1 )
    test3 ) test_string="dmmerge infile=\"@${INDIR}/dmmergeIn.lis_2[EVENTS]\" outfile=$outfile outBlock=\"EVENTS\" columnList=\"time,chipx,chipy\" lookupTab=${INDIR}/dmmerge_header_lookup.txt clobber=yes > /dev/null"
        gtId="GTI"
            ;;
    # !!7
    hrcs1246 ) tmpfile="@${OUTDIR}/hrcs_evt1.lis[cols\ !crsu,!crsv,!amp_sf,!av1,!av2,!av3,!au1,!au2,!au3,!raw,!sumamps]"
      test_string="dmmerge infile=$tmpfile \
          columnList=\"\"  outBlock=\"\" outfile=$outfile \
          lookupTab=${INDIR}/lev2_dmmerge_lookup.txt \
          clobber=yes  >  /dev/null"
        gtId="GTI"
           ;;

    # !!7 use calcgti ; output same as hrcs1246 ;
    hrcs1246_calcgti ) tmpfile="@${OUTDIR}/hrcs_evt1.lis[cols\ !crsu,!crsv,!amp_sf,!av1,!av2,!av3,!au1,!au2,!au3,!raw,!sumamps]"
      test_string="dmmerge infile=$tmpfile \
          columnList=\"\"  outBlock=\"\" outfile=$outfile \
          lookupTab=${INDIR}/calcgti_lev2_dmmerge_lookup.txt \
          clobber=yes  >  /dev/null"
        gtId="GTI"
           ;;

    # !!8
    acisout)tmpIn="${INDIR}/acisIn1.fits[events],${INDIR}/acisIn2.fits[events]"
       test_string="dmmerge infile=$tmpIn outfile=$outfile outBlock=\"\"  \
            lookupTab=${INDIR}/dmmerge_header_lookup.txt  \
            columnList=\"time,pi,grade\" clobber=yes  >  /dev/null "
        gtId="GTI5 GTI6 GTI7 GTI8 GTI9"
           ;;

    # !! use calcgti ; output same as acisout
    acisout_calcgti )tmpIn="${INDIR}/acisIn1.fits[events],${INDIR}/acisIn2.fits[events]"
       test_string="dmmerge infile=$tmpIn outfile=$outfile outBlock=\"\"  \
            lookupTab=${INDIR}/lkTab_calcGTI.txt  \
            columnList=\"time,pi,grade\" clobber=yes  >  /dev/null "
        gtId="GTI5 GTI6 GTI7 GTI8 GTI9"
           ;;

    # !!9 test unsigned dtype
    acis_2469) tmpIn2="${INDIR}/acis_2469_in.fits[events]"
     test_string="dmmerge infile=$tmpIn2 outfile=$outfile outBlock=\"\"  \
            lookupTab=${INDIR}/acis_2469_lookup.txt  \
            columnList=\" \" clobber=yes  >  /dev/null"
        gtId="GTI7 GTI6 GTI3 GTI2"
           ;;

    ##  1 infile ;  lkTab_calcGTI.txt from dmmerge_header_lookup.txt ;
    calcgti_1 ) IN2="${INDIR}/acis_2469_in.fits[events]"
     test_string="dmmerge infile=${IN2} outfile=$outfile outBlock=\"\"  \
            lookupTab=${INDIR}/lkTab_calcGTI.txt  \
            columnList=\" \" clobber=yes  >  /dev/null"
        gtId="GTI7 GTI6 GTI3 GTI2"
           ;;

    ##  2 same infile 
    calcgti_2 ) IN2="${INDIR}/acis_2469_in.fits[events],${INDIR}/acis_2469_in.fits[events]"
     test_string="dmmerge infile=${IN2} outfile=$outfile outBlock=\"\"  \
            lookupTab=${INDIR}/lkTab_calcGTI.txt  \
            columnList=\" \" clobber=yes  >  /dev/null"
        gtId="GTI7 GTI6 GTI3 GTI2"
           ;;

    ##  3 infiles : all gti's are not overlapped ; 
    calcgti_3 ) test_string="dmmerge infile=\"@${INDIR}/evt1a_in.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes > /dev/null"
gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9 GTI7_CPT7 GTI4_CPT8 GTI5_CPT9 GTI6_CPT10 GTI8_CPT11 GTI9_CPT12 GTI7_CPT13 GTI4_CPT14 GTI5_CPT15 GTI6_CPT16 GTI8_CPT17 GTI9_CPT18"
           ;;

    ##  calcforce_3 : same as calcgti_3 except lookupTab ; 
    ##              : output should be same as calcgti_3 ; 
    calcforce_3 ) test_string="dmmerge infile=\"@${INDIR}/evt1a_in.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/dmmerge_header_lookup.txt clobber=yes > /dev/null"
gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9 GTI7_CPT7 GTI4_CPT8 GTI5_CPT9 GTI6_CPT10 GTI8_CPT11 GTI9_CPT12 GTI7_CPT13 GTI4_CPT14 GTI5_CPT15 GTI6_CPT16 GTI8_CPT17 GTI9_CPT18"
            ;;

    ##  3 infiles :  gti's intersect ; 
    calcgti_4 ) test_string="dmmerge infile=\"@${INDIR}/interst.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes > /dev/null"
    gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9"
           ;;

    ##  3 infiles :  gti's intersect ; use old lkTab
    calcforce_4 ) test_string="dmmerge infile=\"@${INDIR}/interst.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/dmmerge_header_lookup.txt clobber=yes > /dev/null"
    gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9"
           ;;

    ##  3 infiles :  gti's intersect ; same as calcgti_4 except infile's order 
    calcgti_5 ) test_string="dmmerge infile=\"@${INDIR}/interst2.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes > /dev/null"
    gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9"
           ;;    

   ##  3 infiles :  gti's intersect ; same as calcgti_5 except tlmin/tlmax union
   sky_mosaic ) test_string="dmmerge infile=\"@${INDIR}/interst2_mosaic.lis[EVENTS]\" \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId="GTI7 GTI4 GTI5 GTI6 GTI8 GTI9"
           ;;

   ##  2 infiles :  demo pos column tlmin/tlmax is not typical (x,y) column file
   pos_mosaic ) test_string="dmmerge infile="${INDIR}/acisD1999-07-22badpixN0005.fits'[AXAF_BADPIX0]',${INDIR}/acisD1999-07-22badpixN0005_addTLrange.fits[AXAF_BADPIX0]"\
               outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
               lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId=""
           ;;
  
   ##  2 infiles :  demo  pos column tlmin/tlmax is not typical (x,y) column file
   pos_mosaic_no ) test_string="dmmerge infile="${INDIR}/acisD1999-07-22badpixN0005.fits'[AXAF_BADPIX1]',${INDIR}/acisD1999-07-22badpixN0005_addTLrange.fits[AXAF_BADPIX1]"\
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId=""
            ;;
   ## show union sky column tlmin/tlmax, and also update the subspace range as default
   ss_default ) test_string="dmmerge infile='${INDIR}/7017.ssdefault[subspace -expno],${INDIR}/7016.ssdefault[subspace -expno]' \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId="GTI3 GTI6 GTI7 GTI2 GTI1 GTI0"
           ;;

   ## show union sky column tlmin/tlmax, and also not update the subspace range as it has filter
   ss_filter ) test_string="dmmerge infile='${INDIR}/7017.ssfilter[subspace -expno],${INDIR}/7016.ssfilter[subspace -expno]' \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId="GTI3 GTI6 GTI7 GTI2 GTI1 GTI0"
           ;;

  ## show union sky column tlmin/tlmax, and also not update the subspace range as it has filter
  ss_filter_x) test_string="dmmerge infile='${INDIR}/7017.ssfilter_x[subspace -expno],${INDIR}/7016.ssfilter_x[subspace -expno]' \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/lkTab_calcGTI.txt clobber=yes  > /dev/null"
   gtId="GTI3 GTI6 GTI7 GTI2 GTI1 GTI0"
           ;;
   ## show union sky column tlmin/tlmax, and also not update the subspace range as it has filter
  b13651) test_string="dmmerge infile='${INDIR}/b13651_a.fits,${INDIR}/b13651_b.fits' \
                outfile=$outfile outBlock=\"EVENTS\" columnList=\" \" \
                lookupTab=${INDIR}/b13651_lookup.txt clobber=yes  > /dev/null"
   gtId="GTI7 GTI2 GTI5 GTI6 GTI3 GTI8"
            ;;
    
  esac

  echo $test_string | tee -a  $LOGFILE
  echo "" | tee -a  $LOGFILE
  #eval $test_string  | tee -a  $LOGFILE  2>&1
  eval  $test_string

  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0
  fi


  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  #dmlist $outfile header,data > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  #dmlist $savfile header,data > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  #keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #          2>>$LOGFILE

  # compare
  # !!12
  #diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #     /dev/null 2>>$LOGFILE

for gtid in $gtId 
do
  dmdiff $outfile"[${gtid}]" $savfile"[${gtid}]" tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi
done

# check the timing keys
dmdiff $outfile"[EVENTS]" $savfile"[EVENTS]" tol=$SAVDIR/tolerance data- > /dev/null 2>>$LOGFILE
if  test $? -ne 0 ; then
   echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
   mismatch=0
fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
