#! /bin/sh

# 11 February 2002

# dmreadpar.t
# test script for dmreadpar

# syntax:
# dmreadpar.t [<testid> ... ]


######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Ddaattaa/g;s/COMMENT/Ddaattaa/g;
  s/DATE/Ddaattaa/g;s/CREATOR/Ddaattaa/g;s/HISTORY/Ddaattaa/g;
  s/DATASUM/Ddaattaa/g;s/ASCDSVER/Ddaattaa/g;s/CVSD0001/Ddaattaa/g;
  s/CVST0001/Ddaattaa/g;s/HISTNUM/Ddaattaa/g' | \
  grep -v Ddaattaa > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

toolname="dmreadpar"

# set up list of tests
alltests="test1 test2 test3 test4"

# "short" test to run
shortlist="test1"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname
FTPDIR="ftp://cxc.harvard.edu/pub/incoming/regtest"

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*


# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc


# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  outfile=$OUTDIR/${testid}.fits
  outgz=$OUTDIR/${testid}.fits.gz
  savfile=$SAVDIR/${testid}.fits
  savgz=$SAVDIR/${testid}.fits.gz
  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    test1 ) command_string="dmreadpar $INDIR/dmreadpar_input.par \
                            \"$OUTDIR/dmreadpar.fits[4]\" \
                            case=upper clobber=yes"
            cp $INDIR/dmreadpar.fits $OUTDIR
	    extn="dmreadpar.fits[4]"
            ;;

      test2) command_string="dmreadpar $INDIR/tools-dmcoords1.par $OUTDIR/tools-dmcoords1.fits[1] case=upp clob+"
             cp $INDIR/tools-dmcoords1.fits $OUTDIR
             chmod a+rw $OUTDIR/tools-dmcoords1.fits 
	     extn="tools-dmcoords1.fits[1]"
           ;;
    
    ##ST-32: Testing writabilty for various input file types. 
    #.gz files should not be writable
    test3 ) command_string="dmreadpar $INDIR/dmreadpar_input.par \
                        \"$outgz\" \
                        case=upper clobber=yes"
        cp $INDIR/dmreadpar.fits.gz $outgz
      extn="$testid.fits.gz"
	     ;;

    #file:// should be writable
    test4 ) command_string="dmreadpar $INDIR/dmreadpar_input.par \
                        \"file://$outfile[4]\" \
                        case=upper clobber=yes"
        cp $INDIR/dmreadpar.fits $outfile
      extn="$testid.fits"
       ;;


  esac
  if [ ${testid} = "test3" ]; then
      echo $command_string | tee -a  $LOGFILE
      eval $command_string
      mystatus=$?
      echo " -- status of call was $mystatus"  >>  $LOGFILE
      echo " -- status of call was $mystatus"  > ${outgz}.status
  else
    echo $command_string | tee -a  $LOGFILE 
    eval $command_string  | tee -a  $LOGFILE  2>&1
  fi

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1


  ######################################################################
  # ascii files

  #dmlist infile="$OUTDIR/dmreadpar.fits[4]" opt=header \
  #       > $OUTDIR/dmreadpar.${testid}.out
  #dmlist infile="$SAVDIR/dmreadpar.std[4]" opt=header \
  #       > $OUTDIR/dmreadpar.${testid}.std
  #keyfilter $OUTDIR/dmreadpar.${testid}.out $OUTDIR/dmreadpar.${testid}.out2
  #keyfilter $OUTDIR/dmreadpar.${testid}.std $OUTDIR/dmreadpar.${testid}.std2
  #  diff $OUTDIR/dmreadpar.${testid}.out2 \
  #       $OUTDIR/dmreadpar.${testid}.std2 > \
  #       /dev/null 2>>$LOGFILE
  if [ ${testid} = "test3" ]; then 
     # this test checks for expected errors so diff the exit status of run
     diff $outgz.status $savgz.status  > /dev/null 2>>$LOGFILE
     if  test $? -ne 0 ; then
       echo "ERROR: MISMATCH in $outgz.status" >> $LOGFILE
       mismatch=0
     fi
  else
    dmdiff "$OUTDIR/${extn}" "$SAVDIR/${extn}" \
      tol=$SAVDIR/tolerance > /dev/null  2>&1 
    if  test $? -ne 0 ; then
      echo "ERROR: TEXT MISMATCH in $OUTDIR/${extn}" >> $LOGFILE
      mismatch=0
    fi
  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
