#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# glvary.t
# test script for glvary


# !!2
# syntax:
# glvary.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="glvary"

# set up list of tests
# !!4
alltests="test_periodic test_edge test_edge_eff empty bkg preBKG BKG2 emptyProb empty_time"

# "short" test to run
# !!5
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}_prob.fits
  outlite=$OUTDIR/${testid}_lc.fits
  savfile=$SAVDIR/${testid}_prob.fits
  savlite=$SAVDIR/${testid}_lc.fits

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test_periodic ) test1_string="glvary $INDIR/real_period.fits $outfile $outlite eff='' probfile='' clob+"
            ;;

    test_edge ) test1_string="glvary $INDIR/src_at_edge.fits $outfile $outlite eff='' probfile='' clob+"
            ;;

    test_edge_eff ) test1_string="glvary $INDIR/src_at_edge.fits $outfile $outlite eff=$INDIR/src_at_edge_eff.fits probfile='' clob+"
            ;;

    #same as test_periodic except no data in input evt file
    empty ) test1_string="glvary $INDIR/empty.fits $outfile $outlite eff='' probfile='' clob+"
            ;;

    # bug 13501
    empty_time ) test1_string="glvary $INDIR/empty_time.fits $outfile $outlite eff='' probfile='' clob+"
            ;;

    #same as test_edge_eff but with probfile 
    bkg ) test1_string="glvary $INDIR/src_at_edge.fits $outfile $outlite eff=$INDIR/src_at_edge_eff.fits probfile=$INDIR/test_edge_eff_prob.fits clob+"
            ;;

    #same as test_edge but with time and mmin/mmax
    preBKG ) test1_string="glvary $INDIR/src_at_edge.fits"[time=118631480:118631869]" $outfile $outlite eff='' probfile=''  mmin=2 mmax=7 clob+"
            ;;

    #same as prepBKG but with probfile 
    BKG2 ) test1_string="glvary $INDIR/src_at_edge.fits"[time=118631480:118631869]" $outfile $outlite eff='' probfile=$INDIR/preBKG_prob.fits mmin=2 mmax=7 clob+"
            ;;

    #same as preBKG but with probfile that has no data. 
    emptyProb ) test1_string="glvary $INDIR/src_at_edge.fits"[time=118631480:118631869]" $outfile $outlite eff='' probfile=$INDIR/empty_prob.fits  mmin=2 mmax=7 clob+"
            ;;

  esac

  echo $test1_string | tee -a  $LOGFILE 
  eval $test1_string  | tee -a  $LOGFILE  2>&1
 

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
#dmlist $outfile header,data,array > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
#dmlist $savfile header,data,array > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
#keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
#          2>>$LOGFILE

  # compare
  # !!12
#diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
#     /dev/null 2>>$LOGFILE


dmdiff $outfile $savfile tol=$SAVDIR/tolerance error_on_comment- > /dev/null 2>>$LOGFILE
if  test $? -ne 0 ; then
  echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
  mismatch=0
fi

dmdiff $outlite $savlite tol=$SAVDIR/tolerance error_on_comment- > /dev/null 2>>$LOGFILE
if  test $? -ne 0 ; then
  echo "ERROR: MISMATCH in $outlite" >> $LOGFILE
  mismatch=0
fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
#   dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
#   if test $? -ne 0; then
#     echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
#     mismatch=0
#   fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
#   dmdiff $outfile $savfile tol=$SAVDIR/tolerance verb=0 > \
#         /dev/null 2>>$LOGFILE
#   if  test $? -ne 0 ; then
#     echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
#     mismatch=0
#   fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo ""
    echo "${testid} NOT-OK"
    echo ""
    script_succeeded=1
  else
    # No
    echo ""
    echo "${testid} OK"
    echo ""
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo ""
    echo "${toolname} : PASS" | tee -a $LOGFILE
    echo ""
else
    echo ""
    echo "${toolname} : FAIL" | tee -a $LOGFILE
    echo ""
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
