#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.

# output file conventions :
#
#   full_pd.fits     --> infile=hdrPAR_FILE  outfile=hdrDM_FILE  
#   full_pc.fits     --> infile=hdrPAR_FILE  outfile=hdrCFITS_FILE 
#
#   full_dd.fits  --> infile=hdrDM_FILE   outfile=hdrDM_FILE      
#
#   full_cc.fits  --> infile=hdrCFITS_FILE  outfile=hdrCFITS_FILE   
#
# !!1
# tool.t
# test script for tool


# !!2
# syntax:
# tool.t [<testid> ... ]
 

######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, HISTORY, DATASUM, 
#             HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;
  s/ASCDSVE/Dataset/g; 
  s/CHECKSUM/Dataset/g; 
  s/CREATOR/Dataset/g;
  s/DATASUM/Dataset/g;
  s/DATE-OBS/date-obs/g;
  s/DATE-END/date-end/g;
  s/DATE/Dataset/g;
  s/HISTORY/Dataset/g; 
  s/HISTNUM/Dataset/g; 
  s/COMMENT/Dataset/g;
  s/LONGSTRN/Dataset/g;
  s/NAXIS2/Dataset/g; 
  s/EXTNAME/Dataset/g;' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
# executable file = hdrlibtest
toolname="hdrlibtest"
libname="hdrlib"
# set up list of tests
# !!4
alltests="ob_4_pd ob_3_pd ob_2_pd ob_1_pd t_4_pd t_3_pd t_2_pd t_1_pd cc_3_pd cc_2_pd cc_1_pd m_4_pd m_3_pd m_2_pd m_1_pd full_pd none_pd comb1_pd comb2_pd ob_2_pc  ob_1_pc t_1_pc none_pc full_pc comb1_dd none_dd full_dd full2_dd full3_dd full4_dd none_cc full_cc nonsi_pd basic_pd event_pd prim_pd basic_pc event_pc prim_pc basic_dd event_dd event_dd_2 prim_dd basic_cc event_cc prim_cc miscB_pd miscE_pd def_bas_pd def_evt_pd merge1_dd merge2_dd merge3_dd merge4_dd dmmerge_dd merge1_cc merge2_cc merge3_cc dmmerge_cc 62410_dd"

# "short" test to run
# !!5
shortlist="full_dd full2_dd full3_dd full4_dd merge1_dd merge2_dd merge3_dd dmmerge_dd merge1_cc merge2_cc merge3_cc merge4_dd dmmerge_cc 62410_dd"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$libname
SAVDIR=$TESTSAV/$libname
INDIR=$TESTIN/$libname
LOGDIR=$TESTOUT/$libname

THISDIR=`dirname $0`

# set up hdrlibtest.par file
export ASCDS_TEST_PARAM=${THISDIR}/hdrlib
export PDIRS="${UPARM};${ASCDS_SYS_PARAM}:${ASCDS_TEST_PARAM}"
export PFILES="$PDIRS"


# set up log file name
LOGFILE=$LOGDIR/${libname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${libname}_log.*

# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x"; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV   not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool pset
# find_tool punlearn
# find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
#bose=`pget $toolname verbose`
#punlearn $toolname
pset dmlist verbose=0

script_succeeded=0
TESTPROG="${THISDIR}/hdrlib/hdrlibtest"

######################################################################
# Begin per-test loop
#
# infile_type  = 1 (hdrPAR_FILE); 2 (hdrDM_FILE); 3 (hdrCFITS_FILE);        
# outfile_type = 2 (hdrDM_FILE) .or. 3 (hdrCFITS_FILE) ;
#
for testid in $testlist
do
  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  # Set up file names
  pset ${toolname} outfile=${outfile}
  pset ${toolname} infile_type=2
  pset ${toolname} outfile_type=2
  pset ${toolname} infile=${INDIR}/obs.par
  pset ${toolname} lookupTab=${INDIR}/dmmerge_header_lookup.txt

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!
    # !!  (hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ; )
    # !!   infile = obs.par || test_in.fits
    # !!
    # !! The first 19 tests : 
    # !!   infile = obs.par ; 
    # !!   infile_type  = hdrPAR_FILE; 
    # !!   key_compnt_2_put = 1-19 ;
    # !!   outfile_type = hdrDM_FILE ; 
    # !!   testid = ob_4_pd ob_3_pd ob_2_pd ob_1_pd t_4_pd t_3_pd t_2_pd 
    # !!            t_1_pd cc_3_pd c c_2_pd cc_1_pd m_4_pd m_3_pd m_2_pd 
    # !!            m_1_pd full_pd none_pd comb1_pd comb2_pd 
    # !! 
    # !! the test (20-24) is for  
    # !!   infile = obs.par ; 
    # !!   infile_type  = hdrPAR_FILE; 
    # !!   outfile_type=hdrCFITS_FILE
    # !!   key_compnt_2_put = 3, 4, 8, 16, 17
    # !!   ( testid = ob_2_pc  ob_1_pc  t_1_pc  none_pc full_pc )
    # !!
    # !! the test (25-27) is for  
    # !!   infile = dm_in.fits  (  hdrDM_FILE )
    # !!   infile_type  = hdrDM_FILE; 
    # !!   outfile_type=hdrDM_FILE
    # !!   key_compnt_2_put = 18, 16, 17
    # !!   ( testid =  comb1_dd  none_dd full_dd  )
    # !!
    # !! the test (28-29) is for
    # !!   infile = dm_in.fits  
    # !!   infile_type  = hdrCFITS_FILE;
    # !!   outfile_type=hdrCFITS_FILE
    # !!   key_compnt_2_put = 16, 17
    # !!   ( testid =  full_cc  none_cc )
    # !!
    # !! the test (30-32) is for
    # !!   infile = obs.par     
    # !!   infile_type  = hdrPAR_FILE;
    # !!   outfile_type=hdrDM_FILE
    # !!   key_compnt_2_put = 20,21,22
    # !!   ( testid =  nonsi_pd basic_pd  event_pd  prim_pd )
    # !!
    # !! the test (33-35) is for
    # !!   infile = obs.par     
    # !!   infile_type  = hdrPAR_FILE;
    # !!   outfile_type=hdrCFITS_FILE
    # !!   key_compnt_2_put = 20,21,22
    # !!   ( testid =  basic_pc  event_pc  prim_pc )
    # !!
    # !! the test (36-38) is for
    # !!   infile = dm_in.fits
    # !!   infile_type  = hdrDM_FILE;
    # !!   outfile_type=hdrDM_FILE
    # !!   key_compnt_2_put = 20,21,22
    # !!   ( testid =  basic_dd  event_dd  prim_dd )
    # !!
    # !! the test (39-41) is for
    # !!   infile = dm_in.fits
    # !!   infile_type  = hdrCFITS_FILE;
    # !!   outfile_type=hdrCFITS_FILE
    # !!   key_compnt_2_put = 20,21,22
    # !!   ( testid =  basic_cc  event_cc  prim_cc )
    # !!
    # !! the test (42) is for date-obs and date-end 
    # !!   infile = misc.par
    # !!   infile_type  = hdrPAR_FILE
    # !!   outfile_type = hdrDM_FILE
    # !!   key_compnt_2_put = 20 ( basic hdr )
    # !!   ( testid =  miscB_pd )
    # !! 
    # !!   key_compnt_2_put = 21 ( event hdr )
    # !!   ( testid =  miscE_pd )
    # !! 
    # !! the test (43-44) is for set default value if key not found 
    # !!   infile = default.par
    # !!   infile_type  = hdrPAR_FILE
    # !!   outfile_type = hdrDM_FILE
    # !!   key_compnt_2_put = 20,21 ( basic , event hdrs)
    # !!   ( testid =  def_bas_pd def_evt_pd)
    # !!
    # !! merge test will start with 'merge...'
    # !!==============================
    # !!
    # !! infile_type=hdrPAR_FILE outfile_type=hdrDM_FILE
    # !!
    # !! 1  
    # !!
    ob_4_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=1
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=1 > /dev/null"
            echo $test_string | tee -a  $LOGFILE 
            echo "" | tee -a  $LOGFILE
	    eval $test_string
            ;;
    # !! 2
    # !!
    ob_3_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=2
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=2 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 3
    # !!
    ob_2_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=3
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=3 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 4
    # !!
    ob_1_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=4
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=4 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 5
    # !!
    t_4_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=5
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=5 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 6
    # !!
    t_3_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=6
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=6 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 7
    # !!
    t_2_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=7
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=7 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 8
    # !!
    t_1_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=8
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=8 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 9
    # !!
    cc_3_pd  ) pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=9
             test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=9 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 10
    # !!
    cc_2_pd  ) pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=10
             test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=10 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 11
    # !!
    cc_1_pd  ) pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=11
             test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=11 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 12
    # !!
    m_4_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=12
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=12 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 13
    # !!
    m_3_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=13
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=13 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 14
    # !!
    m_2_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=14
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=14 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 15
    # !!
    m_1_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=15
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=15 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 16
    # !!
    # !!  ALL_STS
    full_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=16
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 17
    # !!
    # !!  NONE_STS
    none_pd  )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=17
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=17 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 18
    # !!
    # !!  O_1_STS | T_1_STS | C_1_STS | M_1_STS
    #
    comb1_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=18
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=18 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            echo "O_1_STS | T_1_STS | C_1_STS | M_1_STS" 
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 19
    # !!
    # !!  O_2_STS | C_3_STS
    # !!
    comb2_pd )  pset ${toolname}  infile_type = 1
               pset ${toolname}  key_compnt_2_put=19
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=19 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            echo "O_2_STS | C_3_STS " 
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!   
    # !!
    # !!  C_1_STS | C_2_STS ( intersection of cc_1_pd and cc_2_pd NOT WORK )
    # !!
    ##  comb3_pd ) test_string="${TESTPROG} infile=${INDIR}/obs.par \
    ##         outfile=${outfile} key_compnt_2_put=20 > /dev/null"
    ##      echo $test_string | tee -a  $LOGFILE
    ##      echo "" | tee -a  $LOGFILE
    ##      echo "C_1_STS | C_2_STS "
    ##      echo "" | tee -a  $LOGFILE
    ##      eval $test_string
    ##      ;;
    # !!==============================
    # !!
    # !! infile_type=hdrPAR_FILE outfile_type=hdrCFITS_FILE
    # !!
    # !! 20
    # !!
    # hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;           
    # !!
    ob_2_pc ) pset ${toolname}  infile_type = 1
            pset ${toolname} outfile_type=3 
               pset ${toolname}  key_compnt_2_put=3
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=3 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 21
    # !!
    ob_1_pc ) pset ${toolname}  infile_type = 1
            pset ${toolname} outfile_type=3
               pset ${toolname}  key_compnt_2_put=4
            test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=4 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 22
    # !!
    t_1_pc ) pset ${toolname}  infile_type = 1
           pset ${toolname}  outfile_type=3
               pset ${toolname}  key_compnt_2_put=8
           test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=8 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 23
    # !!
    # !!  ALL_STS
    full_pc ) pset ${toolname}  infile_type = 1
                 pset ${toolname}  outfile_type=3
               pset ${toolname}  key_compnt_2_put=16
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 24
    # !!
    # !!  NONE_STS
    none_pc ) pset ${toolname}  infile_type = 1
                 pset ${toolname}  outfile_type=3
               pset ${toolname}  key_compnt_2_put=17
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=17 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! infile_type=hdrDM_FILE outfile_type=hdrDM_FILE
    # !!
    # !! 25
    # !!
    # !!  O_1_STS | T_1_STS | C_1_STS | M_1_STS
    # !!
    # !!  hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;           
    comb1_dd ) 
            pset ${toolname}  infile_type=2
            pset ${toolname}  outfile_type=2
            pset ${toolname} infile=${INDIR}/dm_in.fits
            pset ${toolname}  key_compnt_2_put=18
            test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
            outfile=${outfile} key_compnt_2_put=18 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            echo "O_1_STS | T_1_STS | C_1_STS | M_1_STS"
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 26
    # !!
    # !!  ALL_STS
    full_dd ) pset ${toolname} infile=${INDIR}/dm_in.fits 
               pset ${toolname}  key_compnt_2_put=16
               test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    full2_dd ) pset ${toolname} infile=${INDIR}/dm_in_2.fits
               pset ${toolname}  key_compnt_2_put=16
                 test_string="${TESTPROG} infile=${INDIR}/dm_in_2.fits \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    full3_dd ) pset ${toolname} infile=${INDIR}/dm_in_3.fits
               pset ${toolname}  key_compnt_2_put=16
                 test_string="${TESTPROG} infile=${INDIR}/dm_in_3.fits \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
	    ;;
    full4_dd ) pset ${toolname} infile=${INDIR}/dm_in_4.fits
               pset ${toolname}  key_compnt_2_put=16
                 test_string="${TESTPROG} infile=${INDIR}/dm_in_4.fits \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
	    ;;


    # !! 27
    # !!
    # !!  NONE_STS
    none_dd ) pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=17
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=17 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! infile_type=hdrCFITS_FILE  outfile_type_type=hdrCFITS_FILE
    # !!
    # !! 28
    # !!
    # !!  ALL_STS
    full_cc ) pset ${toolname} infile_type=3
              pset ${toolname} outfile_type=3
              pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=16
                  test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 29
    # !!
    # !!  NONE_STS
    none_cc ) pset ${toolname} infile_type=3
              pset ${toolname} outfile_type=3
              pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=17
                  test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=17 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! testing the basic + event + primary header
    # !!
    # !! infile_type=hdrPAR_FILE  outfile_type_type=hdrDM_FILE
    # !!
    # !!  hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;
    # !!
# !! 30
    # !!
# !!  NONSI_STS
    nonsi_pd ) pset ${toolname} infile_type=1
               pset ${toolname}  key_compnt_2_put=23
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=23 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 30
    # !!
    # !!  BASIC_STS
    basic_pd ) pset ${toolname} infile_type=1
               pset ${toolname}  key_compnt_2_put=20
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 31
    # !!
    # !!  EVENT_STS
    event_pd ) pset ${toolname} infile_type=1
               pset ${toolname}  key_compnt_2_put=21
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 32
    # !!
    # !!  PRIMARY_STS
    prim_pd ) pset ${toolname} infile_type=1
               pset ${toolname}  key_compnt_2_put=22
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=22 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! testing the basic + event + primary header
    # !!
    # !! infile_type=hdrPAR_FILE  outfile_type_type=hdrCFITS_FILE
    # !!
    # !!  hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;
    # !!
    # !! 33
    # !!
    # !!  BASIC_STS
    basic_pc ) pset ${toolname} infile_type=1
              pset ${toolname} outfile_type=3
               pset ${toolname}  key_compnt_2_put=20
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 34
    # !!
    # !!  EVENT_STS
    event_pc ) pset ${toolname} infile_type=1
              pset ${toolname} outfile_type=3
               pset ${toolname}  key_compnt_2_put=21
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 35
    # !!
    # !!  PRIMARY_STS
    prim_pc ) pset ${toolname} infile_type=1
              pset ${toolname} outfile_type=3
               pset ${toolname}  key_compnt_2_put=22
               test_string="${TESTPROG} infile=${INDIR}/obs.par \
               outfile=${outfile} key_compnt_2_put=22 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! testing the basic + event + primary header
    # !!
    # !! infile_type=hdrDM_FILE  outfile_type_type=hdrDM_FILE
    # !!
    # !!  hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;
    # !!
    # !! 36
    # !!
    # !!  BASIC_STS
    basic_dd ) pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=20
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
                 outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 37
    # !!
    # !!  EVENT_STS
    event_dd ) pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=21
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  EVENT_STS ( no output DATE-OBS DATE-END MJD-OBS
    event_dd_2 ) pset ${toolname} infile=${INDIR}/dm_in_bad.fits
               pset ${toolname}  key_compnt_2_put=21
                 test_string="${TESTPROG} infile=${INDIR}/dm_in_bad.fits \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 38
    # !!
    # !!  PRIMARY_STS
    prim_dd ) pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=22
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=22 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!==============================
    # !!
    # !! testing the basic + event + primary header
    # !!
    # !! infile_type=hdrCFITS_FILE  outfile_type_type=hdrCFITS_FILE
    # !!
    # !!  hdrPAR_FILE=1 ; hdrDM_FILE=2 ; hdrCFITS_FILE=3 ;
    # !!
    # !! 39
    # !!
    # !!  BASIC_STS
    basic_cc ) pset ${toolname} infile_type=3
              pset ${toolname} outfile_type=3
               pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=20
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 40
    # !!
    # !!  EVENT_STS
    event_cc ) pset ${toolname} infile_type=3
              pset ${toolname} outfile_type=3
               pset ${toolname}  key_compnt_2_put=21
               pset ${toolname} infile=${INDIR}/dm_in.fits
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 41
    # !!
    # !!  PRIMARY_STS
    prim_cc ) pset ${toolname} infile_type=3
              pset ${toolname} outfile_type=3
               pset ${toolname} infile=${INDIR}/dm_in.fits
               pset ${toolname}  key_compnt_2_put=22
                 test_string="${TESTPROG} infile=${INDIR}/dm_in.fits \
               outfile=${outfile} key_compnt_2_put=22 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 42
    # !!
    # !!  test dateObsEnd ( 20=basic_hdr )
    # !!
    miscB_pd ) pset ${toolname} infile_type=1
               pset ${toolname} infile=${INDIR}/misc.par
               pset ${toolname}  key_compnt_2_put=20
                 test_string="${TESTPROG} infile=${INDIR}/misc.par \
               outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 42
    # !!
    # !!  test dateObsEnd ( 21=basic_hdr )
    # !!
    miscE_pd ) pset ${toolname} infile_type=1
               pset ${toolname} infile=${INDIR}/misc.par
               pset ${toolname}  key_compnt_2_put=21
                 test_string="${TESTPROG} infile=${INDIR}/misc.par \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 43
    # !!
    # !!  set default if key not found ( 20=basic_hdr )
    # !!
    def_bas_pd ) pset ${toolname} infile_type=1
               pset ${toolname} infile=${INDIR}/default.par
               pset ${toolname}  key_compnt_2_put=20
                 test_string="${TESTPROG} infile=${INDIR}/default.par \
               outfile=${outfile} key_compnt_2_put=20 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 44
    # !!
    # !!  set default if key not found ( 21=event_hdr )
    # !!
    def_evt_pd ) 
              pset ${toolname} infile_type=1
              pset ${toolname} infile=${INDIR}/default.par
               pset ${toolname}  key_compnt_2_put=21
               test_string="${TESTPROG} infile=${INDIR}/default.par \
               outfile=${outfile} key_compnt_2_put=21 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !! 44
    # !!
    # !!  test unsigned dtype for DM lib (ref. full_dd )
    # !!
    acis_2469_dd ) 
            pset ${toolname} infile=${INDIR}/acis_2469_in.fits
            pset ${toolname} key_compnt_2_put=16
            test_string="${TESTPROG} infile=${INDIR}/acis_2469_in.fits \
                   outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # || -------begin :  merge test ---------------------------
    # !! 45
    # !!  merge test; ref to full_dd ;
    # !!  diff. order of infiles in merge1_dd,2,3 ;
    # !! 
    merge1_dd ) pset ${toolname} infile="@${INDIR}/merge1.lis"
             pset ${toolname} key_compnt_2_put=16
               pset ${toolname}  key_compnt_2_put=16
               pset ${toolname}  infile_type=2
               pset ${toolname}  outfile_type=2
               test_string="${TESTPROG} infile=\"@${INDIR}/merge1.lis\"  \
             outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to full2_dd ;
    # !!  diff. order of infiles in merge1_dd,2,3 ;
    # !! 
    merge2_dd ) pset ${toolname} infile="@${INDIR}/merge2.lis"
             pset ${toolname} key_compnt_2_put=16
               test_string="${TESTPROG} infile=\"@${INDIR}/merge2.lis\"  \
             outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to full3_dd ;
    # !!  diff. order of infiles in merge1_dd,2,3 ;
    # !!
    merge3_dd ) pset ${toolname} infile="@${INDIR}/merge3.lis"
             pset ${toolname} key_compnt_2_put=16
               test_string="${TESTPROG} infile=\"@${INDIR}/merge3.lis\"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to full4_dd ;
    # !!  diff. order of infiles in merge1_dd,2,3,4 ;
    # !!
    merge4_dd ) pset ${toolname} infile="@${INDIR}/merge4.lis"
             pset ${toolname} key_compnt_2_put=16
               test_string="${TESTPROG} infile=\"@${INDIR}/merge4.lis\"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to dmmerge_dd reg. test ;
    # !!  test TSTART/TSTOP/DATE-OBS/DATE-END/EQUINOX
    # !!
    dmmerge_dd ) pset ${toolname} infile="@${INDIR}/dmmergeIn.lis "
             pset ${toolname} key_compnt_2_put=16
               test_string="${TESTPROG} infile=\"@${INDIR}/dmmergeIn.lis\"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;

    # || -------begin :  merge test (CFITS) ---------------------------
    # !!
    # !!  merge test; ref to full_dd ;
    # !!  diff. order of infiles in merge1_cc,2,3 ;
    # !! 
    merge1_cc ) pset ${toolname} infile="@${INDIR}/merge1.lis"
             pset ${toolname} key_compnt_2_put=16
               pset ${toolname}  key_compnt_2_put=16
               pset ${toolname}  infile_type=3
               pset ${toolname}  outfile_type=3
               test_string="${TESTPROG} infile=\"@${INDIR}/merge1.lis\"  \
             outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to full2_cc ;
    # !!  diff. order of infiles in merge1_cc,2,3 ;
    # !! 
    merge2_cc ) pset ${toolname} infile="@${INDIR}/merge2.lis"
             pset ${toolname} key_compnt_2_put=16
               pset ${toolname}  infile_type=3
               pset ${toolname}  outfile_type=3
               test_string="${TESTPROG} infile=\"@${INDIR}/merge2.lis\"  \
             outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to full3_cc ;
    # !!  diff. order of infiles in merge1_cc,2,3 ;
    # !!
    merge3_cc ) pset ${toolname} infile="@${INDIR}/merge3.lis"
             pset ${toolname} key_compnt_2_put=16
               pset ${toolname}  infile_type=3
               pset ${toolname}  outfile_type=3
               test_string="${TESTPROG} infile=\"@${INDIR}/merge3.lis\"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; ref to dmmerge_cc reg. test ;
    # !!  test TSTART/TSTOP/DATE-OBS/DATE-END/EQUINOX
    # !!
    dmmerge_cc ) pset ${toolname} infile="@${INDIR}/dmmergeIn.lis"
             pset ${toolname} key_compnt_2_put=16
               pset ${toolname}  infile_type=3
               pset ${toolname}  outfile_type=3
               test_string="${TESTPROG} infile=\"@${INDIR}/dmmergeIn.lis\"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  ONTIME/LIVETIME/EXPOSURE ; ref to obsid_62410 reg. test ;
    # !!
    62410_dd ) pset ${toolname} infile="${INDIR}/62410_in.fits"
             pset ${toolname} key_compnt_2_put=16
               test_string="${TESTPROG} infile="${INDIR}/62410_in.fits"  \
               outfile=${outfile} key_compnt_2_put=16 > /dev/null"
            echo $test_string | tee -a  $LOGFILE
            echo "" | tee -a  $LOGFILE
            eval $test_string
            ;;
    # !!
    # !!  merge test; need to add test for input/output_filetype=hdrCFITS_FILE=3
    # !!
    # || -------end :  merge test ---------------------------
    # !!==============================
  esac


  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0           
  fi

  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  # new output
  # !!10
  dmlist $outfile header,raw,all,clean > $OUTDIR/${testid}.dmp1  2>>$LOGFILE 
  keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # reference output
  # !!11
  dmlist $savfile header,raw,all,clean > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
            2>>$LOGFILE

  # compare
  # !!12
  diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > /dev/null 2>>$LOGFILE

  if  test $? -ne 0 ; then
    echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
    mismatch=0
  fi
  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK" | tee -a $LOGFILE
    script_succeeded=1
  else
    # No
    echo "${testid} OK" | tee -a $LOGFILE
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
