#! /bin/sh

# 30 January 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]
 
# Developer's comments:
# lack of comparison of the higher extensions of outputs (FITS)
#
######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/WasteDataset/g;s/COMMENT/WasteDataset/g;
  s/DATE/WasteDataset/g;s/CREATOR/WasteDataset/g;s/HISTORY/WasteDataset/g; 
  s/DATASUM/WasteDataset/g;s/ASCDSVER/WasteDataset/g;s/HISTNUM/WasteDataset/g; 
  s/CVSD0001/WasteDataset/g; s/CVST0001/WasteDataset/g; s/FEFFILE/WasteDataset/g' | \
  grep -v WasteDataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}
######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="mkrmf"

# set up list of tests
# !!4
alltests="3207 wrmf_legacy wrmf_cxc piwrmf_legacy cti_fi_pha cti_fi_pi cti_bi_pha cti_bi_pi wrmf_caldb_pi wrmf_caldb_pi_p"

# "short" test to run
# !!5
#shortlist="wrmf_caldb_pi"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi



# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an o/data/regression_test/development/indata/mkrmf/6128.wgtutput directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=$SAVDIR/${testid}.fits

  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
   case ${testid} in

    # !!8
    3207) command_str="${toolname} \
	infile='CALDB(CTI_APP=PPPPPNPNPP)'\
	outfile=$outfile  \
	weights=${INDIR}/sources_3207.wgt \
	axis1=\"energy=0.1:11:0.01\" \
	axis2=\"pi=1:1024:1\" mode=h> /dev/null"
    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
           ;;

    # !!8
    wrmf_legacy) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_piN0001.fits \
	outfile=$outfile  \
	weights=${INDIR}/weight_small.fits \
	axis1=\"energy=grid( ${INDIR}/weight_small.fits[EBOUNDS] )\" \
	axis2=\"pi=1:1024:1\" \
	logfile=\"\"  outfmt=legacy thresh=1.e-5 clobber=yes > /dev/null"
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
           ;;

    # !!9
    wrmf_cxc) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_piN0001.fits \
	outfile=$outfile  \
	weights=${INDIR}/weight_small.fits \
	axis1=\"energy=grid( ${INDIR}/weight_small.fits[EBOUNDS] )\" \
	axis2=\"pi=1:1024:1\" \
	logfile=\"\"  outfmt=cxc thresh=1.e-5 clobber=yes > /dev/null"
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
           ;;

    # !20
    piwrmf_legacy) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_phaN0002.fits \
	outfile=$outfile  \
	weights=${INDIR}/weight_pha_4reg.fits \
	axis1=\"energy=grid( ${INDIR}/weight_pha_4reg.fits[EBOUNDS] )\" \
	axis2=\"pi=1:1024:1\" \
	logfile=\"\"  outfmt=legacy thresh=1.e-5 clobber=yes > /dev/null"
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_fi_pha) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=3,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
	outfile=$outfile \
	axis1=energy=0.5:9.5:0.01 \
	axis2=pha=1:4096:4 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_fi_pi) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=3,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
        outfile=$outfile \
        axis1=energy=0.5:9.5:0.01 \
        axis2=pi=1:1024:1 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;


    cti_bi_pha) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=7,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
	outfile=$outfile \
	axis1=energy=0.5:9.5:0.01 \
	axis2=pha=1:4096:4 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_bi_pi) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=7,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
        outfile=$outfile \
        axis1=energy=0.5:9.5:0.01 \
        axis2=pi=1:1024:1 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_fi_pha) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=3,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
	outfile=$outfile \
	axis1=energy=0.5:9.5:0.01 \
	axis2=pha=1:4096:4 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_fi_pi) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=3,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
        outfile=$outfile \
        axis1=energy=0.5:9.5:0.01 \
        axis2=pi=1:1024:1 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;


    cti_bi_pha) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=7,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
	outfile=$outfile \
	axis1=energy=0.5:9.5:0.01 \
	axis2=pha=1:4096:4 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    cti_bi_pi) command_str="${toolname} \
	infile=${INDIR}/acisD2000-01-29fef_pha_ctiN0004.fits\"[ccd_id=7,chipx_lo<=340,chipx_hi>=340,chipy_lo<=900,chipy_hi>=900]\" \
        outfile=$outfile \
        axis1=energy=0.5:9.5:0.01 \
        axis2=pi=1:1024:1 "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    wrmf_caldb_pi) command_str="${toolname} \
	infile='CALDB(DETNAM=)' \
        outfile=$outfile \
        axis1=energy=1.0:1.001:0.1\
        axis2=pi=1:1024:1 \
        weights=${INDIR}/6128.wgt "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;

    wrmf_caldb_pi_p) command_str="${toolname} \
	infile='CALDB(DETNAM=)' \
        outfile=$outfile \
        axis1=energy=1.0:1.001:0.1\
        axis2=pi=69:128:1 \
        weights=${INDIR}/6128.wgt "

    punlearn mkrmf
    echo $command_str  | tee -a $LOGFILE
    eval $command_str  | tee -a $LOGFILE  2>&1
	    ;;
 esac



  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

   dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
   if  test $? -ne 0 ; then
     echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
     mismatch=0
   fi
   dmdiff ${outfile}"[EBOUNDS]" ${savfile}"[EBOUNDS]" tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
   if  test $? -ne 0 ; then
     echo "ERROR: MISMATCH in ${outfile}[EBOUNDS]" >> $LOGFILE
     mismatch=0
   fi


  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
