#! /bin/sh

# 07 March 2002

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# roi.t
# test script for roi


# !!2
# syntax:
# roi.t [<testid> ... ]




######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM,
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
  punlearn $1
}

######################################################################
# subroutine
# zerotest <file>
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="roi"

# set up list of tests
# !!4
alltests="grow_excl_src test_group test_indi test_exclude group_all indi_all indi_targ excl_all excl_targ skip_conf fov_group all_STREAK c_all_STREAK skip_reg test_group2 mask_fov mask_strk test_polys test_non_supported_shapes test_empty_set test_circle"

# "short" test to run
# !!5
shortlist="$alltests"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined"
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on
# both the output and reference files of a tool the resultant utility output
# files will both exist and be empty, and will pass a diff.

find_tool dmlist
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do

  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile="$OUTDIR/${testid}_%04d"
  savfile=$SAVDIR/${testid}.fits

  echo "" | tee -a $LOGFILE
  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    test_group ) test_string="roi infile=$INDIR/srclist.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=3 group=group targetbkg=all fovregion= streakregion=  streakres=0.25  ignore_streaksrc=yes srcfun=add srcfactor=0"
            ;;

    test_indi ) test_string="roi infile=$INDIR/srclist.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=3 group=indi targetbkg=all fovregion= streakregion=  streakres=0.25   ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;

    test_exclude ) test_string="roi infile=$INDIR/srclist.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=3 group=exclude targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;

    group_all ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=group targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;
    indi_all ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=indi targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;
    indi_targ ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=indi targetbkg=targ fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;
    excl_targ ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=excl targetbkg=targ fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0  "
            ;;
    excl_all ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=excl targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 "
            ;;
    skip_conf ) test_string="roi infile=$INDIR/simple.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=excl targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 compute_conf=no"
            ;;
    fov_group ) test_string="roi infile=$INDIR/obsid_635_srcs.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=3 group=group fovregion=\"region($INDIR/fov.reg)\" streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0 compute_conf=yes "
            ;;
 # new streak region file
 all_STREAK ) test_string="roi \
  infile=$INDIR/acisf00635_000N001_merge_src3.fits\"[#row=1:10]\" \
  fovregion=\"region($INDIR/acisf00635_000N001_fov3.fits)\" \
  outsrcfile=${outfile}_srcreg.fits \
  radiusmode='mul' bkgradius='5' group='exclude' targetbkg='target' \
  maxpix='INDEF' fovres='1' clobber='yes'  \
  streakregion=\"region($INDIR/acisf00635_000N001_B_streak3.reg)\" \
  streakres=0.25  ignore_streaksrc=no srcfun=add srcfactor=0"
             ;;
 # same as all_STREAK except ignore_streaksrc=yes
 c_all_STREAK ) test_string="roi \
  infile=$INDIR/acisf00635_000N001_merge_src3.fits\"[#row=1:10]\" \
  fovregion=\"region($INDIR/acisf00635_000N001_fov3.fits)\" \
  outsrcfile=${outfile}_srcreg.fits \
  radiusmode='mul' bkgradius='5' group='exclude' targetbkg='target' \
  maxpix='INDEF' fovres='1' clobber='yes'  \
  streakregion=\"region($INDIR/acisf00635_000N001_B_streak3.reg)\" \
  streakres=0.25  ignore_streaksrc=yes srcfun=add srcfactor=0"
             ;;
 # nonexisting fov and streak regions
 skip_reg ) test_string="roi \
  infile=$INDIR/acisf00635_000N001_merge_src3.fits\"[#row=1:10]\" \
  fovregion=\"region($INDIR/nonexist.fits)\" \
  outsrcfile=${outfile}_srcreg.fits \
  radiusmode='mul' bkgradius='5' group='exclude' targetbkg='target' \
  maxpix='INDEF' fovres='1' clobber='yes'  \
  streakregion=\"region($INDIR/nonexist.reg)\" \
  streakres=0.25  ignore_streaksrc=no srcfun=add srcfactor=0"
             ;;
 #BUGFIX
 #Test data from L3 testing, cohort 512 (orion nebula)
  test_group2 ) test_string="roi infile=$INDIR/acisfJ0535130m052259_002N020_b0381_LargeRadius_merge_src3_tmpA.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=3 group=group targetbkg=all fovregion= streakregion=  streakres=0.25  ignore_streaksrc=yes srcfun=add srcfactor=0"
            ;;
 #BUGFIX PTPL-16
  test_polys ) test_string="roi infile=$INDIR/mix_ellipse_polygons_srcreg.fits outsrcfile=${outfile}_srcreg.fits clobber+ mode=h radius=mul bkgrad=3 group=group targetbkg=all fovregion= streakregion= streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0" 
            ;;
  test_non_supported_shapes ) test_string="roi infile=$INDIR/non_supported_shapes.fits outsrcfile=${outfile}_srcreg.fits clobber+ mode=h radius=mul bkgrad=3 group=group targetbkg=all fovregion= streakregion= streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0" 
            ;;
  test_empty_set ) test_string="roi infile=$INDIR/empty_list_srcreg.fits outsrcfile=${outfile}_srcreg.fits clobber+ mode=h radius=mul bkgrad=3 group=group targetbkg=all fovregion= streakregion= streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0" 
            ;;

 #Test ENH# to allow increase in radius of excluded overlapping sources
  grow_excl_src ) test_string="roi infile=$INDIR/grow_excl_src.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=5 group=exclude targetbkg=target fovregion= streakregion=  streakres=0.25  ignore_streaksrc=yes srcfun=mul srcfactor=1.1 bkgfun=mul bkgfactor=1.1 bkgfactarg=all"
            ;;

 #PTOBST-11: Reject image mask in fovregion and streakregion
 #same as c_all_streak but with mask instead of region
 #mask in fovregion
 mask_fov ) test_string="roi \
  infile=$INDIR/acisf00635_000N001_merge_src3.fits\"[#row=1:10]\" \
  fovregion=\"mask($INDIR/acisf00635_000N001_fov3.fits)\" \
  outsrcfile=${outfile}_srcreg.fits \
  radiusmode='mul' bkgradius='5' group='exclude' targetbkg='target' \
  maxpix='INDEF' fovres='1' clobber='yes'  \
  streakregion=\"region($INDIR/acisf00635_000N001_B_streak3.reg)\" \
  streakres=0.25  ignore_streaksrc=yes srcfun=add srcfactor=0"
             ;;

  #mask in streakregion
  mask_strk ) test_string="roi \
   infile=$INDIR/acisf00635_000N001_merge_src3.fits\"[#row=1:10]\" \
   fovregion=\"region($INDIR/acisf00635_000N001_fov3.fits)\" \
   outsrcfile=${outfile}_srcreg.fits \
   radiusmode='mul' bkgradius='5' group='exclude' targetbkg='target' \
   maxpix='INDEF' fovres='1' clobber='yes'  \
   streakregion=\"mask($INDIR/acisf00635_000N001_B_streak3.reg)\" \
   streakres=0.25  ignore_streaksrc=yes srcfun=add srcfactor=0"
              ;;

    test_circle ) test_string="roi infile=$INDIR/simple_circle.fits outsrcfile=${outfile}_srcreg.fits radius=mul bkgrad=2 group=excl targetbkg=all fovregion= streakregion=  streakres=0.25 ignore_streaksrc=yes srcfun=add srcfactor=0"

   esac

  echo $test_string | tee -a  $LOGFILE
  eval $test_string | tee -a  $LOGFILE  2>&1


  ####################################################################
  # check the outputs

  # Init per-test error flag
  smismatch=1
  bmismatch=1

  # if different tests need different kinds of comparisons, use a
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  case ${testid} in

# The executable roi produces no output on test_non_supported_shapes, test_empty_set.
# The command line `\ls $OUTDIR/${testid}*.fits` raises an error, so the tests without
# output are encased in a case statement, and all else runs in the case else section.

    test_non_supported_shapes | test_empty_set )  
       if  test $? -eq 0 ; then
         smismatch=0
         bmismatch=0
       fi
            ;;
    *) 

  for outfile in `\ls $OUTDIR/${testid}*.fits`
  do
    savfile=`echo $outfile | tr "/" " " | awk '{print $NF}'`


    #---- SRCREG
    dmdiff $outfile"[SRCREG]" $SAVDIR/$savfile"[SRCREG]" tol=$SAVDIR/tolerance \
      > /dev/null 2>>$LOGFILE 
    
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile[SRCREG]" >> $LOGFILE
      ###echo dmdiff $outfile'"[SRCREG]"' $SAVDIR/$savfile'"[SRCREG]"' tol=$SAVDIR/tolerance

      smismatch=1
    else
      smismatch=0
    fi

    #---- BKGREG
    dmdiff $outfile"[BKGREG]" $SAVDIR/$savfile"[BKGREG]" tol=$SAVDIR/tolerance \
      > /dev/null 2>>$LOGFILE 
    if  test $? -ne 0 ; then
      echo "ERROR: MISMATCH in $outfile[BKGREG]" >> $LOGFILE
      bmismatch=1
    else
      bmismatch=0
    fi

  done
  ;;

  esac


  ####################################################################
  # Did we get an error?
  echo ""
  if test $smismatch -eq 1 ; then
    # Yes
    echo "${testid} src NOT-OK"
    script_succeeded=1
  else
    if test $bmismatch -eq 1; then
    # Yes
      echo "${testid} bkg NOT-OK"
      script_succeeded=1
    else
      # No
      echo "${testid} OK"
    fi
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
