#!/bin/sh

##########################################################
# test script to run the stk_build code and generate
# the Test documentation
#
# IMPORTANT:  IF MAKING ANY CHANGES TO THE TEST CASES
#     THERE ARE 2 LINES THAT MUST BE UNCOMMENTED
#     - FIRST LINE WILL ALLOW OUTPUT FILES TO BE WRITTEN TO
#       THE SAVE DIRECTORY
#     - SECOND LINE WILL ALLOW THE TEST DOCUMENT IN THE
#       UDF TO BE MODIFIED
#     THESE 2 LINES ARE PRECEDED WITH THE LINE:
#       "THE FOLLOWING LINE SHOULD BE COMMENTED"
#     ALSO BE SURE TO CHANGE THE NUMBER OF REFERENCE
#      FILES AND INPUT FILES IN:
#        verify_reference_files()
#        verify_input_files()
#
# Joseph Masters,  24 Aug 2000  -  Last Modified 9/8/2000
############################################################

# UNCOMMENT THE FOLLOWING LINE FOR DEBUGGING
# set -x

#######################################################
# set the date, environment variables, etc.
#######################################################
setup()
{
  echo
  echo "STK_BUILD REGRESSION TEST"
  echo

  DATE=`date +'%y%m%d_%T'`
  exit_flag=0

  # TESTIN  is where the test data resides
  # TESTOUT is where output goes
  # TESTLOG is where the log output goes
  # TESTSAV is where reference output exists

  # check to see if the necessary environment variables were set
  if [ "$TESTIN" = "" ]; then
    echo "Environment variable \$TESTIN is not set."
    exit_flag=1
  fi
  if [ "$TESTOUT" = "" ]; then
    echo "Environment variable \$TESTOUT is not set."
    exit_flag=1
  fi
  if [ "$TESTLOG" = "" ]; then
    echo "Environment variable \$TESTLOG is not set."
    exit_flag=1
  fi
  if [ "$TESTSAV" = "" ]; then
    echo "Environment variable \$TESTSAV is not set."
    exit_flag=1
  fi

  # if the exit flag was set, then exit
  if [ $exit_flag = 1 ]; then
    exit 1
  fi
  
  # set the environment variable to local variables
  indir=${TESTIN}/stk_build
  outdir=${TESTOUT}/stk_build
  savdir=${TESTSAV}/stk_build
  logdir=${TESTLOG}/stk_build

  LOGFILE=$logdir/stk_build_log.${DATE}

  # Make sure we have a log directory
  if test -d $logdir ; then
   :
  else
    mkdir -p $logdir 
    if test $? -ne 0 ; then
      error_exit ""
    fi
  fi

  # Make sure we have an output directory
  if test -d $outdir ; then
   :
  else
    mkdir -p $outdir >> $LOGFILE 2>&1
    if test $? -ne 0 ; then
      error_exit "can't create output directory $outdir"
    fi
  fi

  echo "Input directory     = "$indir  | tee -a $LOGFILE
  echo "Output directory    = "$outdir | tee -a $LOGFILE
  echo "Reference directory = "$savdir | tee -a $LOGFILE
  echo "Log directory       = "$logdir | tee -a $LOGFILE
  echo

  # temporary path for stk_build Test document
  temp_udf_test=$outdir/stk_build_Test

  # path for stk_build Test document
  udf_test=/stage/ascds_extra/staff/UDF/Tools/flight/stktools/stk_build/Test
}

######################################################################
#  verify that all input files exist                                 #
######################################################################
verify_input_files()
{
  for FILE_NUM in 1 2
  do
    if test -f $indir/stk_build.lis$FILE_NUM; then
      :
    else
      echo "Input file $FILE_NUM does not exist:"| tee -a $LOGFILE
      echo " Missing $indir/stk_build.lis$FILE_NUM"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All input files exist" | tee -a $LOGFILE
}

######################################################################
#  verify that all saved reference files exist                       #
######################################################################
verify_reference_files()
{
  for FILE_NUM in 1 2 3 4 5 6 7 8 9
  do
    if test -f $savdir/stk_build$FILE_NUM.out; then
      :
    else
      echo "Reference file $FILE_NUM does not exist:"| tee -a $LOGFILE
      echo " Missing $savdir/stk_build$FILE_NUM.out"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All reference files exist" | tee -a $LOGFILE
  echo
}

####################################
# delete old output files          #
####################################
delete_old_outputs()
{
  rm -f $outdir/stk_build*
}

###############################################################################
# generate the header for the Test document in the UDF                        #
###############################################################################
write_test_doc_header()
{
   echo "Testing Status" > $temp_udf_test
   echo "Date: 24 August 2000" >> $temp_udf_test

   echo "Last Modified: `date +'%d %B %Y'`" >> $temp_udf_test
   echo "Author: Joseph Masters" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Regression test:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Testing is run by the regression script stk_build.t" >> $temp_udf_test
   echo "The input data is in $indir" >> $temp_udf_test
   echo "The script runs the following test cases:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo >> $temp_udf_test
}

##########################################################################
# copy the output file to the "save directory" for a later comparison    #
##########################################################################
copy_output()
{
  # when a change is made in the test cases
  #  the output files need to be copied
  #  over to the TESTSAV directory

  # THE FOLLOWING LINE SHOULD BE COMMENTED
  # cp $outdir/stk_build$ii.out $savdir/stk_build$ii.out

  echo > /dev/null  # keep this line, function must execute a statement
}


###############################################################################
# compare the saved and current outputs                                       #
###############################################################################
compare_outputs()
{
  # compare the current output with the saved output.
  # if there is a difference between the two,
  # tell the user that the test fails.

  diff $outdir/stk_build$ii.out $savdir/stk_build$ii.out

  if [ $? -eq 0 ]; then
      echo "Test # $ii OK" | tee -a $LOGFIL
      echo
  else
      tool_status=FAIL
      echo "Test # $ii NOT-OK" | tee -a $LOGFILE
      echo "     saved and latest outputs do not match for test# $ii" | tee -a $LOGFILE
      echo
  fi
}

###############################################
# write_test_doc                              #
###############################################
write_test_doc()
{
  echo "TEST #$ii:" >> $temp_udf_test
  printf " $summary\n" >> $temp_udf_test
  printf " $command\n" >> $temp_udf_test
  echo >> $temp_udf_test
}

  
##########################
# increase count         #
##########################
increase_count()
{
  ii=`expr $ii + 1`
}


###########################################################################
# generate the trailer for the test document                              #
###########################################################################
write_test_doc_trailer()
{
   printf "~make test~ will run the test script AND CAN BE USED TO REGENERATE THIS FILE.\n" >> $temp_udf_test
}


##########################
# cleanup                #
##########################
cleanup()
{
  # copy the temporary Test document to the more permanent Test document
  # THE FOLLOWING LINE SHOULD BE COMMENTED
  #  cp $temp_udf_test $udf_test

  # remove the temporary Test document
    rm $temp_udf_test

    printf "\nstk_build: $tool_status\n" | tee -a $LOGFILE
}



#################
#---------------#
#   MAIN CODE   #
#---------------#
#################

ii=1        # used for test count
tool_status=PASS

# set the date, environment variables, etc.
setup

verify_input_files
verify_reference_files
delete_old_outputs
write_test_doc_header

# run each test

# 1 #
stk_build infile="@$indir/stk_build.lis1" outfile=stdout clobber=no verbose=0 > $outdir/stk_build$ii.out
summary="This test builds a stack from a single file and sends the output to stdout."
command="stk_build infile=\"@$indir/stk_build.lis1\" outfile=stdout clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 2 #
stk_build infile="stk_build.lis1" outfile=stderr clobber=no verbose=0 2> $outdir/stk_build$ii.out
summary="This test builds a stack from a single filename without an @\n and sends output to stderr."
command="stk_build infile=/"stk_build.lis1/" outfile=stderr clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 3 #
stk_build infile="one two three" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test builds a stack from the infile string to contain three elements.\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"one two three\" outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 4 #
stk_build infile="@$indir/stk_build.lis1[time=123:876]" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test builds a stack from a single file with an extension.\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"@$indir/stk_build.lis1[time=123:876]\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 5 #
stk_build infile="@$indir/stk_build.lis1 @$indir/stk_build.lis2" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test builds a stack from two files.\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"@$indir/stk_build.lis1 @$indir/stk_build.lis2\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 6 #
stk_build infile="$indir/realdir/" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test check the output for a path of directory that does not exist.\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"$indir/realdir/\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count


# 7 #
stk_build infile="$indir/realdir1/" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test check the output for a path of directory that exist.\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"$indir/realdir1/\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

# 8 #
stk_build infile="$indir/realdir" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test check the output for a path of directory that exist (no slash at end).\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"$indir/realdir\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count


# 9 #
stk_build infile="$indir/realdir1/*.fits" outfile="$outdir/stk_build$ii.out" clobber=no verbose=0
summary="This test check the output for a files with matching string .\n The output is sent to $outdir/stk_build$ii.out"
command="stk_build infile=\"$indir/realdir1/*.fits\"\n    outfile=\"$outdir/stk_build$ii.out\" clobber=no verbose=0"
echo ${command}
copy_output
compare_outputs
write_test_doc
increase_count

write_test_doc_trailer
cleanup
echo
exit 0
