#!/bin/sh

##########################################################
# test script to run the stk_read_num code and generate
# the Test documentation
#
# IMPORTANT:  IF MAKING ANY CHANGES TO THE TEST CASES
#     THERE ARE 2 LINES THAT MUST BE UNCOMMENTED
#     - FIRST LINE WILL ALLOW .PAR FILES TO BE WRITTEN TO
#       THE SAVE DIRECTORY
#     - SECOND LINE WILL ALLOW THE TEST DOCUMENT IN THE
#       UDF TO BE MODIFIED
#     THESE 2 LINES ARE PRECEDED WITH THE LINE:
#       "THE FOLLOWING LINE SHOULD BE COMMENTED"
#     ALSO BE SURE TO CHANGE THE NUMBER OF REFERENCE
#      FILES AND INPUT FILES IN:
#        verify_reference_files()
#        verify_input_files()
#
# Joseph Masters,  24 Aug 2000  - Last Modified 9/8/2000
############################################################

# UNCOMMENT THE FOLLOWING LINE FOR DEBUGGING
# set -x

#######################################################
# set the date, environment variables, etc.
#######################################################
setup()
{
  echo
  echo "STK_READ_NUM REGRESSION TEST"
  echo

  DATE=`date +'%y%m%d_%T'`
  exit_flag=0

  # TESTIN  is where the test data resides
  # TESTOUT is where output goes
  # TESTLOG is where the log output goes
  # TESTSAV is where reference output exists

  # check to see if the necessary environment variables were set
  if [ "$TESTIN" = "" ]; then
    echo "Environment variable \$TESTIN is not set."
    exit_flag=1
  fi
  if [ "$TESTOUT" = "" ]; then
    echo "Environment variable \$TESTOUT is not set."
    exit_flag=1
  fi
  if [ "$TESTLOG" = "" ]; then
    echo "Environment variable \$TESTLOG is not set."
    exit_flag=1
  fi
  if [ "$TESTSAV" = "" ]; then
    echo "Environment variable \$TESTSAV is not set."
    exit_flag=1
  fi

  # if the exit flag was set, then exit
  if [ $exit_flag = 1 ]; then
    exit 1
  fi
  
  # set the environment variable to local variables
  indir=${TESTIN}/stk_read_num
  outdir=${TESTOUT}/stk_read_num
  savdir=${TESTSAV}/stk_read_num
  logdir=${TESTLOG}/stk_read_num

  LOGFILE=$logdir/stk_read_num_log.${DATE}

  # Make sure we have a log directory
  if test -d $logdir ; then
   :
  else
    mkdir -p $logdir 
    if test $? -ne 0 ; then
      error_exit ""
    fi
  fi

  # Make sure we have an output directory
  if test -d $outdir ; then
   :
  else
    mkdir -p $outdir >> $LOGFILE 2>&1
    if test $? -ne 0 ; then
      error_exit "can't create output directory $outdir"
    fi
  fi

  echo "Input directory     = "$indir  | tee -a $LOGFILE
  echo "Output directory    = "$outdir | tee -a $LOGFILE
  echo "Reference directory = "$savdir | tee -a $LOGFILE
  echo "Log directory       = "$logdir | tee -a $LOGFILE
  echo

  # temporary path for stk_read_num Test document
  temp_udf_test=$outdir/stk_read_num_Test

  # path for stk_read_num Test document
  udf_test=/stage/ascds_extra/staff/UDF/Tools/flight/stktools/stk_read_num/Test
}

######################################################################
#  verify that all input files exist                                 #
######################################################################
verify_input_files()
{
  for FILE_NUM in 1 2
  do
    if test -f $indir/stk_read_num.lis$FILE_NUM; then
      :
    else
      echo "Input file $FILE_NUM does not exist:"| tee -a $LOGFILE
      echo " Missing $indir/stk_read_num.lis$FILE_NUM"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All input files exist" | tee -a $LOGFILE
}

######################################################################
#  verify that all saved reference files exist                       #
######################################################################
verify_reference_files()
{
  for FILE_NUM in 1 2 3 4 5 6 7 8
  do
    if test -f $savdir/stk_read_num$FILE_NUM.par; then
      :
    else
      echo "Reference file $FILE_NUM does not exist:"| tee -a $LOGFILE
      echo " Missing $savdir/stk_read_num$FILE_NUM.par"| tee -a $LOGFILE
      echo
      exit 1
    fi
  done

  echo "All reference files exist" | tee -a $LOGFILE
  echo
}

####################################
# delete old output files          #
####################################
delete_old_outputs()
{
  rm -f $outdir/stk_read_num*
}

###############################################################################
# generate the header for the Test document in the UDF                        #
###############################################################################
write_test_doc_header()
{
   echo "Testing Status" > $temp_udf_test
   echo "Date: 24 August 2000" >> $temp_udf_test
   echo "Last Modified: 25 August 2000" >> $temp_udf_test
   echo "Author: Joseph Masters" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Regression test:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo "Testing is run by the regression script stk_read_num.t" >> $temp_udf_test
   echo "The input data is in $indir" >> $temp_udf_test
   echo "The script runs the following test cases:" >> $temp_udf_test
   echo >> $temp_udf_test
   echo >> $temp_udf_test
}

##########################################################################
# copy the paramater file from the "save directory" for comparison       #
##########################################################################
copy_param()
{
  # baseline parameter file needs to be writable for pquery command
  # copy to output directory to allow this.

  cp $savdir/stk_read_num$ii.par $outdir/sav_stk_read_num$ii.par

}

###############################################################################
# do plists on saved and current parameter files and compare                  #
###############################################################################
compare_outputs()
{
  pquery stk_read_num outelement > $outdir/stk_read_num$ii.latest
  #pquery $savdir/stk_read_num$ii.par outelement > $outdir/stk_read_num$ii.saved
  pquery $outdir/sav_stk_read_num$ii.par outelement > $outdir/stk_read_num$ii.saved

  # compare the current parameter output with the saved parameter output.
  # if there is a difference between the two,
  # tell the user that the test fails.

  diff $outdir/stk_read_num$ii.latest $outdir/stk_read_num$ii.saved >> \
       $LOGFILE

  if [ $? -eq 0 ]; then
      echo "Test # $ii OK" | tee -a $LOGFILE
      echo
  else
      tool_status=FAIL
      echo "Test # $ii NOT-OK" | tee -a $LOGFILE
      echo "     saved and latest param outputs do not match for test# $ii"
      echo
  fi
}

###############################################
# write_test_doc                              #
###############################################
write_test_doc()
{
  echo "TEST #$ii:" >> $temp_udf_test
  printf " $summary\n" >> $temp_udf_test
  printf " $command\n" >> $temp_udf_test
  echo >> $temp_udf_test
}

  
##########################
# increase count         #
##########################
increase_count()
{
  ii=`expr $ii + 1`
}


###########################################################################
# generate the trailer for the test document                              #
###########################################################################
write_test_doc_trailer()
{
   printf "~make test~ will run the test script AND CAN BE USED TO REGENERATE THIS FILE.\n" >>\
           $temp_udf_test
}


##########################
# cleanup                #
##########################
cleanup()
{
  # copy the temporary Test document to the more permanent Test document
  # THE FOLLOWING LINE SHOULD BE COMMENTED
  #  cp $temp_udf_test $udf_test

  # remove the temporary Test document
    rm $temp_udf_test

    printf "\nstk_read_num: $tool_status\n" | tee -a $LOGFILE
}



#################
#---------------#
#   MAIN CODE   #
#---------------#
#################

ii=1        # used for test count
tool_status=PASS

# set the date, environment variables, etc.
setup

verify_input_files
verify_reference_files
delete_old_outputs
write_test_doc_header

# run each test

# 1 #
stk_read_num infile="@$indir/stk_read_num.lis1" num=5 verbose=0
summary="This test builds a stack from a single file and retrieves the 5th element."
command="stk_read_num infile=\"@$indir/stk_read_num.lis1\" num=5 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count


# 2 #
stk_read_num infile="stk_read_num.lis1" num=1 verbose=0
summary="This test builds a stack from a single filename without an @ and retrieves\n the 1st element, which is the filename itself."
command="stk_read_num infile=\"stk_read_num.lis1\" num=1 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count


# 3 #
stk_read_num infile="one two three" num=2 verbose=0
summary="This test builds a stack from the infile string to contain three elements.\n  The second element is retrieved."
command="stk_read_num infile=\"one two three\" num=2 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

# 4 #
stk_read_num infile="@$indir/stk_read_num.lis1[time=123:876]" num=4 verbose=0
summary="This test builds a stack from a single file with an extension, and retrieves the 4th element."
command="stk_read_num infile=\"@$indir/stk_read_num.lis1[time=123:876]\" num=4 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

# 5 #
stk_read_num infile="@$indir/stk_read_num.lis1 @$indir/stk_read_num.lis2" num=14 verbose=0
summary="This test builds a stack from two files and retrieves the 14th element"
command="stk_read_num infile=\"@$indir/stk_read_num.lis1 @$indir/stk_read_num.lis2\" num=14 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

# 6 #
stk_read_num infile="one two three" num=-2  verbose=0 2> /dev/null
summary="This test builds a stack from the infile string to contain three elements.\n  The first element should be retrieved."
command="stk_read_num infile=\"one two three\" num=-2 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

# 7 #
stk_read_num infile="one two three" num=4 verbose=0 2> /dev/null
summary="This test builds a stack from the infile string to contain three elements.\n  Since num=4, stk_read_num should not retrieve an element."
command="stk_read_num infile=\"one two three\" num=4 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

# 8 #
stk_read_num infile="one two three" num=0  verbose=0 2> /dev/null
summary="This test builds a stack from the infile string to contain three elements.\n  The first element should be retrieved."
command="stk_read_num infile=\"one two three\" num=0 verbose=0"
echo ${command}
copy_param
compare_outputs
write_test_doc
increase_count

write_test_doc_trailer
cleanup

echo
exit 0
