#! /bin/sh

# 15 Sept. 2016
#
# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.

# ---------------------------------------------------------
# !!1
# stklib.t
# test script for stack library and module(s)
#
# !!2
# syntax:
# stklib.t [<testid> ... ]
# 
# Developer's comments:
#  Since this library is ported outside the CXC system, 
#  the test is expected to be stand-alone as well.
#
#  This means no dependency on the CXC system configuration
#  It does require 2 environment variables to be set:
#    TESTOUT = location to write test output.
#    TESTLOG = location to write test log file.
#
# ---------------------------------------------------------


######################################################################
# subroutine: error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine: keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.
keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine: find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# Initialization
#
# !!3
toolname="stklib"

# set up list of tests
# !!4
alltests="test1"

# "short" test to run
# !!5
shortlist=$alltests

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# check for directory environment variables
if test "x${TESTOUT}" = "x" -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTOUT/TESTLOG not defined" 
fi

# convenience definitions
INDIR="dne"
OUTDIR=$TESTOUT/$toolname
SAVDIR="dne"
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*

# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi

# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.
#
#find_tool <utility>

# announce ourselves
echo ""
echo "${toolname} regression tests" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, 
# but clear them to be safe.
#punlearn <toolname>.par

script_succeeded=0

######################################################################
# Begin per-test loop

# where is this test script?
# Test routine path is fixed/known relative to this base location.
basedir=$(dirname "$0")

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.txt
  savfile=$SAVDIR/${testid}.txt

  echo "" >> $LOGFILE
  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in

      # C Library Test
      test1) command_str=" ${basedir}/stklib/stktest > $outfile 2>${outfile}_err"
	  ;;
  esac

  #  echo $command_str  | tee -a $LOGFILE
  echo $command_str >> $LOGFILE
  eval $command_str  | tee -a $LOGFILE  2>&1

  mismatch=1
  # Check return code..
  if test $? -ne 0; then
     mismatch=0
  fi

  # Check output and baseline file exists
  if  test -f $outfile; then
      x=1
  else
      echo "not found " $outfile  >> $LOGFILE
      mismatch=0
  fi

  # These tests are internally vetted, there are no baselines to 
  # compare against.. We have removed this section of the template
  # from this test script.

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
      echo "${testid} NOT-OK"
      script_succeeded=1
  else
    # No
      echo "${testid} OK"
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
    rm -rf ${OUTDIR}    # clean up output on PASS
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
