#! /bin/sh
#
# 11/2004

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# tg_create_mask
# test script for tg_create_mask


# !!2
# syntax:
# tg_create_mask
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;s/COMMENT/Dataset/g;
  s/CHECKSUM/Dataset/g; s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="tg_create_mask"

# set up list of tests
# !!4
alltests="standard acishetg acishetgCC acisletg hrcletg hrcletg_1 hrcletg_2 hrcletg_2_B hrcletg_2_C hrcletg_2_D hetgCC_2 hetgCC_3 hetgCC_4 hetgCC_4_B hetgCC_4_C hetgCC_4_D acishetg2 acishetg3 acishetg4 acishetg4_B acishetg4_C acishetg4_D acishetg4_E acishetg4_F acishetg4_G"

# "short" test to run
# !!5
shortlist="standard"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# In some views, ASCDS_CALIB isn't defined.  In others, ASCDS_DATA isn't
# defined.  Therefore, we use A_C if it's defined, otherwise we use A_D.
# Hopefully, there is no third case.
if [ -n "$ASCDS_CALIB" ]
then
  psf_dir=$ASCDS_CALIB
else
  psf_dir=$ASCDS_DATA
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # delete old outputs
  rm -f $OUTDIR/*${testid}_reg1a.fits*


  # Set up file names
  fileroot=${toolname}_${testid}
  posTab=${INDIR}/${fileroot}_src1a.fits
  outfile=$OUTDIR/${fileroot}_reg1a.fits
  savfile=$SAVDIR/${fileroot}_reg1a.fits

  radius0=50
  width_hetg=20
  width_letg=40

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    standard )
        infile=${INDIR}/${fileroot}_obs1.par
        psfTab=${INDIR}/${fileroot}_psf.fits
        test_string="tg_create_mask infile=${infile} outfile=${outfile} \
                     input_pos_tab=${posTab} grating_obs=header_value \
                     input_psf_tab=${psfTab}  \
                     detector=header_value  \
                     radius_factor_zero=${radius0}  \
                     width_factor_hetg=${width_hetg}  \
                     width_factor_letg=${width_letg}  \
                     use_user_pars=no clobber=yes"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    acishetg | acishetgCC | acisletg | hrcletg )
            infile=${INDIR}/${fileroot}_evt1.fits
            psfTab=CALDB
        test_string="tg_create_mask infile=${infile} outfile=${outfile} \
                     input_pos_tab=${posTab} grating_obs=header_value \
                     input_psf_tab=${psfTab}  \
                     detector=header_value  \
                     radius_factor_zero=${radius0}  \
                     width_factor_hetg=${width_hetg}  \
                     width_factor_letg=${width_letg}  \
                     use_user_pars=no clobber=yes"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # both radius and width are set.  psffile=doesn't matter.
    hrcletg_1 ) 
        Infile=${INDIR}/tg_create_mask_hrcletg_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=33300.0 sA_zero_y=33300.0 sA_width_leg=1900.0 grating=header_value verb=0 clobber=yes sA_zero_rad=300.0 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # width is set but not radius.  psffile=CALDB.
    hrcletg_2 ) 
        Infile=${INDIR}/tg_create_mask_hrcletg_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=33300.0 sA_zero_y=33300.0 sA_width_leg=1950.0 grating=header_value verb=0 clobber=yes sA_zero_rad= "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is set but not width.  psffile=CALDB.
    hrcletg_2_B ) 
        Infile=${INDIR}/tg_create_mask_hrcletg_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=33300.0 sA_zero_y=33300.0 sA_width_leg= grating=header_value verb=0 clobber=yes sA_zero_rad=325.0 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is set but not width.  psffile=ANY.  no warning
    hrcletg_2_C ) 
        Infile=${INDIR}/tg_create_mask_hrcletg_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY sA_zero_x=33300.0 sA_zero_y=33300.0 sA_width_leg= grating=header_value verb=0 clobber=yes sA_zero_rad=325.0 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # both radius and width are not set.  psffile=ANY.  warning
    hrcletg_2_D ) 
        Infile=${INDIR}/tg_create_mask_hrcletg_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY sA_zero_x=33300.0 sA_zero_y=33300.0 sA_width_leg= grating=header_value verb=0 clobber=yes sA_zero_rad= "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    hetgCC_2 )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=4096 sA_zero_y=4097 sA_width_meg=600 grating=HETG verb=0 clobber=yes sA_zero_rad=26"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    hetgCC_3 )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=4096 sA_zero_y=4097 sA_width_meg=600 grating=HETG verb=0 clobber=yes"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    hetgCC_4 )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=4096 sA_zero_y=4097 grating=HETG verb=0 clobber=yes"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is given by the user but not width; psffile is CALDB. 
    hetgCC_4_B )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB sA_zero_x=4096 sA_zero_y=4097 grating=HETG verb=0 clobber=yes sA_width_heg=  sA_width_meg=  sA_width_leg=   sA_zero_rad=25 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is given by the user but not width; psffile can't be found. no warning.
    hetgCC_4_C )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY sA_zero_x=4096 sA_zero_y=4097 grating=HETG verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg= sA_zero_rad=25 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # both radius and width are not given by the user; psffile can't be found. warning.
    hetgCC_4_D )
        Infile=${INDIR}/hetgCC_2_evt1.fits
        test_string="tg_create_mask use_user_pars=yes infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY sA_zero_x=4096 sA_zero_y=4097 grating=HETG verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg= sA_zero_rad= "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    acishetg2 )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 sA_width_meg=500 sA_width_heg=400 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_zero_rad=25"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    acishetg3 )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 sA_width_meg=500 sA_width_heg=400 last_source_toread=A grating_obs=hetg verb=0 clobber=yes"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    acishetg4 )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is given by the user but not width, and psffile=CALDB.
    acishetg4_B )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg=   sA_zero_rad=50 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius is given by the user but not width, and psffile can't be found. no warning.
    acishetg4_C )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg= sA_zero_rad=50 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # both radius and width are not given by the user and psffile can't be found. warning.
    acishetg4_D )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg= sA_zero_rad= "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius and width-heg are set in par, but not width-meg, and psffile=CALDB.
    acishetg4_E )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg=100  sA_width_meg= sA_width_leg=   sA_zero_rad=50 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius and width-meg are set in par, but not width-heg, and psffile=CALDB.
    acishetg4_F )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=CALDB grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg= sA_width_meg=150 sA_width_leg=   sA_zero_rad=50 "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    # radius, width-meg, width-heg are not set in par and psffile can't be found. issue a warning.
    acishetg4_G )
        Infile=${INDIR}/tg_create_mask_acishetg_evt1.fits
        test_string="tg_create_mask infile=${Infile} outfile=${outfile} input_pos_tab=NONE input_psf_tab=ANY grating_obs=HETG use_user_pars=yes sA_zero_x=4096 sA_zero_y=4097 last_source_toread=A grating_obs=hetg verb=0 clobber=yes sA_width_heg= sA_width_meg= sA_width_leg=   sA_zero_rad= "
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;
    * ) echo "Test  ${testid} not implemented." | tee -a $LOGFILE
        echo "" | tee -a  $LOGFILE
        script_succeeded=1     # mismatch=0 
        continue               # go to the next testid
        ;;
  esac


  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0           
  fi

  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  ### compare output files
  dmdiff $outfile $savfile keys=yes tol=$SAVDIR/tolerance comment=no >>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: -- $toolname mismatch in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
