#! /bin/sh
#
# 11/2004

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# tgdetect.t
# test script for tgdetect


# !!2
# syntax:
# tgdetect.t [<testid> ... ]
 



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;s/COMMENT/Dataset/g;
  s/CHECKSUM/Dataset/g; s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}


######################################################################
# Initialization

# !!3
toolname="tgdetect"

# set up list of tests
# !!4
alltests="acishetg acishetg_b acisletg hrcletg standard use_psfmap create_psfmap"

# "short" test to run
# !!5
shortlist="acishetg acishetg_b acisletg hrcletg standard"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi

# In some views, ASCDS_CALIB isn't defined.  In others, ASCDS_DATA isn't
# defined.  Therefore, we use A_C if it's defined, otherwise we use A_D.
# Hopefully, there is no third case.
if [ -n "$ASCDS_CALIB" ]
then
  psf_dir=$ASCDS_CALIB
else
  psf_dir=$ASCDS_DATA
fi

# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

#alltests="acishetg acisletg hrcletg standard use_psfmap"
for testid in $testlist
do
  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # delete old outputs
  rm -f $OUTDIR/${toolname}_${testid}*

  # Set up file names
  fileroot=${toolname}_${testid}
  psffile=${INDIR}/acisf03226_000N000_b_full_image_psfmap3.fits
  infile=${INDIR}/${fileroot}_evt1.fits
  outfile=$OUTDIR/${fileroot}_src1a.fits
  savfile=$SAVDIR/${fileroot}_src1a.fits

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6

    acishetg | acisletg | hrcletg | standard | acishetgCC ) 

        test_string="tgdetect infile=${infile} outfile=${outfile} \
          OBI_srclist_file=NONE temproot=${fileroot} keeptemp=no \
          keepexit=no zo_pos_x=default zo_pos_y=default \
          zo_sz_filt_x=default zo_sz_filt_y=default snr_thresh=40 \
          expstk=none thresh=10 ellsigma=3 expratio=0 findpeaks=yes \
          celldetect_log=no \
          fixedcell=15 fixedcell_cc_mode=99 bkgfile=none bkgvalue=0 \
          bkgerrvalue=0 eband=1.4967 eenergy=0.9 snrfile=none convolve=no \
          cellfile=none centroid=yes \
          snr_ratio_limit=0 setsrcid=yes max_separation=1 \
          clobber=yes snr_diminution=1.0 verbose=5 \
          xoffset=INDEF yoffset=INDEF" 

        punlearn tgdetect
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    acishetg_b ) 
        test_string="tgdetect infile=${infile} outfile=${outfile} \
          OBI_srclist_file=NONE temproot=${fileroot} keeptemp=no \
          keepexit=no zo_pos_x=default zo_pos_y=default \
          zo_sz_filt_x=default zo_sz_filt_y=default snr_thresh=40 \
          expstk=none thresh=10 ellsigma=3 expratio=0 findpeaks=yes \
          celldetect_log=no \
          fixedcell=15 fixedcell_cc_mode=99 bkgfile=none bkgvalue=0 \
          bkgerrvalue=0 eband=1.4967 eenergy=0.9 snrfile=none convolve=no \
          cellfile=none centroid=yes \
          snr_ratio_limit=0 setsrcid=yes max_separation=1 \
          clobber=yes snr_diminution=1.0 verbose=5 \
          xoffset=1 yoffset=1" 

        punlearn tgdetect
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    use_psfmap ) 

        test_string="tgdetect infile=${infile} outfile=${outfile} \
          OBI_srclist_file=NONE temproot=${fileroot} keeptemp=no \
          psffile=${psffile}
          keepexit=no zo_pos_x=default zo_pos_y=default \
          zo_sz_filt_x=default zo_sz_filt_y=default snr_thresh=4 \
          expstk=none thresh=4 ellsigma=3 expratio=0 findpeaks=yes \
          celldetect_log=no bkgfile=none bkgvalue=0 \
          bkgerrvalue=0 eband=1.4967 eenergy=0.98 snrfile=none convolve=no \
          cellfile=none centroid=yes fixedcell=0 \
          snr_ratio_limit=0 setsrcid=yes max_separation=1 \
          clobber=yes verbose=5 snr_diminution=1.0" 

        punlearn tgdetect
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    # The snr_diminution has no effect here, just proof that the optional parameter caries through.
    create_psfmap ) 

        test_string="tgdetect infile=${infile} outfile=${outfile} \
          OBI_srclist_file=NONE temproot=${fileroot} keeptemp=no \
          keepexit=no zo_pos_x=default zo_pos_y=default \
          zo_sz_filt_x=default zo_sz_filt_y=default snr_thresh=4 \
          expstk=none thresh=4 ellsigma=3 expratio=0 findpeaks=yes \
          celldetect_log=no bkgfile=none bkgvalue=0 \
          bkgerrvalue=0 eband=1.4967 eenergy=0.9 snrfile=none convolve=no \
          cellfile=none centroid=yes fixedcell=0 \
          snr_ratio_limit=0 setsrcid=yes max_separation=1 \
          clobber=yes verbose=5 snr_diminution=0.9" 

        punlearn tgdetect
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
        ;;

    * ) echo "Test  ${testid} not implemented." | tee -a $LOGFILE
        echo "" | tee -a  $LOGFILE
        script_succeeded=1     # mismatch=0 
        continue               # go to the next testid
        ;;
  esac


  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0           
  fi

  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  ### compare output files
  echo "dmdiff $outfile $savfile keys=yes tol=$SAVDIR/tolerance"
  dmdiff $outfile $savfile keys=yes tol=$SAVDIR/tolerance >>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: -- tgdetect mismatch in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
