#! /bin/sh
#
# 7/2006
# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# tgextract2.t
# test script for tgextract2


# !!2
# syntax:
# tgextract2.t [<testid> ... ]
######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo ""   | tee -a $LOGFILE
  echo "$1" | tee -a $LOGFILE
  echo ""   | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/ROLL_RMS/Dataset/g;s/COMMENT/Dataset/g;
  s/CHECKSUM/Dataset/g; s/DATE/Dataset/g;s/CREATOR/Dataset/g;
  s/HISTORY/Dataset/g; s/DATASUM/Dataset/g;s/ASCDSVER/Dataset/g;
  s/HISTNUM/Dataset/g' | \
  grep -v Dataset > $2
  zerotest $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
    :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# subroutine
# zerotest <file> 
# Makes sure that file is not 0 length.
# Use this to protect yourself against empty files  (which will 
# 'diff' without error).  This can happen when the input file to
# cat $infile | do_something >> $outfile
# is missing.  This is used by keyfilter(), above.

zerotest()
{
 if test -s $1 ;
 then
   :
 else
   echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
   mismatch=0
 fi
}

######################################################################
# Initialization

# !!3
toolname="tgextract2"

# set up list of tests
# !!4
alltests="standard acishetg acishetg_2 acishetg_2_a acisletg acisletg_a acisletg_3 hrcletg hrcletg_2 hrcletg_2_a hrcletg_3 acishetgCC acishetgCC_a hrcNan"

# "short" test to run
# !!5
shortlist="standard acishetg acisletg hrcletg acishetgCC"

# compute date string for log file
DT=`date +'%d%b%Y_%T'`

# convenience definitions
INDIR=$TESTIN/$toolname
OUTDIR=$TESTOUT/$toolname
LOGDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*


# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# Make sure we have an input directory
if test -d $INDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find input directory : $INDIR"
  fi
fi

# Make sure we have a save directory
if test -d $SAVDIR ; then
:
else
  if test $? -ne 0 ; then
    error_exit "can't find save directory : $SAVDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

find_tool dmdiff
find_tool dmimgcalc



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo "" | tee -a $LOGFILE

# All parameters except verbose should be set anyway, but clear them
# to be safe.
bose=`pget $toolname verbose`
punlearn $toolname
pset $toolname verbose=$bose

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
  ####################################################################
  # Init per-test error flag
  #
  mismatch=1    

  # Set up file names 
  infile=${INDIR}/${testid}_evt2.fits
  outfile=$OUTDIR/${testid}_pha2.fits
  savfile=$SAVDIR/${testid}_pha2.fits

  # delete old outputs
  rm -f ${outfile}

  Part=header_value
  Order=default
  eFilter=[status=0]

  wav_grid=use_header
  wav_grid_heg=0.333333:7.16:0.000833333
  wav_grid_meg=1:100:2    #? 
  wav_grid_leg=1:205.8:0.0125
  wav_grid_leg_acis=1:205.8:0.0125

  echo "running $testid" | tee -a $LOGFILE
  echo "" | tee -a $LOGFILE
  ####################################################################
  # run the tool
  case ${testid} in
    # !!6
    standard ) Part=HEG
               Order=1
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=none wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	#  eval $test_string >> ${LOGFILE} 2>> ${LOGFILE}
	eval $test_string 
        ;;
    acishetg ) test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=none wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string 
        ;;
    # same as acishetg except region_file=CALDB 
    # acishetg_2 : calib file exists in test_2 (only!)
    acishetg_2 ) infile=${INDIR}/acishetg_evt2.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=CALDB wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string 
        ;;
    # same as acishetg_2 except region_file=hetgD1999-07-22disp_regN0001.fits
    acishetg_2_a ) infile=${INDIR}/acishetg_evt2.fits
                   regFile=${INDIR}/hetgD1999-07-22disp_regN0001.fits 
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=${regFile} wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE  
        echo "" | tee -a  $LOGFILE
        eval $test_string 
        ;;
# set region_file=CALDB (=NONE)
# acisletg : calib file exists in test_2 (only!)
    acisletg | hrcletg | acishetgCC )
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=CALDB wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string 
        ;;
# same as acisletg except regFile=letgD1999-07-22disp_regN0002.fits
    acisletg_a ) infile=${INDIR}/acisletg_evt2.fits
                 regFile=${INDIR}/letgD1999-07-22disp_regN0002.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=${regFile} wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
# same as acishetgCC except error=gaussian 
    acishetgCC_a )
        infile=${INDIR}/acishetgCC_evt2.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=none wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gaussian geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
# same as acisletg except infile with time filters
    acisletg_3 ) infile=${INDIR}/acisletg_evt2.fits"[time=63833865.609:63838239.158]"
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=none wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
# same as hrcletg except region_file 
    hrcletg_2 ) infile=${INDIR}/hrcletg_evt2.fits 
                regFile=${INDIR}/letgD1999-07-22regN0002.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=${regFile} wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE 
        echo "" | tee -a  $LOGFILE
	eval $test_string 
        ;;
# same as hrcletg_2 except error=gaussian; 
    hrcletg_2_a ) infile=${INDIR}/hrcletg_evt2.fits 
                  regFile=${INDIR}/letgD1999-07-22regN0002.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=${regFile} wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gaussian geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
# same as hrcletg_2 except infile with time filter; output same as hrcletg_2 except time keys;
    hrcletg_3 ) infile=${INDIR}/hrcletg_evt2.fits"[time=53320000.0:53323500.0]" 
                regFile=${INDIR}/letgD1999-07-22regN0002.fits
        test_string="${toolname}  infile=${infile} outfile=${outfile} tg_srcid_list=all tg_part_list=${Part} tg_order_list=${Order} opt=pha2 region_file=${regFile} wav_grid=use_header wav_grid_leg_acis=${wav_grid_leg_acis} wav_grid_leg=${wav_grid_leg} wav_grid_meg=${wav_grid_meg} wav_grid_heg=${wav_grid_heg} extract_background=yes error=gehrels geompar=geom ignore_source_id=yes clobber=yes evt_filter=${eFilter} backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
# bug13515: allow nan in the region file; 
# tg_srcid_list=1 tg_part_list=header_value tg_order_list=default
    hrcNan ) infile=${INDIR}/hrcf00029N006_evt2.fits
             regFile=${INDIR}/hrc29_nan_region.fits
             punlearn tgextract2
        test_string="${toolname}  infile=${infile} outfile=${outfile} region_file=${regFile} clobber=yes mode=h backscale_method=events"
        echo $test_string | tee -a  $LOGFILE
        echo "" | tee -a  $LOGFILE
        eval $test_string
        ;;
    * ) echo "Test  ${testid} not implemented." | tee -a $LOGFILE
        echo "" | tee -a  $LOGFILE
        script_succeeded=1     # mismatch=0 
        continue               # go to the next testid
        ;;
  esac


  ####################################################################
  # if the tool failed to run, set mismatch to 0.
  #
  if test $? -ne 0; then
     echo "" | tee -a $LOGFILE
     echo "$toolname failed to run" | tee -a $LOGFILE
     echo "" | tee -a $LOGFILE
     mismatch=0           
  fi

  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  ####################################################################
  # FITS table    (duplicate for as many tables per test as needed)

  ### compare output files
  dmdiff $outfile $savfile keys=yes tol=$SAVDIR/tolerance >>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: -- ${toolname}  mismatch in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ### compare - REGION 
  dmdiff ${outfile}"[REGION]" ${savfile}"[REGION]" keys=yes tol=$SAVDIR/tolerance >>$LOGFILE
  if  test $? -ne 0 ; then
    echo "ERROR: -- ${toolname}  REGION mismatch in $outfile" >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/${testid}.txt $OUTDIR/${testid}.txt_std > \
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK"
    script_succeeded=1
  else
    # No
    echo "${testid} OK"
  fi

  echo "" | tee -a $LOGFILE

done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo "" | tee -a $LOGFILE

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo ""
echo "log file in ${LOGFILE}"
echo ""

exit $script_succeeded
