#! /bin/sh

# wavdetect.t
# test script for wavdetect

# syntax:
# wavdetect.t [<testid> ... ]


######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# keyfilter infile outfile
# filters out CHECKSUM, Dataset, CREATOR, HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

keyfilter()
{
  cat $1 | sed -e 's/CHECKSUM/Dataset/g;s/COMMENT/Dataset/g;
  s/DATE/Dataset/g;s/CREATOR/Dataset/g;s/HISTORY/Dataset/g;
  s/DATASUM/Dataset/g;s/HISTNUM/Dataset/g;s/ASCDSVER/Dataset/g' | \
  grep -v Dataset > $2
}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable

find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
   :
  else
    error_exit "tool $1 not found"
  fi
}

######################################################################
# Initialization

toolname="wavdetect"

# set up list of tests
# tests 7 through 12 test that the WCS information is transferred from image to table.  SAT-145.
alltests="test1 test2 test2a test3 test4 test5 test6 test7 test8 test9 test10 test11 test12 test_rmdup no_hist"

# "short" test to run
shortlist="test1"


# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

mysys=`uname -m`
if test "$mysys" = "x86_64" ; then
 SAVDIR="$TESTSAV/$toolname"
fi

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi


# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.



# announce ourselves
echo ""
echo "${toolname} regression" | tee $LOGFILE
echo ""

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.fits
  savfile=${SAVDIR}/${testid}.fits
  # ignore 64bit specific savdir for stderr/stdout save file
  savout=$TESTSAV/${toolname}/${testid}_output.txt

  echo ""
  echo "running $testid" | tee -a $LOGFILE
  echo ""
  ####################################################################
  # run the tool
  case ${testid} in
    test1 ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_img.fits outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile=$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2 4 8' psffile=$INDIR/acis_650_ccd6_psf.fits expfile=$INDIR/acis_650_ccd6_emap.fits mode=h clob+ sigthresh=1.0e-07"
            ;;

    test2 ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_img.fits outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile=$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2 4 8' psffile=$INDIR/acis_650_ccd6_psf.fits expfile=$INDIR/acis_650_ccd6_emap.fits bkginput=$INDIR/acis_650_ccd6_bkg.fits mode=h clob+ sigthresh=1.0e-07"
            ;;

    # check for no WCS info on input
    test2a ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_no_wcs_img.fits outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile=
$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2 4 8' mode=h clob+ sigthresh=1.0e-07 psffile=none"
            ;;

    # check error for blank scellfile 
    test3 ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_img.fits outfile=$outfile scell='' imagefile=$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2 4 8' psffile=$INDIR/acis_650_ccd6_psf.fits expfile=$INDIR/acis_650_ccd6_emap.fits bkginput=$INDIR/acis_650_ccd6_bkg.fits mode=h clob+ sigthresh=1.0e-07"
            ;;

    # check error for blank imagefile 
    test4 ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_img.fits outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile='' defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2 4 8' psffile=$INDIR/acis_650_ccd6_psf.fits expfile=$INDIR/acis_650_ccd6_emap.fits bkginput=$INDIR/acis_650_ccd6_bkg.fits mode=h clob+ sigthresh=1.0e-07"
            ;;
    # check error for blank background 
    test5 ) test_string="wavdetect infile=$INDIR/acis_650_ccd6_img.fits outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile=$OUTDIR/${testid}_recon.fits defnbkg='' scales='2 4 8' psffile=$INDIR/acis_650_ccd6_psf.fits expfile=$INDIR/acis_650_ccd6_emap.fits bkginput='' mode=h clob+ sigthresh=1.0e-07"
            ;;
    # Testing that full WCS info is transferred to output files. 
    test6 ) test_string="wavdetect infile=$INDIR/ifake.img outfile=$outfile scell=$OUTDIR/${testid}_scell.fits imagefile=$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits scales='2' psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test7 ) test_string="wavdetect infile=$INDIR/faked_sin.fits outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test8 ) test_string="wavdetect infile=$INDIR/wav_rot.fits outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test9 ) test_string="wavdetect infile=$INDIR/wav_nowcs.fits outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test10 ) test_string="wavdetect infile=$INDIR/chip.img outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test11 ) test_string="wavdetect infile=$INDIR/chip.mod.img outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;
    test12 ) test_string="wavdetect infile=$INDIR/separate_linear_wcs.img outfile=$outfile scell=$OUTDIR/wav.cell imagefile=$OUTDIR/wav.img defnbkg=$OUTDIR/wav.bkg scales=4 psffile="" expfile="" bkginput='' mode=h clob+"
            ;;

    test_rmdup) test_string="wavdetect infile=$INDIR/acisf02233_broad_thresh.img'[x=4400:5200,y=4000:4600]' \
        expfile=$INDIR/acisf02233_broad_thresh.expmap'[x=4400:5200,y=4000:4600]' \
        psffile=$INDIR/acisf02233_broad.psfmap'[x=4400:5200,y=4000:4600]' \
        scales='1.4 2 4 8 12 16' \
        outfile=$outfile \
        scell=$OUTDIR/${testid}_scell.fits imagefile=$OUTDIR/${testid}_recon.fits defnbkg=$OUTDIR/${testid}_nbkg.fits \
        clobber=yes mode=h "
            ;;

    ##### sat-195 : don't add history to the autoname files 
    no_hist ) test_string="wavdetect infile=$INDIR/no_hist_img.fits outfile=$OUTDIR/. scellfile=$OUTDIR/. defnbkgfile=$OUTDIR/. imagefile=$OUTDIR/. mode=h clobber=yes"
              outfile=$OUTDIR/${testid}_image.fits
              savfile=${SAVDIR}/${testid}_image.fits
              # ?? savout=$TESTSAV/${toolname}/${testid}_output.txt
            ;;

  esac


  echo $test_string | tee -a $LOGFILE 
  eval $test_string | tee -a $OUTDIR/${testid}_output.txt

  ####################################################################
  # check the outputs

  # Init per-test error flag
  mismatch=1

  case ${testid} in
      test3 | test4 | test5 | test6 | test7 | test8 | test9 | test10 | test11 | test12)
      ;; 

      *) 

         ####################################################################
         # FITS table    (duplicate for as many tables per test as needed)

         dmdiff $outfile $savfile tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
         if  test $? -ne 0 ; then
           printf "\n dmdiff $outfile $savfile tol=$SAVDIR/tolerance\n" | tee -a $LOGFILE
           echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
           mismatch=0
         fi


         ####################################################################
         # FITS image  (duplicate for as many images per test as needed)

         dmdiff  $OUTDIR/${testid}_scell.fits  $SAVDIR/${testid}_scell.fits \
           tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
         if  test $? -ne 0 ; then
          printf "\n dmdiff $OUTDIR/${testid}_scell.fits $SAVDIR/${testid}_scell.fits tol=$SAVDIR/tolerance\n" | tee -a $LOGFILE
          echo "ERROR: mismatch in ${testid}_scell.fits" >> $LOGFILE
           mismatch=0
         fi

         if test ${testid} = "no_hist" ; then
            reconF=${testid}_wav.fits  # autoname
         else
            reconF=${testid}_recon.fits
         fi
         dmdiff  $OUTDIR/${reconF} $SAVDIR/${reconF} \
           tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
         if  test $? -ne 0 ; then
          printf "\n dmdiff $OUTDIR/${reconF} $SAVDIR/${reconF} tol=$SAVDIR/tolerance \n" |tee -a $LOGFILE
          echo "ERROR: mismatch in ${testid}_recon.fits" >> $LOGFILE
          mismatch=0
         fi

       #  dmdiff  $OUTDIR/${testid}_nbkg.fits  $SAVDIR/${testid}_nbkg.fits \
       #    tol=$SAVDIR/tolerance > /dev/null 2>>$LOGFILE
       #  if  test $? -ne 0 ; then
       #    echo "ERROR: mismatch in ${testid}_nbkg.fits" >> $LOGFILE
       #    mismatch=0
       #  fi

  esac

  ####################################################################
  # check output text file for ERROR messages
  diff $OUTDIR/${testid}_output.txt $savout > /dev/null 2>>$LOGFILE
  if  test $? -ne 0 ; then
    printf "\n diff $OUTDIR/${testid}_output.txt $savout \n" |tee -a $LOGFILE
    echo "ERROR: MISMATCH in $OUTDIR/${testid}_output.txt " >> $LOGFILE
    mismatch=0
  fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
    echo "${testid} NOT-OK" |tee -a $LOGFILE
    script_succeeded=1
  else
    # No
    echo "${testid} OK" |tee -a $LOGFILE
  fi

done
# end per-test loop
######################################################################


######################################################################
# report results

echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"


exit $script_succeeded
